@@ -68,7 +68,7 @@
 #include "auth-pam-common.h"
 #include "sshpacketstream.h"
 #include "sshserver.h"
-#include "ssholdfsm.h"
+#include "sshfsm.h"
 #include "sshencode.h"
 #include "auths-common.h"
 #include "sshunixpipestream.h"
@@ -103,7 +103,7 @@ typedef struct SshServerPAMAuthRec
   SshServer server;
   SshUser uc;
   PAMAuthSentState sent_state;
-  /* Packetstream to ssh-pam-client. */
+  /* Packetstream to PAM subprocess. */
   SshPacketWrapper wrapper;
 
   /* Received packet from client. */
@@ -120,11 +120,25 @@ typedef struct SshServerPAMAuthRec
 
   void **state_placeholder;
 
-  SshOldFSM fsm;
-  SshOldFSMThread main_thread;
-  SshOldFSMThread aux_thread;
+  SshFSM fsm;
+  SshFSMThread main_thread;
+  SshFSMThread aux_thread;
 } *SshServerPAMAuth, SshServerPAMAuthStruct;
 
+SSH_FSM_STEP(ssh_pams_start);
+SSH_FSM_STEP(ssh_pams_set_item_rhost);
+SSH_FSM_STEP(ssh_pams_set_item_tty);
+SSH_FSM_STEP(ssh_pams_set_item_ruser);
+SSH_FSM_STEP(ssh_pams_authenticate);
+SSH_FSM_STEP(ssh_pams_acct_mgmt);
+SSH_FSM_STEP(ssh_pams_chauthtok);
+SSH_FSM_STEP(ssh_pams_open_session);
+SSH_FSM_STEP(ssh_pams_setcred);
+SSH_FSM_STEP(ssh_pams_auth_successful);
+SSH_FSM_STEP(ssh_pams_process_client_packet);
+SSH_FSM_STEP(ssh_pams_suspend);
+SSH_FSM_STEP(ssh_pams_finish);
+
 void received_packet_cb(SshPacketType packet_type,
                         const unsigned char *data, size_t len,
                         void *context)
@@ -149,7 +163,7 @@ void received_packet_cb(SshPacketType packet_type,
           /* Protocol error. */
           ssh_log_event(gdata->server->config->log_facility,
                         SSH_LOG_ERROR,
-                        "auths-pam: ssh-pam-client sent a malformed "
+                        "auths-pam: PAM subprocess sent a malformed "
                         "packet. (packet type: %d)", packet_type);
 
           *gdata->state_placeholder = NULL;
@@ -158,8 +172,8 @@ void received_packet_cb(SshPacketType packet_type,
             (SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED,
              gdata->client_packet,
              gdata->completion_context);
-          ssh_oldfsm_set_next(gdata->main_thread, "ssh_pam_abort");
-          ssh_oldfsm_continue(gdata->main_thread);
+          ssh_fsm_set_next(gdata->main_thread, ssh_pams_finish);
+          ssh_fsm_continue(gdata->main_thread);
           return;
         }
     }
@@ -181,7 +195,7 @@ void received_packet_cb(SshPacketType packet_type,
         {
           ssh_log_event(gdata->server->config->log_facility,
                         SSH_LOG_ERROR,
-                        "auths-pam: ssh-pam-client returned packet "
+                        "auths-pam: PAM subprocess returned packet "
                         "SSH_PAM_OP_SUCCESS_WITH_PAYLOAD, expecting "
                         "SSH_PAM_OP_SUCCESS.");
 
@@ -190,21 +204,24 @@ void received_packet_cb(SshPacketType packet_type,
             (SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED,
              gdata->client_packet,
              gdata->completion_context);
-          ssh_oldfsm_set_next(gdata->main_thread, "ssh_pam_abort");
+          ssh_fsm_set_next(gdata->main_thread, ssh_pams_finish);
         }
       break;
     case SSH_PAM_OP_ERROR:
       /* Check if operation can fail, and what will we do about
          it. Otherwise, fall through to generic error.*/
+      SSH_DEBUG(2, ("PAM_OP_ERROR (err_num %d)", err_num));
       switch (gdata->sent_state)
         {
-        case SSH_PAM_ACCT_MGMT:
+        case SENT_PAM_ACCT_MGMT:
           if (err_num == PAM_NEW_AUTHTOK_REQD)
             {
-              ssh_oldfsm_set_next(gdata->main_thread, "ssh_pam_chauthtok");
+              SSH_DEBUG(2, ("New authentication token required."));
+              ssh_fsm_set_next(gdata->main_thread, ssh_pams_chauthtok);
               /* XX clean up. */
               ssh_xfree(err_msg);
-              ssh_oldfsm_continue(gdata->main_thread);
+              ssh_fsm_continue(gdata->main_thread);
+              return;
             }
           break;
         default:
@@ -216,9 +233,10 @@ void received_packet_cb(SshPacketType packet_type,
       SSH_ASSERT(err_msg);
       ssh_log_event(gdata->server->config->log_facility,
                     SSH_LOG_ERROR,
-                    "auths-pam: ssh-pam-client returned packet "
-                    "SSH_PAM_OP_ERROR. (err_num: %d, err_msg: %s)",
-                    err_num, err_msg);
+                    "auths-pam: PAM subprocess returned packet "
+                    "%s. (err_num: %d, err_msg: %s)",
+                    packet_type == SSH_PAM_OP_ERROR ? "SSH_PAM_OP_ERROR" :
+                    "SSH_PAM_ERROR", err_num, err_msg);
       ssh_xfree(err_msg);
 
       *gdata->state_placeholder = NULL;
@@ -227,7 +245,7 @@ void received_packet_cb(SshPacketType packet_type,
         (SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED,
          gdata->client_packet,
          gdata->completion_context);
-      ssh_oldfsm_set_next(gdata->main_thread, "ssh_pam_abort");
+      ssh_fsm_set_next(gdata->main_thread, ssh_pams_finish);
 
       break;
     case SSH_PAM_CONVERSATION_MSG:
@@ -245,17 +263,17 @@ void received_packet_cb(SshPacketType packet_type,
                                 gdata->completion_context);
       gdata->client_packet = NULL;
 
-      /* Don't revive thread; we will come here again (after the
-         client sends us back a response, the aux_thread will send it
-         to ssh-pam-client, and the ssh-pam-client will send us the
-         acknowledgement for the original message). */
+      /* Don't revive thread; we will come here again (after the client
+         sends us back a response, the aux_thread will send it to PAM
+         subprocess, and it will send us the acknowledgement for the
+         original message). */
       return;
 
     default:
       ssh_log_event(gdata->server->config->log_facility,
                     SSH_LOG_ERROR,
-                    "auths-pam: ssh-pam-client returned packet "
-                    "%d, which is invalid from the ssh-pam-client.",
+                    "auths-pam: PAM subprocess returned packet "
+                    "%d, which is invalid.",
                     packet_type);
 
       *gdata->state_placeholder = NULL;
@@ -263,10 +281,10 @@ void received_packet_cb(SshPacketType packet_type,
         (SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED,
          gdata->client_packet,
          gdata->completion_context);
-      ssh_oldfsm_set_next(gdata->main_thread, "ssh_pam_abort");
+      ssh_fsm_set_next(gdata->main_thread, ssh_pams_finish);
       break;
     }
-  ssh_oldfsm_continue(gdata->main_thread);
+  ssh_fsm_continue(gdata->main_thread);
 }
 
 void received_eof_cb(void *context)
@@ -280,19 +298,19 @@ void received_eof_cb(void *context)
                              gdata->client_packet,
                              gdata->completion_context);
 
-  ssh_oldfsm_set_next(gdata->main_thread, "ssh_pam_abort");
-  ssh_oldfsm_continue(gdata->main_thread);
+  ssh_fsm_set_next(gdata->main_thread, ssh_pams_finish);
+  ssh_fsm_continue(gdata->main_thread);
 }
 
-SSH_OLDFSM_STEP(ssh_pam_start)
+SSH_FSM_STEP(ssh_pams_start)
 {
-  SSH_OLDFSM_GDATA(SshServerPAMAuth);
-
-  SSH_OLDFSM_SET_NEXT("ssh_pam_set_item_rhost");
+  SshServerPAMAuth gdata = (SshServerPAMAuth)fsm_context;
+  
+  SSH_FSM_SET_NEXT(ssh_pams_set_item_rhost);
 
   gdata->sent_state = SENT_PAM_START;
 
-  SSH_TRACE(4,("Sending service_name:%s, user:%s to ssh-pam-client.",
+  SSH_TRACE(4,("Sending service_name:%s, user:%s to PAM subprocess.",
                gdata->service_name, ssh_user_name(gdata->uc)));
 
   ssh_packet_wrapper_send_encode(gdata->wrapper,
@@ -303,14 +321,14 @@ SSH_OLDFSM_STEP(ssh_pam_start)
                                  ssh_user_name(gdata->uc),
                                  strlen(ssh_user_name(gdata->uc)),
                                  SSH_FORMAT_END);
-  return SSH_OLDFSM_SUSPENDED;
+  return SSH_FSM_SUSPENDED;
 }
 
-SSH_OLDFSM_STEP(ssh_pam_set_item_rhost)
+SSH_FSM_STEP(ssh_pams_set_item_rhost)
 {
   char *client_host;
-  SSH_OLDFSM_GDATA(SshServerPAMAuth);
-  SSH_OLDFSM_SET_NEXT("ssh_pam_set_item_tty");
+  SshServerPAMAuth gdata = (SshServerPAMAuth)fsm_context;
+  SSH_FSM_SET_NEXT(ssh_pams_set_item_tty);
 
   if (gdata->server->authenticated_client_host)
     /* If set, use authenticated remote host name from "hostbased". */
@@ -323,7 +341,7 @@ SSH_OLDFSM_STEP(ssh_pam_set_item_rhost)
   gdata->sent_state = SENT_PAM_SET_RHOST;
 
   SSH_DEBUG(4, ("Sending a SSH_PAM_SET_ITEM (PAM_RHOST) packet to "
-                "ssh-pam-client..."));
+                "PAM subprocess..."));
 
   ssh_packet_wrapper_send_encode(gdata->wrapper,
                                  SSH_PAM_SET_ITEM,
@@ -332,18 +350,18 @@ SSH_OLDFSM_STEP(ssh_pam_set_item_rhost)
                                  client_host,
                                  strlen(client_host),
                                  SSH_FORMAT_END);
-  return SSH_OLDFSM_SUSPENDED;
+  return SSH_FSM_SUSPENDED;
 }
 
-SSH_OLDFSM_STEP(ssh_pam_set_item_tty)
+SSH_FSM_STEP(ssh_pams_set_item_tty)
 {
-  SSH_OLDFSM_GDATA(SshServerPAMAuth);
-  SSH_OLDFSM_SET_NEXT("ssh_pam_set_item_ruser");
+  SshServerPAMAuth gdata = (SshServerPAMAuth)fsm_context;
+  SSH_FSM_SET_NEXT(ssh_pams_set_item_ruser);
 
   gdata->sent_state = SENT_PAM_SET_TTY;
 
-  SSH_DEBUG(4, ("Sending a SSH_PAM_SET_ITEM (PAM_RUSER) packet to "
-                "ssh-pam-client..."));
+  SSH_DEBUG(4, ("Sending a SSH_PAM_SET_ITEM (PAM_TTY) packet to "
+                "PAM subprocess..."));
 
   ssh_packet_wrapper_send_encode(gdata->wrapper,
                                  SSH_PAM_SET_ITEM,
@@ -351,23 +369,23 @@ SSH_OLDFSM_STEP(ssh_pam_set_item_tty)
                                  SSH_FORMAT_UINT32_STR,
                                  "ssh", strlen("ssh"),
                                  SSH_FORMAT_END);
-  return SSH_OLDFSM_SUSPENDED;
+  return SSH_FSM_SUSPENDED;
 }
 
-SSH_OLDFSM_STEP(ssh_pam_set_item_ruser)
+SSH_FSM_STEP(ssh_pams_set_item_ruser)
 {
-  SSH_OLDFSM_GDATA(SshServerPAMAuth);
-  SSH_OLDFSM_SET_NEXT("ssh_pam_authenticate");
+  SshServerPAMAuth gdata = (SshServerPAMAuth)fsm_context;
+  SSH_FSM_SET_NEXT(ssh_pams_authenticate);
 
   if (!gdata->server->authenticated_client_user_name)
     {
-      return SSH_OLDFSM_CONTINUE;
+      return SSH_FSM_CONTINUE;
     }
 
   gdata->sent_state = SENT_PAM_SET_RUSER;
 
   SSH_DEBUG(4, ("Sending a SSH_PAM_SET_ITEM (PAM_RUSER) packet to "
-                "ssh-pam-client..."));
+                "PAM subprocess..."));
 
   ssh_packet_wrapper_send_encode(gdata->wrapper,
                                  SSH_PAM_SET_ITEM,
@@ -377,17 +395,17 @@ SSH_OLDFSM_STEP(ssh_pam_set_item_ruser)
                                  strlen(gdata->server->
                                         authenticated_client_user_name),
                                  SSH_FORMAT_END);
-  return SSH_OLDFSM_SUSPENDED;
+  return SSH_FSM_SUSPENDED;
 }
 
-SSH_OLDFSM_STEP(ssh_pam_authenticate)
+SSH_FSM_STEP(ssh_pams_authenticate)
 {
-  SSH_OLDFSM_GDATA(SshServerPAMAuth);
-  SSH_OLDFSM_SET_NEXT("ssh_pam_acct_mgmt");
+  SshServerPAMAuth gdata = (SshServerPAMAuth)fsm_context;
+  SSH_FSM_SET_NEXT(ssh_pams_acct_mgmt);
 
   gdata->sent_state = SENT_PAM_AUTHENTICATE;
 
-  SSH_DEBUG(4, ("Sending a SSH_PAM_AUTHENTICATE packet to ssh-pam-client..."));
+  SSH_DEBUG(4, ("Sending a SSH_PAM_AUTHENTICATE packet to PAM subprocess..."));
 
   ssh_packet_wrapper_send_encode(gdata->wrapper,
                                  SSH_PAM_AUTHENTICATE,
@@ -395,17 +413,17 @@ SSH_OLDFSM_STEP(ssh_pam_authenticate)
                                  SSH_FORMAT_UINT32,
                                  (SshUInt32) PAM_DISALLOW_NULL_AUTHTOK,
                                  SSH_FORMAT_END);
-  return SSH_OLDFSM_SUSPENDED;
+  return SSH_FSM_SUSPENDED;
 }
 
-SSH_OLDFSM_STEP(ssh_pam_acct_mgmt)
+SSH_FSM_STEP(ssh_pams_acct_mgmt)
 {
-  SSH_OLDFSM_GDATA(SshServerPAMAuth);
-  SSH_OLDFSM_SET_NEXT("ssh_pam_open_session");
+  SshServerPAMAuth gdata = (SshServerPAMAuth)fsm_context;
+  SSH_FSM_SET_NEXT(ssh_pams_open_session);
 
   gdata->sent_state = SENT_PAM_ACCT_MGMT;
 
-  SSH_DEBUG(4, ("Sending a SSH_PAM_ACCT_MGMT packet to ssh-pam-client..."));
+  SSH_DEBUG(4, ("Sending a SSH_PAM_ACCT_MGMT packet to PAM subprocess..."));
 
   ssh_packet_wrapper_send_encode(gdata->wrapper,
                                  SSH_PAM_ACCT_MGMT,
@@ -413,47 +431,47 @@ SSH_OLDFSM_STEP(ssh_pam_acct_mgmt)
                                  SSH_FORMAT_UINT32,
                                  (SshUInt32) PAM_DISALLOW_NULL_AUTHTOK,
                                  SSH_FORMAT_END);
-  return SSH_OLDFSM_SUSPENDED;
+  return SSH_FSM_SUSPENDED;
 }
 
-SSH_OLDFSM_STEP(ssh_pam_chauthtok)
+SSH_FSM_STEP(ssh_pams_chauthtok)
 {
-  SSH_OLDFSM_GDATA(SshServerPAMAuth);
+  SshServerPAMAuth gdata = (SshServerPAMAuth)fsm_context;
 
-  SSH_OLDFSM_SET_NEXT("ssh_pam_acct_mgmt");
+  SSH_FSM_SET_NEXT(ssh_pams_acct_mgmt);
 
   gdata->sent_state = SENT_PAM_CHAUTHTOK;
 
-  SSH_DEBUG(4, ("Sending a SSH_PAM_CHAUTHTOK packet to ssh-pam-client..."));
+  SSH_DEBUG(4, ("Sending a SSH_PAM_CHAUTHTOK packet to PAM subprocess..."));
 
   ssh_packet_wrapper_send_encode(gdata->wrapper,
                                  SSH_PAM_CHAUTHTOK,
                                  SSH_FORMAT_UINT32,
                                  (SshUInt32) PAM_CHANGE_EXPIRED_AUTHTOK,
                                  SSH_FORMAT_END);
-  return SSH_OLDFSM_SUSPENDED;
+  return SSH_FSM_SUSPENDED;
 }
 
-SSH_OLDFSM_STEP(ssh_pam_open_session)
+SSH_FSM_STEP(ssh_pams_open_session)
 {
-  SSH_OLDFSM_GDATA(SshServerPAMAuth);
-  SSH_OLDFSM_SET_NEXT("ssh_pam_setcred");
+  SshServerPAMAuth gdata = (SshServerPAMAuth)fsm_context;
+  SSH_FSM_SET_NEXT(ssh_pams_setcred);
 
   gdata->sent_state = SENT_PAM_OPEN_SESSION;
 
-  SSH_DEBUG(4, ("Sending a SSH_PAM_OPEN_SESSION packet to ssh-pam-client..."));
+  SSH_DEBUG(4, ("Sending a SSH_PAM_OPEN_SESSION packet to PAM subprocess..."));
 
   ssh_packet_wrapper_send_encode(gdata->wrapper,
                                  SSH_PAM_OPEN_SESSION,
                                  SSH_FORMAT_UINT32, (SshUInt32) 0,
                                  SSH_FORMAT_END);
-  return SSH_OLDFSM_SUSPENDED;
+  return SSH_FSM_SUSPENDED;
 }
 
-SSH_OLDFSM_STEP(ssh_pam_setcred)
+SSH_FSM_STEP(ssh_pams_setcred)
 {
-  SSH_OLDFSM_GDATA(SshServerPAMAuth);
-  SSH_OLDFSM_SET_NEXT("ssh_pam_auth_successful");
+  SshServerPAMAuth gdata = (SshServerPAMAuth)fsm_context;
+  SSH_FSM_SET_NEXT(ssh_pams_auth_successful);
 
   gdata->sent_state = SENT_PAM_SETCRED;
 
@@ -463,39 +481,56 @@ SSH_OLDFSM_STEP(ssh_pam_setcred)
                                  SSH_FORMAT_UINT32,
                                  (SshUInt32) PAM_ESTABLISH_CRED,
                                  SSH_FORMAT_END);
-  return SSH_OLDFSM_SUSPENDED;
+  return SSH_FSM_SUSPENDED;
 }
 
-SSH_OLDFSM_STEP(ssh_pam_auth_successful)
+void pam_session_close_cb(void *context)
 {
-  SSH_OLDFSM_GDATA(SshServerPAMAuth);
+  SshPacketWrapper wrapper = (SshPacketWrapper)context;
+  SSH_DEBUG(2, ("Closing user's session, and destroying wrapper."));
+  SSH_PRECOND(wrapper);
+  ssh_packet_wrapper_send_encode(wrapper,
+                                 SSH_PAM_CLOSE_SESSION,
+                                 /* XXX flags? */
+                                 SSH_FORMAT_UINT32, 0L,
+                                 SSH_FORMAT_END);
+  ssh_packet_wrapper_send_encode(wrapper,
+                                 SSH_PAM_END,
+                                 /* XXX flags? */
+                                 SSH_FORMAT_UINT32, 0L,
+                                 SSH_FORMAT_END);
+  ssh_packet_wrapper_destroy(wrapper);  
+}
+
+SSH_FSM_STEP(ssh_pams_auth_successful)
+{
+  SshServerPAMAuth gdata = (SshServerPAMAuth)fsm_context;
 
   ssh_packet_wrapper_can_receive(gdata->wrapper, FALSE);
   ssh_packet_wrapper_set_callbacks(gdata->wrapper,
-                                   NULL, NULL, NULL, NULL);
+                                   NULL_FNPTR, NULL_FNPTR, NULL_FNPTR, NULL);
 
-  ssh_user_pam_set_handle(gdata->uc, gdata->wrapper);
+  ssh_common_register_clean_up(gdata->server->common,
+                               pam_session_close_cb, gdata->wrapper);
+  
   gdata->wrapper = NULL;
 
-  ssh_oldfsm_kill_thread(gdata->aux_thread);
-  gdata->aux_thread = NULL;
-
-  ssh_oldfsm_destroy(gdata->fsm);
-
   *gdata->state_placeholder = NULL;
+
   /* inform userauth-layer of successful authentication. */
   (*gdata->completion_proc)(SSH_AUTH_SERVER_ACCEPTED,
                             gdata->client_packet,
                             gdata->completion_context);
 
-  return SSH_OLDFSM_FINISH;
+  SSH_FSM_SET_NEXT(ssh_pams_finish);
+  return SSH_FSM_CONTINUE;
 }
 
 /* The aux_thread handles these. */
-SSH_OLDFSM_STEP(ssh_pam_process_client_packet)
+SSH_FSM_STEP(ssh_pams_process_client_packet)
 {
   Boolean msg_code;
-  SSH_OLDFSM_GDATA(SshServerPAMAuth);
+  SshServerPAMAuth gdata = (SshServerPAMAuth)fsm_context;
 
   SSH_PRECOND(gdata->client_packet);
   SSH_PRECOND(thread == gdata->aux_thread);
@@ -526,9 +561,9 @@ SSH_OLDFSM_STEP(ssh_pam_process_client_packet)
         (SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED,
          gdata->client_packet,
          gdata->completion_context);
-      ssh_oldfsm_set_next(gdata->main_thread, "ssh_pam_abort");
-      ssh_oldfsm_continue(gdata->main_thread);
-      return SSH_OLDFSM_FINISH;
+      ssh_fsm_set_next(gdata->main_thread, ssh_pams_finish);
+      ssh_fsm_continue(gdata->main_thread);
+      return SSH_FSM_FINISH;
     }
 
   ssh_packet_wrapper_send_encode(gdata->wrapper,
@@ -537,68 +572,29 @@ SSH_OLDFSM_STEP(ssh_pam_process_client_packet)
                                  ssh_buffer_ptr(gdata->client_packet),
                                  ssh_buffer_len(gdata->client_packet),
                                  SSH_FORMAT_END);
-  return SSH_OLDFSM_SUSPENDED;
+  return SSH_FSM_SUSPENDED;
 }
 
-SSH_OLDFSM_STEP(ssh_pam_suspend)
+SSH_FSM_STEP(ssh_pams_suspend)
 {
-  return SSH_OLDFSM_SUSPENDED;
+  return SSH_FSM_SUSPENDED;
 }
 
-SSH_OLDFSM_STEP(ssh_pam_abort)
+SSH_FSM_STEP(ssh_pams_finish)
 {
-  SSH_OLDFSM_GDATA(SshServerPAMAuth);
+  SshServerPAMAuth gdata = (SshServerPAMAuth)fsm_context;
 
   if (gdata->aux_thread)
     {
-      ssh_oldfsm_kill_thread(gdata->aux_thread);
+      ssh_fsm_kill_thread(gdata->aux_thread);
       gdata->aux_thread = NULL;
     }
   if (gdata->fsm)
     {
-      ssh_oldfsm_destroy(gdata->fsm);
+      ssh_fsm_destroy(gdata->fsm);
       gdata->fsm = NULL;
     }
 
-  return SSH_OLDFSM_FINISH;
-}
-
-SshOldFSMStateMapItemStruct pam_states[] =
-{
-  { "ssh_pam_start", "Start PAM authentication", ssh_pam_start },
-  { "ssh_pam_authenticate", "Request pam_authenticate()",
-    ssh_pam_authenticate },
-  { "ssh_pam_acct_mgmt", "Request pam_acct_mgmt()",
-    ssh_pam_acct_mgmt },
-  { "ssh_pam_chauthtok", "Request pam_chauthtok()",
-    ssh_pam_chauthtok },
-  { "ssh_pam_open_session", "Request pam_open_session()",
-    ssh_pam_open_session },
-  { "ssh_pam_setcred", "Request pam_setcred()",
-    ssh_pam_setcred },
-  { "ssh_pam_set_item_rhost", "Request pam_set_item() for RHOST",
-    ssh_pam_set_item_rhost },
-  { "ssh_pam_set_item_tty", "Request pam_set_item() for TTY",
-    ssh_pam_set_item_tty },
-  { "ssh_pam_set_item_ruser", "Request pam_set_item() for RUSER",
-    ssh_pam_set_item_ruser },
-
-  { "ssh_pam_process_client_packet", "Process packet received from client",
-    ssh_pam_process_client_packet },
-
-  { "ssh_pam_auth_successful", "Authentication was successful",
-    ssh_pam_auth_successful },
-
-  { "ssh_pam_suspend", "Suspend thread", ssh_pam_suspend },
-
-  { "ssh_pam_abort", "Abort PAM ops", ssh_pam_abort }
-};
-
-void pam_destructor(void *gdata_ctx)
-{
-  SshServerPAMAuth gdata = (SshServerPAMAuth) gdata_ctx;
-  SSH_PRECOND(gdata);
-
   ssh_xfree(gdata->service_name);
 
   if (gdata->wrapper)
@@ -609,8 +605,43 @@ void pam_destructor(void *gdata_ctx)
     }
 
   *gdata->state_placeholder = NULL;
+
+  ssh_xfree(gdata);
+  return SSH_FSM_FINISH;
 }
 
+SshFSMStateDebugStruct pam_states[] =
+{
+  { "ssh_pams_start", "Start PAM authentication", ssh_pams_start },
+  { "ssh_pams_authenticate", "Request pam_authenticate()",
+    ssh_pams_authenticate },
+  { "ssh_pams_acct_mgmt", "Request pam_acct_mgmt()",
+    ssh_pams_acct_mgmt },
+  { "ssh_pams_chauthtok", "Request pam_chauthtok()",
+    ssh_pams_chauthtok },
+  { "ssh_pams_open_session", "Request pam_open_session()",
+    ssh_pams_open_session },
+  { "ssh_pams_setcred", "Request pam_setcred()",
+    ssh_pams_setcred },
+  { "ssh_pams_set_item_rhost", "Request pam_set_item() for RHOST",
+    ssh_pams_set_item_rhost },
+  { "ssh_pams_set_item_tty", "Request pam_set_item() for TTY",
+    ssh_pams_set_item_tty },
+  { "ssh_pams_set_item_ruser", "Request pam_set_item() for RUSER",
+    ssh_pams_set_item_ruser },
+
+  { "ssh_pams_process_client_packet", "Process packet received from client",
+    ssh_pams_process_client_packet },
+
+  { "ssh_pams_auth_successful", "Authentication was successful",
+    ssh_pams_auth_successful },
+
+  { "ssh_pams_suspend", "Suspend server thread",
+    ssh_pams_suspend },
+
+  { "ssh_pams_finish", "Abort PAM ops", ssh_pams_finish }
+};
+
 void ssh_server_auth_pam(SshAuthServerOperation op,
                          const char *user,
                          SshUser uc,
@@ -626,7 +657,6 @@ void ssh_server_auth_pam(SshAuthServerOperation op,
   SshServer server = (SshServer)method_context;
   SshConfig config = server->config;
   SshServerPAMAuth state = *state_placeholder;
-  SshOldFSM fsm;
   SshStream child_stdio;
   Boolean msg_code;
 
@@ -670,8 +700,8 @@ void ssh_server_auth_pam(SshAuthServerOperation op,
         {
           ssh_log_event(config->log_facility,
                         SSH_LOG_WARNING,
-                        "Extra data at of SSH_MSG_USERAUTH_REQUEST packet "
-                        "for PAM authentication.",
+                        "Extra data at end of SSH_MSG_USERAUTH_REQUEST "
+                        "packet for PAM authentication.",
                         ssh_user_name(uc));
           (*completion_proc)(SSH_AUTH_SERVER_REJECTED_AND_METHOD_DISABLED,
                              packet,
@@ -695,24 +725,19 @@ void ssh_server_auth_pam(SshAuthServerOperation op,
           break;
         case SSH_PIPE_CHILD_OK:
           {
-            char *pam_argv[2];
-
-            SSH_TRACE(0, ("Child: Execing ssh-pam-client. (path: %s)",
-                          config->ssh_pam_client_path));
-
-            pam_argv[0] = config->ssh_pam_client_path;
-            pam_argv[1] = NULL;
-
-            execvp(pam_argv[0], pam_argv);
-            ssh_log_event(config->log_facility,
-                          SSH_LOG_ERROR,
-                          "ssh_server_auth_pam: execing ssh-pam-client "
-                          "failed (system error message: %s)",
-                          strerror(errno));
-
-            ssh_fatal("ssh_server_auth_pam: execing ssh-pam-client "
-                      "failed (system error message: %s)",
-                      strerror(errno));
+            if (ssh_pam_conv())
+              {
+                ssh_log_event(config->log_facility,
+                              SSH_LOG_ERROR,
+                              "PAM transaction resulted in error.");
+                exit(1);
+              }
+            else
+              {
+                SSH_DEBUG(2, ("PAM transaction was a success."));
+                exit(0);
+              }
+            SSH_NOTREACHED;
             break;
           }
 
@@ -721,16 +746,9 @@ void ssh_server_auth_pam(SshAuthServerOperation op,
           break;
         }
 
-      fsm = ssh_oldfsm_allocate(sizeof(*state),
-                             pam_states,
-                             SSH_OLDFSM_NUM_STATES(pam_states),
-                             pam_destructor);
+      state = ssh_xcalloc(1, sizeof(*state));
 
-      state = ssh_oldfsm_get_gdata_fsm(fsm);
-
-      memset(state, 0, sizeof(*state));
-
-      state->fsm = fsm;
+      state->fsm = ssh_fsm_create(state);
 
       state->server = server;
       state->uc = uc;
@@ -743,19 +761,20 @@ void ssh_server_auth_pam(SshAuthServerOperation op,
       state->doing_conversation = FALSE;
       state->state_placeholder = state_placeholder;
 
+      ssh_fsm_register_debug_names(state->fsm, pam_states,
+                                   SSH_FSM_NUM_STATES(pam_states));
+      
       state->wrapper = ssh_packet_wrap(child_stdio,
                                        received_packet_cb,
                                        received_eof_cb,
-                                       NULL, state);
+                                       NULL_FNPTR, state);
 
 
-      state->main_thread = ssh_oldfsm_spawn(state->fsm,
-                                         0, "ssh_pam_start",
-                                         NULL, NULL);
-
-      state->aux_thread = ssh_oldfsm_spawn(state->fsm,
-                                        0, "ssh_pam_suspend",
-                                        NULL, NULL);
+      state->main_thread = ssh_fsm_thread_create(state->fsm, ssh_pams_start,
+                                                 NULL_FNPTR, NULL_FNPTR, NULL);
+      state->aux_thread = ssh_fsm_thread_create(state->fsm,
+                                                ssh_pams_suspend,
+                                                NULL_FNPTR, NULL_FNPTR, NULL);
 
       *state_placeholder = state;
       /* Rest is done in callbacks. */
@@ -763,8 +782,8 @@ void ssh_server_auth_pam(SshAuthServerOperation op,
 
     case SSH_AUTH_SERVER_OP_ABORT:
       SSH_ASSERT(state && state->main_thread);
-      ssh_oldfsm_set_next(state->main_thread, "ssh_pam_abort");
-      ssh_oldfsm_continue(state->main_thread);
+      ssh_fsm_set_next(state->main_thread, ssh_pams_finish);
+      ssh_fsm_continue(state->main_thread);
       (*completion_proc)(SSH_AUTH_SERVER_REJECTED, packet, completion_context);
       return;
 
@@ -776,8 +795,8 @@ void ssh_server_auth_pam(SshAuthServerOperation op,
       if (uc != state->uc)
         state->uc = uc;
 
-      ssh_oldfsm_set_next(state->aux_thread, "ssh_pam_process_client_packet");
-      ssh_oldfsm_continue(state->aux_thread);
+      ssh_fsm_set_next(state->aux_thread, ssh_pams_process_client_packet);
+      ssh_fsm_continue(state->aux_thread);
       return;
 
     case SSH_AUTH_SERVER_OP_UNDO_LONGTIME:
@@ -790,8 +809,8 @@ void ssh_server_auth_pam(SshAuthServerOperation op,
           if (state->wrapper)
             ssh_packet_wrapper_can_receive(state->wrapper, FALSE);
 
-          ssh_oldfsm_set_next(state->main_thread, "ssh_pam_abort");
-          ssh_oldfsm_continue(state->main_thread);
+          ssh_fsm_set_next(state->main_thread, ssh_pams_finish);
+          ssh_fsm_continue(state->main_thread);
         }
 
       *state_placeholder = NULL;
