@@ -26,4 +26,6 @@ Boolean ssh_compat_rsa_public_key_change_scheme(SshPublicKey pubkey,
 Boolean ssh_compat_rsa_private_key_change_scheme(SshPrivateKey privkey,
                                                  Boolean hash_is_sha1);
 
+/* Returns TRUE for an RSA public key. FALSE otherwise. */
+Boolean ssh_compat_is_rsa_public_key(SshPublicKey pubkey);
 #endif /* _SSH2COMPAT_H_ */
