@@ -16,7 +16,7 @@
 */
 
 /*
- * $Id: sshauth.h,v 1.1 2001/10/05 12:40:00 sjl Exp $
+ * $Id: sshauth.h,v 1.4 2002/03/10 17:59:39 sjl Exp $
  * $Log: sshauth.h,v $ * 
  * 
  * 
@@ -128,6 +128,25 @@
  * 
  * 
  * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -178,14 +197,14 @@ typedef void (*SshAuthPolicyCompletionCB)(char *continuations,
    The returned string should be allocated by ssh_xmalloc, and the caller
    must free the string when no longer needed. */
 typedef void (*SshAuthPolicyProc)(const char *user,
-                                   const char *service,
-                                   const char *client_ip,
-                                   const char *client_port,
-                                   const char *completed_authentications,
-                                   const char *disabled_authentications,
-                                   SshAuthPolicyCompletionCB completion_proc,
-                                   void *completion_ctx,
-                                   void *context);
+                                  const char *service,
+                                  const char *client_ip,
+                                  const char *client_port,
+                                  const char *completed_authentications,
+                                  const char *disabled_authentications,
+                                  SshAuthPolicyCompletionCB completion_proc,
+                                  void *completion_ctx,
+                                  void *context);
 
 /* Return code from an authentication method. */
 typedef enum {
@@ -206,7 +225,7 @@ typedef enum {
      probing whether a public key would be acceptable.  The authentication
      method must have replaced the packet in the buffer by the complete
      payload of a packet to be sent back to the client.  The packet to be
-     sent back must have a packet number in the range 35...39. */
+     sent back must have a packet number in the range 60...79. */
   SSH_AUTH_SERVER_REJECTED_WITH_PACKET_BACK,
 
   /* The authentication request is incomplete.  A packet should be
@@ -214,16 +233,23 @@ typedef enum {
      client sends back a new packet with the appropriate response.
      The authentication method should replace the data in the buffer
      by the complete payload of the packet to send back (it must have
-     a packet number in the range 35...39).  The method can use
+     a packet number in the range 60...79).  The method can use
      *state_placeholder to keep its internal state.  If a packet is
-     received from the client with the number in the 35...39 range, it
+     received from the client with the number in the 60...79 range, it
      is passed to the same method with SSH_AUTH_SERVER_OP_CONTINUE
      operation.  If any other packet number or a new authentication
      request is received from the client, the method will be called
      with SSH_AUTH_SERVER_OP_ABORT before processing the other
      request; in this case the method should free any data stored at
      *state_placeholder. */
-  SSH_AUTH_SERVER_CONTINUE_WITH_PACKET_BACK
+  SSH_AUTH_SERVER_CONTINUE_WITH_PACKET_BACK,
+
+  /* As above, but the whole packet (with packet types etc intact)
+     should be passed back to the authentication method. Also, the
+     clients packet won't be checked for user names, methods, or service
+     names. It is just passed to the authentication method. Also, the
+     continuation message will be SSH_AUTH_SERVER_OP_CONTINUE_SPECIAL. */
+  SSH_AUTH_SERVER_CONTINUE_WITH_PACKET_BACK_SPECIAL
 } SshAuthServerResult;
 
 /* Operation code for the authentication method. */
@@ -253,6 +279,12 @@ typedef enum {
      SSH_AUTH_SERVER_CONTINUE_WITH_PACKET_BACK. */
   SSH_AUTH_SERVER_OP_CONTINUE,
 
+  /* As above, but the whole method-specific authentication packet is
+     passed to the authentication method. This is guaranteed to be
+     called only if the method has previously returned
+     SSH_AUTH_SERVER_CONTINUE_WITH_PACKET_BACK_SPECIAL. */
+  SSH_AUTH_SERVER_OP_CONTINUE_SPECIAL,
+
   /* Undoes any side-effects by previous calls to this function.  This
      is called for all attempted authentications after if the user
      name or service changes.  This must return side-effects to the
@@ -391,6 +423,9 @@ SshStream ssh_auth_server_wrap(SshStream transport_stream,
                                SshAuthServerMethodsDestroy destroy_cb,
                                void *destroy_context,
                                const char *banner,
+
+
+
                                SshUser *uc_return);
 
 
@@ -496,7 +531,12 @@ typedef enum {
   /* Like above, but the method does not terminate upon 
      SSH_MSG_USERAUTH_FAILURE or SSH_MSG_USERAUTH_SUCCESS. These messages
      should be passed to the method. */
-  SSH_AUTH_CLIENT_SEND_AND_CONTINUE_MULTIPLE
+  SSH_AUTH_CLIENT_SEND_AND_CONTINUE_MULTIPLE,
+
+  /* As SSH_AUTH_CLIENT_SEND_AND_CONTINUE, but nothing should be
+     prepended to the response packet (for example keyboard-interactive
+     expects this behaviour).*/
+  SSH_AUTH_CLIENT_SEND_AND_CONTINUE_SPECIAL
 
 } SshAuthClientResult;
 
@@ -515,6 +555,14 @@ typedef void (*SshAuthClientCompletionProc)(SshAuthClientResult result,
                                             SshBuffer packet,
                                             void *completion_context);
 
+/* A callback to display the SSH_MSG_BANNER message. This is because
+   otherwise the banner message could be output only after some
+   authentication has already been started, as the authentication
+   methods are called "out-of-band" (they don't use the cross-layer
+   protocol), and the banner message MUST be displayed first, for legal
+   reasons etc.  */
+typedef void (*SshAuthBannerMsgCB)(const char *msg, void *context);
+
 /* Functions of this type represent authentication methods on the client
    side.  The generic code calls these to test if the method is currently
    available, to construct authentication requests, and to process
@@ -585,6 +633,9 @@ typedef void (*SshAuthClientMethodsDestroy)(SshAuthClientMethod methods,
                            ordered the preferred one first.  The array
                            terminates with a NULL method name.
      `method_context'      context to pass to methods (normally NULL)
+     `banner_msg_cb'       method called to display the banner message
+     `callback_context'    a context given to application level callbacks
+                           (currently only `banner_msg_cb')
      `destroy_cb'          method called when authentication stream is
                            destroyed to destroy the authentication
                            method array. (can be NULL)
@@ -596,6 +647,8 @@ SshStream ssh_auth_client_wrap(SshStream transport_stream,
                                const char *service,
                                SshAuthClientMethodStruct methods[],
                                void *method_context,
+                               SshAuthBannerMsgCB banner_msg_cb,
+                               void *callback_context,
                                SshAuthClientMethodsDestroy destroy_cb,
                                void *destroy_context);
 
