@@ -55,10 +55,15 @@ typedef struct {
   SshAuthServerMethodsDestroy destroy_callback;
   void *destroy_context;
 
-  /* If true, we are waiting for a continuation packet for an authentication
+  /* If TRUE, we are waiting for a continuation packet for an authentication
      method. */
   Boolean waiting_continuation;
 
+  /* If TRUE, the clients packet won't be checked for the standard user
+     names, method name or service names. Provided, of course, that it
+     is a method specific packet. */
+  Boolean waiting_special;
+
   /* Number of the method that is active (valid only if waiting_continuation
      is TRUE). */
   int active_method_index;
@@ -133,6 +138,11 @@ typedef struct {
      (unless this is NULL).  If set, it will be the caller's responsibility
      to free the SshUser object when no longer needed. */
   SshUser *uc_return;
+
+
+
+
+
 } *SshAuthServer;
 
 
@@ -149,7 +159,7 @@ void ssh_auths_send_failure(SshAuthServer auth, Boolean partial_success)
                              SSH_FORMAT_CHAR,
                              (unsigned int) SSH_MSG_USERAUTH_FAILURE,
                              SSH_FORMAT_UINT32_STR,
-                              auth->continuations, strlen(auth->continuations),
+                             auth->continuations, strlen(auth->continuations),
                              SSH_FORMAT_BOOLEAN, partial_success,
                              SSH_FORMAT_END);
 }
@@ -163,6 +173,7 @@ void ssh_auths_disconnect(SshAuthServer auth, int disconnect_code,
     {
       ssh_auths_call_method(auth, SSH_AUTH_SERVER_OP_ABORT, NULL, 0);
       auth->waiting_continuation = FALSE;
+      auth->waiting_special = FALSE;
     }
 
   /* Send the disconnect message and EOF down. */
@@ -197,9 +208,9 @@ void register_success_policy_cb(char *continuations, void *completion_ctx)
 {
   AuthsGenericCompletionCtx ctx = (AuthsGenericCompletionCtx) completion_ctx;
   
-  SSH_PRECOND(ctx);
-  SSH_PRECOND(ctx->auth);
-  SSH_PRECOND(ctx->success_cb);
+  SSH_PRECOND(ctx != NULL);
+  SSH_PRECOND(ctx->auth != NULL);
+  SSH_PRECOND(ctx->success_cb != NULL_FNPTR);
   
   if (ctx->auth->continuations)
     ssh_xfree(ctx->auth->continuations);
@@ -223,7 +234,7 @@ void ssh_auths_register_success(SshAuthServer auth, const char *method,
     ssh_xcalloc(1, sizeof(*completion_ctx));
   char *cp;
   
-  SSH_PRECOND(success_cb);
+  SSH_PRECOND(success_cb != NULL_FNPTR);
   
   /* Add the method to the list of successful authentications. */
   if (auth->successful_authentications == NULL)
@@ -271,6 +282,7 @@ void ssh_auths_clear_all_state(SshAuthServer auth,
 
 void ssh_auths_success(SshAuthServer auth)
 {
+  SSH_ASSERT(auth->active_method_index >= 0);
   /* Set status to successful (but drain pending output), and send
      SUCCESS packet. */
   SSH_DEBUG(6, ("success method = %s",
@@ -347,6 +359,7 @@ void success_completion_proc(Boolean no_more_needed, SshAuthServer auth)
       /* More authentications are needed. */
       SSH_DEBUG(6, ("More authentications needed."));
       auth->waiting_continuation = FALSE;
+      auth->waiting_special = FALSE;
       ssh_auths_send_failure(auth, TRUE);
     }
 }
@@ -354,7 +367,7 @@ void success_completion_proc(Boolean no_more_needed, SshAuthServer auth)
 void rejected_policy_cb(char *continuations, void *completion_ctx)
 {
   SshAuthServer auth = (SshAuthServer) completion_ctx;
-  SSH_PRECOND(auth);
+  SSH_PRECOND(auth != NULL);
 
   auth->continuations = continuations;
   
@@ -369,14 +382,14 @@ void ssh_auths_completion_proc(SshAuthServerResult result,
   SshAuthServer auth;
   int i;
   unsigned int packet_type;
-   
+
   SshAuthServerCompletionCtx completion_context =
     (SshAuthServerCompletionCtx) context;
 
-  SSH_PRECOND(completion_context);  
+  SSH_PRECOND(completion_context != NULL);
   op = completion_context->op;
   auth = completion_context->auth;
-  SSH_ASSERT(auth);
+  SSH_ASSERT(auth != NULL);
   i = completion_context->method_index;  
   
   ssh_xfree(completion_context);  
@@ -386,6 +399,7 @@ void ssh_auths_completion_proc(SshAuthServerResult result,
     {
     case SSH_AUTH_SERVER_OP_START:
     case SSH_AUTH_SERVER_OP_CONTINUE:
+    case SSH_AUTH_SERVER_OP_CONTINUE_SPECIAL:
       switch (result)
         {
         case SSH_AUTH_SERVER_ACCEPTED:
@@ -403,6 +417,7 @@ void ssh_auths_completion_proc(SshAuthServerResult result,
              had side-effects that were recorded in longtime_placeholder). */
           SSH_ASSERT(auth->state_placeholders[i] == NULL);
           auth->waiting_continuation = FALSE;
+          auth->waiting_special = FALSE;
           ssh_auths_send_failure(auth, FALSE);
           break;
 
@@ -414,7 +429,8 @@ void ssh_auths_completion_proc(SshAuthServerResult result,
             char *to_be_deleted;
             SSH_ASSERT(auth->state_placeholders[i] == NULL);
             auth->waiting_continuation = FALSE;
-            auth->methods[i].proc = NULL;
+            auth->waiting_special = FALSE;
+            auth->methods[i].proc = NULL_FNPTR;
             to_be_deleted = auth->disabled_authentications;
             ssh_dsprintf(&auth->disabled_authentications, "%s%s%s",
                          auth->disabled_authentications,
@@ -442,9 +458,10 @@ void ssh_auths_completion_proc(SshAuthServerResult result,
           /* The authentication method wants to send out a packet, and then
              continue authentication with the same method. */
           auth->waiting_continuation = FALSE;
+          auth->waiting_special = FALSE;
 
           /* Sanity check the outgoing packet. */
-          SSH_ASSERT(packet);          
+          SSH_ASSERT(packet != NULL);
           if (ssh_decode_array(ssh_buffer_ptr(packet), ssh_buffer_len(packet),
                                SSH_FORMAT_CHAR, &packet_type,
                                SSH_FORMAT_END) == 0)
@@ -462,12 +479,16 @@ void ssh_auths_completion_proc(SshAuthServerResult result,
           break;
 
         case SSH_AUTH_SERVER_CONTINUE_WITH_PACKET_BACK:
+        case SSH_AUTH_SERVER_CONTINUE_WITH_PACKET_BACK_SPECIAL:
           /* The authentication method wants to send out a packet, and then
              continue authentication with the same method. */
           auth->waiting_continuation = TRUE;
 
+          if (result == SSH_AUTH_SERVER_CONTINUE_WITH_PACKET_BACK_SPECIAL)
+            auth->waiting_special = TRUE;
+          
           /* Sanity check the outgoing packet. */
-          SSH_ASSERT(packet);
+          SSH_ASSERT(packet != NULL);
           if (ssh_decode_array(ssh_buffer_ptr(packet), ssh_buffer_len(packet),
                                SSH_FORMAT_CHAR, &packet_type,
                                SSH_FORMAT_END) == 0 ||
@@ -530,7 +551,7 @@ void ssh_auths_call_method(SshAuthServer auth,
   SSH_ASSERT(i >= 0 && i < auth->num_methods);
   SSH_ASSERT(auth->requested_user != NULL);
   
-  if (auth->methods[i].proc == NULL)
+  if (auth->methods[i].proc == NULL_FNPTR)
     {
       SSH_DEBUG(3, ("Authentication method proc is NULL"));
       return;
@@ -546,8 +567,7 @@ void ssh_auths_call_method(SshAuthServer auth,
   completion_context->auth = auth;
   completion_context->op = op;
   completion_context->method_index = i;
-  
-  /* Call the authentication method. */
+
   /* Call the authentication method. */
   (*auth->methods[i].proc)(op, auth->requested_user, auth->uc, buffer,
                            auth->session_id, auth->session_id_len,
@@ -589,6 +609,7 @@ void ssh_auths_process_request_finalize(void *ctx)
       ssh_xfree(method);
     }
 
+
   /* If we don't have a valid method, return failure. */
   if (completion_ctx->auth->active_method_index == -1 || !cont_found)
     {
@@ -598,13 +619,24 @@ void ssh_auths_process_request_finalize(void *ctx)
   
   /* Process the authentication request. */
   if (completion_ctx->auth->waiting_continuation)
-    ssh_auths_call_method(completion_ctx->auth, SSH_AUTH_SERVER_OP_CONTINUE,
-                          completion_ctx->saved_packet,
-                          completion_ctx->saved_packet_len);
+    {
+      if (completion_ctx->auth->waiting_special)
+        ssh_auths_call_method(completion_ctx->auth,
+                              SSH_AUTH_SERVER_OP_CONTINUE_SPECIAL,
+                              completion_ctx->saved_packet,
+                              completion_ctx->saved_packet_len);
+      else
+        ssh_auths_call_method(completion_ctx->auth,
+                              SSH_AUTH_SERVER_OP_CONTINUE,
+                              completion_ctx->saved_packet,
+                              completion_ctx->saved_packet_len);
+    }
   else
-    ssh_auths_call_method(completion_ctx->auth, SSH_AUTH_SERVER_OP_START,
-                          completion_ctx->saved_packet,
-                          completion_ctx->saved_packet_len);
+    {
+      ssh_auths_call_method(completion_ctx->auth, SSH_AUTH_SERVER_OP_START,
+                            completion_ctx->saved_packet,
+                            completion_ctx->saved_packet_len);
+    }
  free_vars:
   ssh_xfree(completion_ctx->saved_packet);
   ssh_xfree(completion_ctx);
@@ -612,11 +644,43 @@ void ssh_auths_process_request_finalize(void *ctx)
 
 void ssh_auths_process_request_finalize_cb(char *continuations, void *ctx)
 {
+
+
+
+
+
   AuthsGenericCompletionCtx completion_ctx =
     (AuthsGenericCompletionCtx) ctx;
 
-  SSH_PRECOND(completion_ctx);
-  SSH_PRECOND(completion_ctx->auth);
+  SSH_PRECOND(completion_ctx != NULL);
+  SSH_PRECOND(completion_ctx->auth != NULL);
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
   if (completion_ctx->auth->continuations)
     ssh_xfree(completion_ctx->auth->continuations);
@@ -636,16 +700,14 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
   unsigned int packet_type;
   char *user, *service, *method;
   int i;
-  size_t parsed_bytes;
+  size_t parsed_bytes, ret;
   unsigned char ch;
-  AuthsGenericCompletionCtx completion_ctx;  
+  AuthsGenericCompletionCtx completion_ctx = NULL;
 
   /* Decode the common part of the authentication request. */
+  
   parsed_bytes = ssh_decode_array(data, len,
                                   SSH_FORMAT_CHAR, &packet_type,
-                                  SSH_FORMAT_UINT32_STR, &user, NULL,
-                                  SSH_FORMAT_UINT32_STR, &service, NULL,
-                                  SSH_FORMAT_UINT32_STR, &method, NULL,
                                   SSH_FORMAT_END);
   if (parsed_bytes == 0)
     {
@@ -654,6 +716,31 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
       return;
     }
 
+  if (auth->waiting_special &&
+      (packet_type >=  SSH_FIRST_USERAUTH_METHOD_PACKET &&
+       packet_type <= SSH_LAST_USERAUTH_METHOD_PACKET))
+    {
+      /* keyboard-interactive doesn't follow the convention, that method
+         specific packets should include user name, service name or
+         method name. */
+      ssh_auths_call_method(auth, SSH_AUTH_SERVER_OP_CONTINUE_SPECIAL,
+                            data, len);
+      return;
+    }
+
+  ret = ssh_decode_array(data + parsed_bytes, len - parsed_bytes,
+                         SSH_FORMAT_UINT32_STR, &user, NULL,
+                         SSH_FORMAT_UINT32_STR, &service, NULL,
+                         SSH_FORMAT_UINT32_STR, &method, NULL,
+                         SSH_FORMAT_END);
+  if (ret == 0)
+    {
+      ssh_auths_disconnect(auth, SSH_DISCONNECT_PROTOCOL_ERROR,
+                           "Received bad authentication packet");
+      return;
+    }
+  parsed_bytes += ret;
+  
   /* All packets we receive should be of type SSH_MSG_USERAUTH_REQUEST. */
   if (packet_type != SSH_MSG_USERAUTH_REQUEST)
     {
@@ -727,6 +814,7 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
       SSH_DEBUG(6, ("process_request: cancelling old continued method"));
       ssh_auths_call_method(auth, SSH_AUTH_SERVER_OP_ABORT, NULL, 0);
       auth->waiting_continuation = FALSE;
+      auth->waiting_special = FALSE;
     }
 
   /* If the user or service has changed, undo all side-effects by
@@ -738,6 +826,8 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
     {
       SSH_DEBUG(6, ("process_request: undoing old state"));
       ssh_auths_clear_all_state(auth, SSH_AUTH_SERVER_OP_UNDO_LONGTIME);
+      auth->waiting_continuation = FALSE;
+      auth->waiting_special = FALSE;
       if (auth->requested_user)
         {
           ssh_xfree(auth->requested_user);
@@ -759,6 +849,15 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
           auth->successful_authentications = NULL;
         }
       SSH_ASSERT(!auth->waiting_continuation);
+
+      /* XXX Changing user is not properly tested. Currently no
+         implementation that I know of supports this feature, so it only
+         serves to complicate things (and possibly introduces exploitable
+         bugs). */
+      ssh_auths_disconnect(auth, SSH_DISCONNECT_BY_APPLICATION,
+                           "Changing user or service during authentication "
+                           "not allowed.");
+      return;
     }
 
   /* Save user and service if not already saved. */
@@ -766,7 +865,21 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
     {
       auth->requested_user = ssh_xstrdup(user);
       auth->requested_service = ssh_xstrdup(service);
-      auth->uc = ssh_user_initialize(auth->requested_user, TRUE);
+
+
+
+
+
+
+
+
+
+          auth->uc = ssh_user_initialize(auth->requested_user, TRUE);
+
+
+
+
+
     }
 
   /* Find the method. */
@@ -781,9 +894,17 @@ void ssh_auths_process_request(SshAuthServer auth, const unsigned char *data,
   /* If the possible continuations haven't been computed yet, do it now. */
   completion_ctx = ssh_xcalloc(1, sizeof(*completion_ctx));
 
-  completion_ctx->saved_packet = ssh_xmemdup(data + parsed_bytes,
-                                             len - parsed_bytes);
-  completion_ctx->saved_packet_len = len - parsed_bytes;
+  if (auth->waiting_special)
+    {
+      completion_ctx->saved_packet = ssh_xmemdup(data, len);
+      completion_ctx->saved_packet_len = len;
+    }
+  else
+    {
+      completion_ctx->saved_packet = ssh_xmemdup(data + parsed_bytes,
+                                                 len - parsed_bytes);
+      completion_ctx->saved_packet_len = len - parsed_bytes;
+    }
   completion_ctx->auth = auth;
   
   if (auth->continuations == NULL)
@@ -1070,12 +1191,15 @@ SshStream ssh_auth_server_wrap(SshStream transport,
                                SshAuthServerMethodsDestroy destroy_cb,
                                void *destroy_context,
                                const char *banner,
+
+
+
                                SshUser *uc_return)
 {
   SshAuthServer auth;
   int num_methods;
 
-  /* Count the number of authentication methods avaialable. */
+  /* Count the number of authentication methods available. */
   for (num_methods = 0; methods[num_methods].name; num_methods++)
     ;
 
@@ -1084,13 +1208,14 @@ SshStream ssh_auth_server_wrap(SshStream transport,
   auth->down = ssh_cross_down_create(transport,
                                      ssh_auths_down_received_packet,
                                      ssh_auths_down_received_eof,
-                                     NULL, (void *)auth);
-  auth->up = ssh_cross_up_create(NULL, NULL, NULL,
+                                     NULL_FNPTR, (void *)auth);
+  auth->up = ssh_cross_up_create(NULL_FNPTR, NULL_FNPTR, NULL_FNPTR,
                                  ssh_auths_up_destroy,
                                  (void *)auth);
   auth->waiting_continuation = FALSE;
+  auth->waiting_special = FALSE;
   auth->active_method_index = -1;
-  if (policy_proc == NULL)
+  if (policy_proc == NULL_FNPTR)
     {
       /* Use default policy proc if the user supplied NULL. */
       policy_proc = ssh_auths_default_policy_proc;
@@ -1120,7 +1245,10 @@ SshStream ssh_auth_server_wrap(SshStream transport,
   if (banner)
     auth->banner_msg = ssh_xstrdup(banner);
   auth->first_request = TRUE;
-  
+
+
+
+
   auth->uc_return = uc_return;
 
   /* Start receiving packets from the down stream. */
