@@ -219,7 +219,7 @@ Boolean ssh_cross_down_input(SshCrossDown down)
           /* Packet header already in buffer. */
           ptr = ssh_buffer_ptr(&down->incoming);
           data_to_read = 4 + SSH_GET_32BIT(ptr);
-          SSH_ASSERT(data_to_read > data_read);
+          SSH_ASSERT(data_to_read >= data_read);
           ssh_xbuffer_append_space(&down->incoming, &ptr,
                                    data_to_read - data_read);
         }
@@ -393,9 +393,9 @@ SshCrossDown ssh_cross_down_create(SshStream down_stream,
 void ssh_cross_down_destroy(SshCrossDown down)
 {
   /* Clear the callbacks so that user functions are not called. */
-  down->received_packet = NULL;
-  down->received_eof = NULL;
-  down->can_send = NULL;
+  down->received_packet = NULL_FNPTR;
+  down->received_eof = NULL_FNPTR;
+  down->can_send = NULL_FNPTR;
 
   /* If we cannot destroy at this time, set the proper flag and return
      immediately without destroying.  This happens in some callbacks.
