@@ -29,7 +29,7 @@ static struct sgttyb saved_tio;
 #endif
 
 /* Old interrupt signal handler for read_passphrase. */
-static RETSIGTYPE (*old_handler)(int sig) = NULL;
+static RETSIGTYPE (*old_handler)(int sig) = NULL_FNPTR;
 
 /* Interrupt signal handler for read_passphrase. */
 
