@@ -25,15 +25,15 @@
 
 #include "sshfilterstream.h"
 
-int ssh_stdio_output_filter(SshBuffer data,
-                            size_t offset,
-                            Boolean eof_received,
-                            void *context);
+void ssh_stdio_output_filter(void *context,
+                             SshFilterGetCB get_data,
+                             SshFilterCompletionCB completed,
+                             void *internal_context);
 
-int ssh_stdio_input_filter(SshBuffer data,
-                           size_t offset, 
-                           Boolean eof_received,
-                           void *context);
+void ssh_stdio_input_filter(void *context,
+                            SshFilterGetCB get_data,
+                            SshFilterCompletionCB completed,
+                            void *internal_context);
 
 void ssh_stdio_filter_destroy(void *context);
 
