@@ -4,7 +4,7 @@
 
   Author: Mika Kojo <mkojo@ssh.fi>
 
-  Copyright (C) 1997, 2000, 2001 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997, 2000, 2001, 2002 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
   Created: Sat Feb 15 20:16:42 1997 [mkojo]
@@ -27,7 +27,7 @@
   */
 
 /*
- * $Id: sshasn1.h,v 1.1 2001/10/23 16:11:09 kivinen Exp $
+ * $Id: sshasn1.h,v 1.6 2002/03/22 00:40:51 toka Exp $
  * $Log: sshasn1.h,v $ * 
  * 
  * 
@@ -216,6 +216,24 @@
  * 
  * 
  * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -293,7 +311,10 @@ typedef enum
   /* This operation or command or type is not yet implemented in this
      revision. If operation or command or type is needed, then implementation
      should be considered. */
-  SSH_ASN1_STATUS_NOT_YET_IMPLEMENTED
+  SSH_ASN1_STATUS_NOT_YET_IMPLEMENTED,
+
+  /* Generic error, if none of the above is applicable */
+  SSH_ASN1_STATUS_ERROR
 } SshAsn1Status;
 
 /* Type of an ASN.1 encoding. */
@@ -383,24 +404,26 @@ typedef struct
 {
   /* year */
   unsigned int year;
-  /* months range from 01 to 12 */
-  unsigned int month;
-  /* days range from 01 to 31 */
-  unsigned int day;
-  /* hours range from 00 to 23 */
-  unsigned int hour;
-  /* minutes range 00 to 59 */
-  unsigned int minute;
-  /* seconds range 00 to 59. */
-  unsigned int second;
-  /* microseconds range 0 to 1000000. */
-  unsigned long msecond;
-  /* TRUE = local time is later than GMT,
+  /* TRUE  = local time is later than GMT,
      FALSE = local time is earlier than or equal to GMT */
   Boolean local;
+
+  /* months range from 01 to 12 */
+  SshUInt8 month;
+  /* days range from 01 to 31 */
+  SshUInt8 day;
+  /* hours range from 00 to 23 */
+  SshUInt8 hour;
+  /* minutes range 00 to 59 */
+  SshUInt8 minute;
+  /* seconds range 00 to 59. */
+  SshUInt8 second;
+  /* microseconds range 0 to 1000000. */
+  unsigned long msecond;
+
   /* Absolute value of the offset from GMT in hours and minutes. */
-  unsigned int absolute_hours;
-  unsigned int absolute_minutes;
+  SshUInt8 absolute_hours;
+  SshUInt8 absolute_minutes;
 } *SshBerTime, SshBerTimeStruct;
 
 /* The BER time routines are given here to make the include file listings
@@ -451,6 +474,8 @@ Boolean ssh_ber_time_set_from_string(SshBerTime ber_time, const char *str);
    string. */
 void ssh_ber_time_to_string(const SshBerTime ber_time, char **str);
 
+int ssh_ber_time_render(char *buf, int buf_size, int precision, void *datum);
+
 /* Convert only date part of given time to a string. */
 char *ssh_str_bertime_to_date_str(const SshBerTime ber_time);
 
@@ -848,11 +873,6 @@ SshAsn1Status ssh_asn1_remove_node(SshAsn1Node node);
 
 SshAsn1Status ssh_asn1_insert_subnode(SshAsn1Node base, SshAsn1Node node);
 
-/* Remove a subnode (or tree). Does not free any space and thus the subnode
-   or tree can be used later. Returns the subnode (or first in the list). */
-
-SshAsn1Node ssh_asn1_remove_subnode(SshAsn1Node base);
-
 /****** Direct manipulation of nodes (i.e. low level routines) *********/
 
 /* These should not be used if the same effect can be performed with
