@@ -1931,7 +1931,7 @@ ssh_pk_group_dh_setup(SshPkGroup group,
 SshCryptoStatus
 ssh_pk_group_dh_agree(SshPkGroup group,
                       SshPkGroupDHSecret secret,
-                      unsigned char *exchange_buffer,
+                      const unsigned char *exchange_buffer,
                       size_t exchange_buffer_length,
                       unsigned char *secret_value_buffer,
                       size_t secret_value_buffer_length,
@@ -1947,7 +1947,7 @@ ssh_pk_group_dh_agree(SshPkGroup group,
    because it is freed immediately when this callback returns. */
 typedef void (*SshPkGroupDHSetup)(SshCryptoStatus status,
                                   SshPkGroupDHSecret secret,
-                                  unsigned char *exchange_buffer,
+                                  const unsigned char *exchange_buffer,
                                   size_t exchange_buffer_len,
                                   void *context);
 
@@ -1970,7 +1970,7 @@ ssh_pk_group_dh_setup_async(SshPkGroup group,
    data away from the buffer, because it is freed immediately when this
    callback returns. */
 typedef void (*SshPkGroupDHAgree)(SshCryptoStatus status,
-                                  unsigned char *shared_secret_buffer,
+                                  const unsigned char *shared_secret_buffer,
                                   size_t shared_secret_buffer_len,
                                   void *context);
 
@@ -1985,7 +1985,7 @@ typedef void (*SshPkGroupDHAgree)(SshCryptoStatus status,
 SshOperationHandle
 ssh_pk_group_dh_agree_async(SshPkGroup group,
                             SshPkGroupDHSecret secret,
-                            unsigned char *exchange_buffer,
+                            const unsigned char *exchange_buffer,
                             size_t exchange_buffer_len,
                             SshPkGroupDHAgree callback,
                             void *context);
@@ -2043,7 +2043,7 @@ SshOperationHandle
 ssh_pk_group_udh_agree_async(SshPublicKey public_key,
                              SshPrivateKey private_key,
                              SshPkGroupDHSecret secret,
-                             unsigned char *exchange_buffer,
+                             const unsigned char *exchange_buffer,
                              size_t exchange_buffer_length,
                              SshPkGroupDHAgree callback,
                              void *context);
