@@ -14,7 +14,7 @@
   */
 
 /*
- * $Id: namelist.c,v 1.3 2001/10/19 12:52:27 tmo Exp $
+ * $Id: namelist.c,v 1.4 2002/03/01 08:52:22 tmo Exp $
  * $Log: namelist.c,v $ * 
  * 
  * 
@@ -117,6 +117,11 @@
  * 
  * 
  * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -440,7 +445,7 @@ SshNameTreeStatus ssh_ntree_parse(const char *namelist, SshNameTree tree)
 #define FLAG_CLOSE      8
   int level = 0;
 
-  if (namelist == NULL)
+  if (namelist == NULL || tree == NULL)
     return SSH_NTREE_ERROR;
 
   len = strlen(namelist);
