@@ -4,7 +4,7 @@
 
   Author: Mika Kojo <mkojo@ssh.fi>
 
-  Copyright (C) 1997-2001 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2002 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
   Created: Thu Jan  9 12:22:52 1997 [mkojo]
@@ -14,7 +14,7 @@
   */
 
 /*
- * $Id: genmac.c,v 1.4 2001/10/16 18:28:25 kivinen Exp $
+ * $Id: genmac.c,v 1.7 2002/04/09 17:03:34 sjl Exp $
  * $Log: genmac.c,v $ * 
  * 
  * 
@@ -180,6 +180,19 @@
  * 
  * 
  * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -236,9 +249,9 @@
 
 
 
-#ifdef SSHDIST_CRYPT_SSHMACS
-#include "macs.h"
-#endif /* SSHDIST_CRYPT_SSHMACS */
+
+
+
 #ifdef SSHDIST_CRYPT_HMAC
 #include "hmac.h"
 
@@ -275,9 +288,10 @@
 
 #endif /* !KERNEL */
 
-
 /* Control structure. */
-
+/* XXX Remove ifdef's for SSHDIST_SSH2, and make a mapping in the ssh2
+   protocol library instead. This stuff was committed because there was
+   no time for extensive changes before ssh-3.2.0 release. //sjl */
 static const SshMacDefStruct ssh_mac_algorithms[] =
 {
 #ifdef SSHDIST_CRYPT_HMAC
@@ -305,24 +319,48 @@ static const SshMacDefStruct ssh_mac_algorithms[] =
     ssh_hmac_ctxsize, ssh_hmac_init,
     ssh_hmac_start, ssh_hmac_update, ssh_hmac_96_final,
     ssh_hmac_96_of_buffer },
-  { "hmac-sha256", 32, FALSE,
+  {
+
+
+#ifdef SSHDIST_SSH2
+    "hmac-sha256@ssh.com",
+#endif /* SSHDIST_SSH2 */
+    32, FALSE,
     &ssh_hash_sha256_def,
     ssh_hmac_ctxsize, ssh_hmac_init,
     ssh_hmac_start, ssh_hmac_update, ssh_hmac_final,
     ssh_hmac_of_buffer },
-  { "hmac-sha256-96", 12, FALSE,
+  {
+
+
+#ifdef SSHDIST_SSH2
+    "hmac-sha256-96@ssh.com",
+#endif /* SSHDIST_SSH2 */
+    12, FALSE,
     &ssh_hash_sha256_96_def,
     ssh_hmac_ctxsize, ssh_hmac_init,
     ssh_hmac_start, ssh_hmac_update, ssh_hmac_96_final,
     ssh_hmac_96_of_buffer },
 #endif /* SSHDIST_CRYPT_SHA */
 #ifdef SSHDIST_CRYPT_RIPEMD160
-  { "hmac-ripemd160", 20, FALSE,
+  {
+
+
+#ifdef SSHDIST_SSH2
+    "hmac-ripemd160@ssh.com",
+#endif /* SSHDIST_SSH2 */
+    20, FALSE,
     &ssh_hash_ripemd160_def,
     ssh_hmac_ctxsize, ssh_hmac_init,
     ssh_hmac_start, ssh_hmac_update, ssh_hmac_final,
     ssh_hmac_of_buffer },
-  { "hmac-ripemd160-96", 12, FALSE,
+  {
+
+
+#ifdef SSHDIST_SSH2
+    "hmac-ripemd160-96@ssh.com",
+#endif /* SSHDIST_SSH2 */
+    12, FALSE,
     &ssh_hash_ripemd160_def,
     ssh_hmac_ctxsize, ssh_hmac_init,
     ssh_hmac_start, ssh_hmac_update, ssh_hmac_96_final,
@@ -396,52 +434,11 @@ static const SshMacDefStruct ssh_mac_algorithms[] =
 
 
 
-#endif /* !KERNEL */
 
-#endif /* SSHDIST_CRYPT_HMAC */
 
-#ifdef SSHDIST_CRYPT_SSHMACS
 
-#ifdef SSHDIST_CRYPT_SHA
-  { "sha1-8", 8, TRUE,
-    &ssh_hash_sha_def,
-    ssh_kdk_mac_ctxsize, ssh_kdk_mac_init,
-    ssh_kdk_mac_start, ssh_kdk_mac_update, ssh_kdk_mac_64_final,
-    ssh_kdk_mac_64_of_buffer },
-  { "sha1", 20, TRUE,
-    &ssh_hash_sha_def,
-    ssh_kdk_mac_ctxsize, ssh_kdk_mac_init,
-    ssh_kdk_mac_start, ssh_kdk_mac_update, ssh_kdk_mac_final,
-    ssh_kdk_mac_of_buffer },
-#endif /* SSHDIST_CRYPT_SHA */
-#ifdef SSHDIST_CRYPT_MD5
-  { "md5-8", 8, TRUE,
-    &ssh_hash_md5_def,
-    ssh_kdk_mac_ctxsize, ssh_kdk_mac_init,
-    ssh_kdk_mac_start, ssh_kdk_mac_update, ssh_kdk_mac_64_final,
-    ssh_kdk_mac_64_of_buffer },
-  { "md5", 16, TRUE,
-    &ssh_hash_md5_def,
-    ssh_kdk_mac_ctxsize, ssh_kdk_mac_init,
-    ssh_kdk_mac_start, ssh_kdk_mac_update, ssh_kdk_mac_final,
-    ssh_kdk_mac_of_buffer },
-#endif /* SSHDIST_CRYPT_MD5 */
-#ifdef SSHDIST_CRYPT_RIPEMD160
-  { "ripemd160-8", 8, TRUE,
-    &ssh_hash_ripemd160_def,
-    ssh_kdk_mac_ctxsize, ssh_kdk_mac_init,
-    ssh_kdk_mac_start, ssh_kdk_mac_update, ssh_kdk_mac_64_final,
-    ssh_kdk_mac_64_of_buffer },
-  { "ripemd160", 20, TRUE,
-    &ssh_hash_ripemd160_def,
-    ssh_kdk_mac_ctxsize, ssh_kdk_mac_init,
-    ssh_kdk_mac_start, ssh_kdk_mac_update, ssh_kdk_mac_final,
-    ssh_kdk_mac_of_buffer },
-#endif /* SSHDIST_CRYPT_RIPEMD160 */
 
-#ifndef KERNEL
-  /* These MACs can only be used in user-mode code.  See comments
-     above for more information. */
+
 
 
 
@@ -475,37 +472,115 @@ static const SshMacDefStruct ssh_mac_algorithms[] =
 
 #endif /* !KERNEL */
 
-#endif /* SSHDIST_CRYPT_SSHMACS */
+#endif /* SSHDIST_CRYPT_HMAC */
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
   { "none", 0, FALSE, NULL },
   { NULL }
 };
 
 static const SshCipherMacDefStruct ssh_cipher_mac_algorithms[] =
 {
-#ifdef SSHDIST_CRYPT_RIJNDAEL
-  { "cbcmac-aes", 16, ssh_rijndael_ctxsize, ssh_rijndael_init,
-    ssh_rijndael_cbc_mac },
-  { "cbcmac-rijndael", 16, ssh_rijndael_ctxsize, ssh_rijndael_init,
-    ssh_rijndael_cbc_mac },
-#endif /* SSHDIST_CRYPT_RIJNDAEL */
 
 
 
 
 
-#ifdef SSHDIST_CRYPT_BLOWFISH
-  { "cbcmac-blowfish", 8, ssh_blowfish_ctxsize, ssh_blowfish_init,
-    ssh_blowfish_cbc_mac },
-#endif /* SSHDIST_CRYPT_BLOWFISH */
-#ifdef SSHDIST_CRYPT_TWOFISH
-  { "cbcmac-twofish", 16, ssh_twofish_ctxsize, ssh_twofish_init,
-    ssh_twofish_cbc_mac },
-#endif /* SSHDIST_CRYPT_TWOFISH */
-#ifdef SSHDIST_CRYPT_DES
-  { "cbcmac-3des", 8, ssh_des3_ctxsize, ssh_des3_init,
-    ssh_des3_cbc_mac },
-  { "cbcmac-des", 8, ssh_des_ctxsize, ssh_des3_init, ssh_des_cbc_mac },
-#endif /* SSHDIST_CRYPT_DES */
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
   { NULL }
 };
 
@@ -625,6 +700,8 @@ ssh_mac_allocate(const char *type,
   int i;
   SshMac mac;
 
+  *mac_return = NULL;
+
   /* Find the desired mac type from the array. */
   for (i = 0; ssh_mac_algorithms[i].name != NULL; i++)
     {
@@ -748,26 +825,26 @@ ssh_mac_info_derive_from_hash(SshHash hash,
       break;
 #endif /* SSHDIST_CRYPT_HMAC */
 
-#ifdef SSHDIST_CRYPT_SSHMACS
-    case SSH_MAC_TYPE_KDK:
-      mac_def->allocate_key  = TRUE;
-      mac_def->init          = ssh_kdk_mac_init;
-      mac_def->start         = ssh_kdk_mac_start;
-      mac_def->update        = ssh_kdk_mac_update;
-      mac_def->final         = ssh_kdk_mac_final;
-      mac_def->mac_of_buffer = ssh_kdk_mac_of_buffer;
 
-      /* Build a name for it. */
-      ssh_snprintf(buffer, 128, "kdk-mac-%s", hash_def->name);
-      if (!(tmp = ssh_malloc(strlen(buffer) + 1)))
-        {
-          ssh_free(mac_def);
-          return NULL;
-        }
-      memcpy(tmp, buffer, strlen(buffer) + 1);
-      mac_def->name = tmp;
-      break;
-#endif /* SSHDIST_CRYPT_SSHMACS */
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
     default:
       ssh_free(mac_def);
