@@ -14,7 +14,7 @@
   */
 
 /*
- * $Id: dlfix.c,v 1.2 2001/08/15 11:13:41 kivinen Exp $
+ * $Id: dlfix.c,v 1.3 2001/11/13 13:42:59 irwin Exp $
  * $Log: dlfix.c,v $ * 
  * 
  * 
@@ -22,6 +22,10 @@
  * 
  * 
  * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -383,7 +387,7 @@ char *ssh_dlp_param_get_predefined_groups(void)
 }
 
 Boolean ssh_dlp_set_param(const char *name, const char **outname,
-                          SshMPInt p, SshMPInt q, SshMPInt g)
+                          SshMPInteger p, SshMPInteger q, SshMPInteger g)
 {
   int i;
 
@@ -400,10 +404,42 @@ Boolean ssh_dlp_set_param(const char *name, const char **outname,
 
   *outname = ssh_dlp_fixed_params[i].name;
 
-  ssh_mp_set_str(p, ssh_dlp_fixed_params[i].p, 0);
-  ssh_mp_set_str(q, ssh_dlp_fixed_params[i].q, 0);
-  ssh_mp_set_str(g, ssh_dlp_fixed_params[i].g, 0);
+  ssh_mprz_set_str(p, ssh_dlp_fixed_params[i].p, 0);
+  ssh_mprz_set_str(q, ssh_dlp_fixed_params[i].q, 0);
+  ssh_mprz_set_str(g, ssh_dlp_fixed_params[i].g, 0);
+  
+  return TRUE;
+}
 
+Boolean ssh_dlp_is_predefined_group(SshMPInteger p, SshMPInteger q, 
+                                    SshMPInteger g)
+{
+  unsigned int i;
+  SshMPIntegerStruct P, Q, G;
+  
+  ssh_mprz_init(&P);
+  ssh_mprz_init(&Q);
+  ssh_mprz_init(&G);
+  
+  for (i = 0; ssh_dlp_fixed_params[i].name; i++)
+    {  
+      ssh_mprz_set_str(&P, ssh_dlp_fixed_params[i].p, 0);
+      ssh_mprz_set_str(&Q, ssh_dlp_fixed_params[i].q, 0);
+      ssh_mprz_set_str(&G, ssh_dlp_fixed_params[i].g, 0);
+      
+      if ((ssh_mprz_cmp(p, &P) == 0) &&
+          (ssh_mprz_cmp(q, &Q) == 0) &&
+          (ssh_mprz_cmp(g, &G) == 0))     
+        break;
+    }
+  
+  ssh_mprz_clear(&P);
+  ssh_mprz_clear(&Q);
+  ssh_mprz_clear(&G);
+  
+  if (ssh_dlp_fixed_params[i].name == NULL)
+    return FALSE;
+  
   return TRUE;
 }
 
