@@ -4,7 +4,7 @@
 
   Author: Mika Kojo <mkojo@ssh.fi>
 
-  Copyright (C) 1997-2001 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997-2002 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
   Created: Mon Jun  2 18:43:45 1997 [mkojo]
@@ -14,7 +14,7 @@
 */
 
 /*
- * $Id: genpkcs.c,v 1.4.2.1 2001/11/12 06:24:35 sjl Exp $
+ * $Id: genpkcs.c,v 1.11 2002/02/28 11:16:19 kivinen Exp $
  *
  * $Log: genpkcs.c,v $ * 
  * 
@@ -39,6 +39,30 @@
  * 
  * 
  * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -146,6 +170,9 @@ void *ssh_pk_find_generic(const char *name, const void *list, size_t msize)
   const char *buf_name;
   unsigned int i;
 
+  if (name == NULL)
+    return NULL;
+
   /* buf[i] points to start of a structure (which size is msize), buf_name
      is set to the const char * from start of the buf[i]. */
   for (i = 0;
@@ -447,7 +474,7 @@ ssh_private_key_select_scheme(SshPrivateKey key, ...)
       scheme = ssh_pk_find_generic(name, action->type,
                                    action->type_size);
 
-      /* Quit an awful error! Means that our scheme tables are either
+      /* Awful error! Means that our scheme tables are either
          corrupted or application failed. */
       if (scheme == NULL)
         {
@@ -504,7 +531,11 @@ ssh_public_key_define(SshPublicKey *public_key,
       ssh_ntree_free(tree);
       return SSH_CRYPTO_UNKNOWN_KEY_TYPE;
     }
-  tmp = ssh_nnode_get_identifier(node);
+  if ((tmp = ssh_nnode_get_identifier(node)) == NULL)
+    {
+      ssh_ntree_free(tree);
+      return SSH_CRYPTO_NO_MEMORY;
+    }
 
   /* Find out the key type. */
   for (i = 0; ssh_pk_type_slots[i] != NULL && ssh_pk_type_slots[i]->name; i++)
@@ -571,7 +602,7 @@ ssh_public_key_define(SshPublicKey *public_key,
             }
 
           /* Call action_scheme if not set to NULL. */
-          if (((SshPkGen *)scheme)->action_scheme != NULL)
+          if (((SshPkGen *)scheme)->action_scheme != NULL_FNPTR)
             (*((SshPkGen *)scheme)->action_scheme)(context);
 
           /* Set the corresponding scheme to the group. */
@@ -637,7 +668,7 @@ ssh_public_key_define(SshPublicKey *public_key,
 
               /* Call the action_scheme function here if not
                  NULL. */
-              if (((SshPkGen *)scheme)->action_scheme != NULL)
+              if (((SshPkGen *)scheme)->action_scheme != NULL_FNPTR)
                 (*((SshPkGen *)scheme)->action_scheme)(context);
 
               /* Set the corresponding scheme. */
@@ -832,7 +863,7 @@ ssh_private_key_get_info(SshPrivateKey key, ...)
               return SSH_CRYPTO_UNSUPPORTED_IDENTIFIER;
             }
 
-          if (action->action_get == NULL)
+          if (action->action_get == NULL_FNPTR)
             return SSH_CRYPTO_UNSUPPORTED;
 
           r = (*action->action_get)(key->context, ap, NULL, format);
@@ -920,7 +951,7 @@ ssh_public_key_get_info(SshPublicKey key, ...)
               return SSH_CRYPTO_UNSUPPORTED_IDENTIFIER;
             }
 
-          if (action->action_get == NULL)
+          if (action->action_get == NULL_FNPTR)
             return SSH_CRYPTO_UNSUPPORTED;
 
           r = (*action->action_get)(key->context, ap, NULL, format);
@@ -1023,7 +1054,7 @@ ssh_public_key_copy(SshPublicKey key_src, SshPublicKey *key_dest)
 {
   SshPublicKey created;
 
-  if (key_src->type->public_key_copy == NULL)
+  if (key_src->type->public_key_copy == NULL_FNPTR)
     return SSH_CRYPTO_UNSUPPORTED;
 
   if ((created = ssh_malloc(sizeof(*created))) != NULL)
@@ -1057,7 +1088,7 @@ ssh_public_key_derive_pk_group(SshPublicKey key)
 {
   SshPkGroup group = NULL;
 
-  if (key->type->public_key_derive_pk_group == NULL)
+  if (key->type->public_key_derive_pk_group == NULL_FNPTR)
     return NULL;
 
   if ((group = ssh_malloc(sizeof(*group))) != NULL)
@@ -1078,7 +1109,7 @@ ssh_public_key_max_encrypt_input_len(SshPublicKey key)
   if (key->encryption == NULL)
     return 0;
 
-  if (key->encryption->public_key_max_encrypt_input_len == NULL)
+  if (key->encryption->public_key_max_encrypt_input_len == NULL_FNPTR)
     return 0;
 
   return (*key->encryption->public_key_max_encrypt_input_len)(key->context);
@@ -1102,7 +1133,7 @@ ssh_private_key_copy(SshPrivateKey key_src, SshPrivateKey *key_dest)
 {
   SshPrivateKey created;
 
-  if (key_src->type->private_key_copy == NULL)
+  if (key_src->type->private_key_copy == NULL_FNPTR)
     return SSH_CRYPTO_UNSUPPORTED;
 
   if ((created = ssh_malloc(sizeof(*created))) != NULL)
@@ -1135,7 +1166,7 @@ ssh_private_key_derive_public_key(SshPrivateKey key)
   SshPublicKey pub;
   void *pub_context;
 
-  if (key->type->private_key_derive_public_key == NULL)
+  if (key->type->private_key_derive_public_key == NULL_FNPTR)
     return NULL;
 
   (*key->type->private_key_derive_public_key)(key->context, &pub_context);
@@ -1160,7 +1191,7 @@ ssh_private_key_derive_pk_group(SshPrivateKey key)
 {
   SshPkGroup group;
 
-  if (key->type->private_key_derive_pk_group == NULL)
+  if (key->type->private_key_derive_pk_group == NULL_FNPTR)
     return NULL;
 
   if ((group = ssh_malloc(sizeof(*group))) != NULL)
@@ -1204,7 +1235,7 @@ ssh_public_key_encrypt(SshPublicKey key,
   SshRGFHash hash;
 
   if (key->encryption == NULL ||
-      key->encryption->public_key_encrypt == NULL)
+      key->encryption->public_key_encrypt == NULL_FNPTR)
     return SSH_CRYPTO_UNSUPPORTED;
 
   hash = ssh_rgf_hash_allocate(key->encryption->rgf_def);
@@ -1218,7 +1249,6 @@ ssh_public_key_encrypt(SshPublicKey key,
                                              hash))
     return SSH_CRYPTO_OK;
 
-  ssh_rgf_hash_free(hash);
   return SSH_CRYPTO_OPERATION_FAILED;
 }
 
@@ -1231,7 +1261,7 @@ ssh_public_key_decrypt(SshPublicKey key,
                        size_t *plaintext_len_return)
 {
   if (key->encryption == NULL ||
-      key->encryption->public_key_decrypt == NULL)
+      key->encryption->public_key_decrypt == NULL_FNPTR)
     return SSH_CRYPTO_UNSUPPORTED;
 
   /* If true then encryption succeeded. */
@@ -1264,7 +1294,7 @@ ssh_public_key_verify_signature(SshPublicKey key,
 
   if (key == NULL ||
       key->signature == NULL ||
-      key->signature->public_key_verify == NULL)
+      key->signature->public_key_verify == NULL_FNPTR)
     return FALSE;
 
   /* We need to compute the RGFHash ourselves. */
@@ -1287,7 +1317,8 @@ ssh_public_key_verify_signature_with_digest(SshPublicKey key,
 {
   SshRGFHash hash;
 
-  if (key->signature == NULL || key->signature->public_key_verify == NULL)
+  if (key->signature == NULL ||
+      key->signature->public_key_verify == NULL_FNPTR)
     return FALSE;
 
   hash = ssh_rgf_hash_allocate(key->signature->rgf_def);
@@ -1311,7 +1342,8 @@ ssh_public_key_verify_signature_with_hash(SshPublicKey key,
                                           size_t signature_len,
                                           SshRGFHash hash)
 {
-  if (key->signature == NULL || key->signature->public_key_verify == NULL)
+  if (key->signature == NULL ||
+      key->signature->public_key_verify == NULL_FNPTR)
     return FALSE;
 
   return (*key->signature->public_key_verify)(key->context,
@@ -1325,7 +1357,7 @@ size_t
 ssh_private_key_max_signature_input_len(SshPrivateKey key)
 {
   if (key->signature == NULL ||
-      key->signature->private_key_max_signature_input_len == NULL)
+      key->signature->private_key_max_signature_input_len == NULL_FNPTR)
     return 0;
 
   return (*key->signature->private_key_max_signature_input_len)(key->context);
@@ -1335,7 +1367,7 @@ size_t
 ssh_private_key_max_signature_output_len(SshPrivateKey key)
 {
   if (key->signature == NULL ||
-      key->signature->private_key_max_signature_output_len == NULL)
+      key->signature->private_key_max_signature_output_len == NULL_FNPTR)
     return 0;
   return (*key->signature->private_key_max_signature_output_len)(key->context);
 }
@@ -1348,7 +1380,7 @@ size_t
 ssh_private_key_max_decrypt_input_len(SshPrivateKey key)
 {
   if (key->encryption == NULL ||
-      key->encryption->private_key_max_decrypt_input_len == NULL)
+      key->encryption->private_key_max_decrypt_input_len == NULL_FNPTR)
     return 0;
 
   return (*key->encryption->private_key_max_decrypt_input_len)(key->context);
@@ -1361,7 +1393,7 @@ size_t
 ssh_private_key_max_decrypt_output_len(SshPrivateKey key)
 {
   if (key->encryption == NULL ||
-      key->encryption->private_key_max_decrypt_output_len == NULL)
+      key->encryption->private_key_max_decrypt_output_len == NULL_FNPTR)
     return 0;
 
   return (*key->encryption->private_key_max_decrypt_output_len)(key->context);
@@ -1379,7 +1411,7 @@ ssh_private_key_decrypt(SshPrivateKey key,
   SshRGFHash hash;
 
   if (key->encryption == NULL ||
-      key->encryption->private_key_decrypt == NULL)
+      key->encryption->private_key_decrypt == NULL_FNPTR)
     return SSH_CRYPTO_UNSUPPORTED;
 
   hash = ssh_rgf_hash_allocate(key->encryption->rgf_def);
@@ -1406,7 +1438,7 @@ ssh_private_key_encrypt(SshPrivateKey key,
                         size_t *ciphertext_length_return)
 {
   if (key->encryption == NULL ||
-      key->encryption->private_key_encrypt == NULL)
+      key->encryption->private_key_encrypt == NULL_FNPTR)
     return SSH_CRYPTO_UNSUPPORTED;
 
   if ((*key->encryption->private_key_encrypt)(key->context,
@@ -1431,7 +1463,7 @@ ssh_private_key_sign(SshPrivateKey key,
 {
   SshRGFHash hash;
   if (key->signature == NULL ||
-      key->signature->private_key_sign == NULL)
+      key->signature->private_key_sign == NULL_FNPTR)
     return SSH_CRYPTO_UNSUPPORTED;
 
   hash = ssh_rgf_hash_allocate(key->signature->rgf_def);
@@ -1459,7 +1491,7 @@ ssh_private_key_sign_digest(SshPrivateKey key,
 {
   SshRGFHash hash;
   if (key->signature == NULL ||
-      key->signature->private_key_sign == NULL)
+      key->signature->private_key_sign == NULL_FNPTR)
     return SSH_CRYPTO_UNSUPPORTED;
 
   hash = ssh_rgf_hash_allocate(key->signature->rgf_def);
@@ -1490,7 +1522,7 @@ ssh_private_key_sign_hash(SshPrivateKey key,
                           size_t *signature_length_return)
 {
   if (key->signature == NULL ||
-      key->signature->private_key_sign == NULL)
+      key->signature->private_key_sign == NULL_FNPTR)
     return SSH_CRYPTO_UNSUPPORTED;
 
   if ((*key->signature->private_key_sign)(key->context,
@@ -1509,7 +1541,7 @@ size_t
 ssh_pk_group_dh_setup_max_output_length(SshPkGroup group)
 {
   if (group->diffie_hellman == NULL ||
-      group->diffie_hellman->diffie_hellman_exchange_max_length == NULL)
+      group->diffie_hellman->diffie_hellman_exchange_max_length == NULL_FNPTR)
     return 0;
 
   return (*group->diffie_hellman->
@@ -1520,7 +1552,7 @@ size_t
 ssh_pk_group_dh_agree_max_output_length(SshPkGroup group)
 {
   if (group->diffie_hellman == NULL ||
-      group->diffie_hellman->diffie_hellman_secret_value_max_length == NULL)
+      group->diffie_hellman->diffie_hellman_secret_value_max_length == NULL_FNPTR)
     return 0;
 
   return (*group->diffie_hellman->
@@ -1543,7 +1575,7 @@ ssh_pk_group_dh_setup(SshPkGroup group,
                       size_t *return_length)
 {
   if (group->diffie_hellman == NULL ||
-      group->diffie_hellman->diffie_hellman_setup == NULL)
+      group->diffie_hellman->diffie_hellman_setup == NULL_FNPTR)
     return SSH_CRYPTO_UNSUPPORTED;
 
   if ((*group->diffie_hellman->diffie_hellman_setup)(group->context,
@@ -1559,14 +1591,14 @@ ssh_pk_group_dh_setup(SshPkGroup group,
 SshCryptoStatus
 ssh_pk_group_dh_agree(SshPkGroup group,
                       SshPkGroupDHSecret secret,
-                      unsigned char *exchange,
+                      const unsigned char *exchange,
                       size_t exchange_length,
                       unsigned char *secret_value_buffer,
                       size_t secret_value_buffer_length,
                       size_t *return_length)
 {
   if (group->diffie_hellman == NULL ||
-      group->diffie_hellman->diffie_hellman_agree == NULL)
+      group->diffie_hellman->diffie_hellman_agree == NULL_FNPTR)
     return SSH_CRYPTO_UNSUPPORTED;
 
   if ((*group->diffie_hellman->
@@ -1587,7 +1619,7 @@ size_t
 ssh_pk_group_udh_agree_max_output_length(SshPkGroup group)
 {
   if (group->diffie_hellman == NULL ||
-      group->diffie_hellman->udh_secret_value_max_length == NULL)
+      group->diffie_hellman->udh_secret_value_max_length == NULL_FNPTR)
     return 0;
   return (*group->diffie_hellman->
           udh_secret_value_max_length)(group->context);
@@ -1597,7 +1629,7 @@ SshCryptoStatus
 ssh_pk_group_udh_agree(SshPublicKey public_key,
                        SshPrivateKey private_key,
                        void *secret,
-                       unsigned char *exchange,
+                       const unsigned char *exchange,
                        size_t exchange_length,
                        unsigned char *secret_value_buffer,
                        size_t secret_value_buffer_length,
@@ -1605,7 +1637,7 @@ ssh_pk_group_udh_agree(SshPublicKey public_key,
 {
   if (private_key->diffie_hellman == NULL ||
       public_key->diffie_hellman == NULL ||
-      private_key->diffie_hellman->udh_agree == NULL)
+      private_key->diffie_hellman->udh_agree == NULL_FNPTR)
     return SSH_CRYPTO_UNSUPPORTED;
 
   if ((*private_key->diffie_hellman->
@@ -1686,7 +1718,7 @@ ssh_public_key_decrypt_async(SshPublicKey key,
   SshCryptoStatus status;
 
   if (key->encryption &&
-      key->encryption->public_key_decrypt_async != NULL)
+      key->encryption->public_key_decrypt_async != NULL_FNPTR)
     {
       /* Asyncronous operation. */
       return (*key->encryption->
@@ -1860,7 +1892,7 @@ ssh_private_key_decrypt_async(SshPrivateKey key,
   SshRGFHash hash;
 
   if (key->encryption &&
-      (key->encryption->private_key_decrypt_async != NULL) &&
+      (key->encryption->private_key_decrypt_async != NULL_FNPTR) &&
       (hash = ssh_rgf_hash_allocate(key->encryption->rgf_def)) != NULL)
     {
       /* Asyncronous operation. */
@@ -1942,7 +1974,7 @@ ssh_private_key_sign_async(SshPrivateKey key,
   SshCryptoStatus status;
 
   if (key->signature &&
-      key->signature->private_key_sign_async != NULL)
+      key->signature->private_key_sign_async != NULL_FNPTR)
     {
       SshRGFHash hash;
       hash = ssh_rgf_hash_allocate(key->signature->rgf_def);
@@ -1989,7 +2021,7 @@ ssh_private_key_sign_digest_async(SshPrivateKey key,
   SshCryptoStatus status;
 
   if (key->signature &&
-      key->signature->private_key_sign_async != NULL)
+      key->signature->private_key_sign_async != NULL_FNPTR)
     {
       SshRGFHash hash;
       hash = ssh_rgf_hash_allocate(key->signature->rgf_def);
@@ -2038,7 +2070,7 @@ ssh_private_key_sign_hash_async(SshPrivateKey key,
   SshCryptoStatus status;
 
   if (key->signature &&
-      key->signature->private_key_sign_async != NULL)
+      key->signature->private_key_sign_async != NULL_FNPTR)
     {
       /* Asyncronous operation. */
       return (*key->signature->
@@ -2103,7 +2135,7 @@ ssh_pk_group_dh_setup_async(SshPkGroup group,
 SshOperationHandle
 ssh_pk_group_dh_agree_async(SshPkGroup group,
                             SshPkGroupDHSecret secret,
-                            unsigned char *exchange,
+                            const unsigned char *exchange,
                             size_t exchange_length,
                             SshPkGroupDHAgree callback,
                             void *context)
@@ -2153,7 +2185,7 @@ SshOperationHandle
 ssh_pk_group_udh_agree_async(SshPublicKey public_key,
                              SshPrivateKey private_key,
                              SshPkGroupDHSecret secret,
-                             unsigned char *exchange,
+                             const unsigned char *exchange,
                              size_t exchange_length,
                              SshPkGroupDHAgree callback,
                              void *context)
