@@ -30,7 +30,7 @@ ssh_pk_group_export_randomizers(SshPkGroup group,
   size_t tmplen;
   SshCryptoStatus status = SSH_CRYPTO_OPERATION_FAILED;
 
-  if (group->type->pk_group_export_randomizer == NULL)
+  if (group->type->pk_group_export_randomizer == NULL_FNPTR)
     return SSH_CRYPTO_UNSUPPORTED;
 
   ssh_buffer_init(&buffer);
@@ -44,7 +44,7 @@ ssh_pk_group_export_randomizers(SshPkGroup group,
     {
       while (TRUE)
         {
-          if ((*group->type->pk_group_export_randomizer)(group, &tmp, &tmplen))
+          if ((*group->type->pk_group_export_randomizer)(group->context, &tmp, &tmplen))
             {
               if (ssh_encode_buffer(&buffer,
                                     SSH_FORMAT_UINT32_STR, tmp, tmplen,
@@ -78,7 +78,7 @@ ssh_pk_group_import_randomizers(SshPkGroup group,
   SshUInt32 magic;
   int buf_consumed = 0, len_decoded = 0;
 
-  if (group->type->pk_group_import_randomizer == NULL)
+  if (group->type->pk_group_import_randomizer == NULL_FNPTR)
     return SSH_CRYPTO_UNSUPPORTED;
 
   if ((buf_consumed = ssh_decode_array(buf, buf_length,
@@ -140,7 +140,7 @@ ssh_pk_group_export(SshPkGroup group,
   char *name;
   SshCryptoStatus status = SSH_CRYPTO_OPERATION_FAILED;
 
-  if (group->type->pk_group_export == NULL)
+  if (group->type->pk_group_export == NULL_FNPTR)
     return SSH_CRYPTO_UNSUPPORTED;
 
   ssh_buffer_init(&buffer);
@@ -340,7 +340,7 @@ ssh_public_key_export(SshPublicKey key,
   char *name;
   SshCryptoStatus status = SSH_CRYPTO_OPERATION_FAILED;
 
-  if (key->type->public_key_export == NULL)
+  if (key->type->public_key_export == NULL_FNPTR)
     return SSH_CRYPTO_UNSUPPORTED;
 
   ssh_buffer_init(&buffer);
@@ -397,7 +397,7 @@ ssh_public_key_export_canonical(SshPublicKey key,
   size_t tmplen;
   SshCryptoStatus status = SSH_CRYPTO_OPERATION_FAILED;
 
-  if (key->type->public_key_export == NULL)
+  if (key->type->public_key_export == NULL_FNPTR)
       return SSH_CRYPTO_UNSUPPORTED;
 
   /* Encoding of the public key, in SSH format. */
@@ -496,7 +496,7 @@ ssh_private_key_export_internal(SshPrivateKey key,
 
   /* Generate private key blob. */
 
-  if (key->type->private_key_export == NULL ||
+  if (key->type->private_key_export == NULL_FNPTR ||
       (*key->type->private_key_export)(key->context,
                                        &buf, &buf_length) == FALSE)
     {
@@ -600,7 +600,7 @@ ssh_private_key_import_internal(const unsigned char *buf, size_t len,
                         SSH_FORMAT_END) == 0)
     {
       ssh_buffer_uninit(&buffer);
-      return status;
+      return SSH_CRYPTO_CORRUPTED_KEY_FORMAT;
     }
 
   if (pk_magic != SSH_PRIVATE_KEY_MAGIC || pk_length < 8)
@@ -782,7 +782,7 @@ ssh_private_key_import_internal(const unsigned char *buf, size_t len,
       return SSH_CRYPTO_OPERATION_FAILED;
     }
 
-  if (private_key->type->private_key_import == NULL ||
+  if (private_key->type->private_key_import == NULL_FNPTR ||
       !(*private_key->type->private_key_import)(ssh_buffer_ptr(&buffer),
                                                 tmp_length,
                                                 &(private_key->context)))
@@ -1013,7 +1013,7 @@ ssh_public_key_import(const unsigned char *buf, size_t len,
       return SSH_CRYPTO_OPERATION_FAILED;
     }
 
-  if (public_key->type->public_key_import == NULL ||
+  if (public_key->type->public_key_import == NULL_FNPTR ||
       !(*public_key->type->public_key_import)(ssh_buffer_ptr(&buffer),
                                               length,
                                               &(public_key->context)))
