@@ -655,7 +655,7 @@ void pkcs_random_tests(void)
   char *pkcs_name = NULL;
   int i, cnt;
   size_t len;
-  static int use_randomizers = 0;
+  static int use_randomizers = 1;
   int size = 1024;
   char *passphrase = "ssh-communications-security-finland-passphrase";
   char *cipher_name = "blowfish-cfb";
@@ -932,7 +932,8 @@ void pkcs_random_tests(void)
               ssh_private_key_max_decrypt_output_len(private_key);
             data_buf2 = ssh_xmalloc(data_buf2_len);
 
-            for(i = 0; i < data_buf_len; i++)
+            data_buf[0] = 0;
+            for(i = 1; i < data_buf_len; i++)
               data_buf[i] = ssh_random_get_byte();
 
             tstart(&tmit, "encrypt & decrypt (public): %s", p);
@@ -959,7 +960,7 @@ void pkcs_random_tests(void)
               ssh_fatal("Invalid length returned from the ssh_private_key_decrypt");
 
             if (memcmp(data_buf, data_buf2, data_buf_len) != 0)
-              ssh_warning("Encryption failed");
+              ssh_fatal("Encryption failed");
 
             tstart(&tmit, "encrypt & decrypt (private): %s", p);
             status = ssh_private_key_encrypt(private_key,
@@ -984,7 +985,7 @@ void pkcs_random_tests(void)
                   ssh_fatal("Invalid length returned from the ssh_public_key_decrypt");
 
                 if (memcmp(data_buf, data_buf2, data_buf_len) != 0)
-                  ssh_warning("Encryption failed");
+                  ssh_fatal("Encryption failed");
               }
             else if (status == SSH_CRYPTO_UNSUPPORTED)
               {
@@ -1571,12 +1572,142 @@ void pkcs_random_tests(void)
     }
 
   /* Remove progress monitoring functions from use. */
-  ssh_crypto_library_register_progress_func(NULL, NULL);
+  ssh_crypto_library_register_progress_func(NULL_FNPTR, NULL);
 
-  use_randomizers = 1 - use_randomizers;
   ssh_xfree(namelist);
 }
 
+/* This function runs some basic tests on the Diffie-Hellman predefined groups. The
+   Diffie-Hellman computations tests are found in t-dh.c. Only testing dl-modp groups. 
+   Elliptic curves groups can be tested later if anyone feels that such a test would be 
+   worthwhile. */
+void predefined_groups_tests(void)
+{
+  char *namelist = ssh_public_key_get_predefined_groups("dl-modp");
+  const char *tmp_namelist = namelist;
+  SshPkGroup group;
+  char *group_name = NULL;
+  /* Test pointers. */
+  unsigned char *a;
+  size_t a_len;
+  int i, status, size;
+
+  /* Register a progress monitoring function. */
+  ssh_crypto_library_register_progress_func(my_progress_func,
+                                            NULL);
+
+  while ((group_name = ssh_name_list_get_name(tmp_namelist)) != NULL)
+    {
+      printf("Testing predefined group %s.\n", group_name);
+      /* Allocation of the predefined group. */
+      {
+        char *type;
+        SshMPIntegerStruct p, q, g;
+        int ent, entropy;
+
+        /* Put the randomizer entropy to 200. */
+        ent = 200;
+        status = ssh_pk_group_generate(&group, "dl-modp",
+                                       SSH_PKF_PREDEFINED_GROUP, group_name,
+                                       SSH_PKF_RANDOMIZER_ENTROPY, ent,
+                                       SSH_PKF_END);
+
+        if (status) ssh_fatal("can not generate the group %s.\n", group_name);
+
+        ssh_mprz_init(&p);
+        ssh_mprz_init(&q);
+        ssh_mprz_init(&g);
+        
+        status = ssh_pk_group_get_info(group,
+                                       SSH_PKF_KEY_TYPE, &type,
+                                       SSH_PKF_SIZE, &size,
+                                       SSH_PKF_PRIME_Q, &q,
+                                       SSH_PKF_PRIME_P, &p, 
+                                       SSH_PKF_GENERATOR_G, &g, 
+                                       SSH_PKF_RANDOMIZER_ENTROPY, &entropy,
+                                       SSH_PKF_END); 
+        
+        if (status) ssh_fatal("can not get the group info");
+
+        if (group == NULL) ssh_fatal("the generated group is NULL");    
+        if (size == 0) ssh_fatal("the group has zero elements");
+        
+        ssh_mp_clear(&p);
+        ssh_mp_clear(&q);
+        ssh_mp_clear(&g);
+      }
+      
+      if (verbose) printf("Testing the group import/export. \n");
+      
+      if (ssh_pk_group_export(group, &a, &a_len) != SSH_CRYPTO_OK)
+        ssh_fatal("error: cannot export public key group.");
+      
+      ssh_pk_group_free(group);
+      
+      if (ssh_pk_group_import(a, a_len, &group) != SSH_CRYPTO_OK)
+        ssh_fatal("error: cannot import public key group.");
+
+      ssh_xfree(a);
+      if (verbose) printf("Import/export test succeeded.\n");
+
+      /* Randomizers. */
+      if (verbose) printf("Generating randomizers.\n");
+      for (i = 0; i < 20; i++)
+        {       
+          status = ssh_pk_group_generate_randomizer(group);
+          if (status) ssh_fatal("Cannot generate randomizers");
+        }
+      if (ssh_pk_group_count_randomizers(group) != 20)
+        ssh_fatal("error: generated incorrect amount of randomizers.");
+     
+      if (verbose) printf("Exporting randomizers.\n");
+   
+      if (ssh_pk_group_export_randomizers(group, &a, &a_len) != SSH_CRYPTO_OK)
+          ssh_fatal("error: cannot export randomizers.");
+   
+      if (verbose) printf("Importing randomizers.\n");
+
+      if (ssh_pk_group_import_randomizers(group, a, a_len)
+          != SSH_CRYPTO_OK)
+        ssh_fatal("error: cannot import randomizers.");
+          
+      ssh_xfree(a);
+      if (verbose) printf("Randomizer test succeeded.\n");
+
+        /* Do precomputation. */
+      if (verbose) printf("Precomputation tests.\n");
+
+      status = ssh_pk_group_precompute(group);
+
+      if (status) 
+        {printf("status is %d\n" , status);
+        ssh_fatal("Error in precomputation");
+        }
+      if (verbose) printf("Precomputation test succeeded.\n");
+      
+      /* Diffie-Hellman tests are done in t-dh.c, here just check we can set the scheme. */
+
+      status = ssh_pk_group_select_scheme(group, SSH_PKF_DH, "plain", SSH_PKF_END);
+      if (status) 
+        ssh_fatal("Cannot set the scheme"); 
+
+      ssh_pk_group_free(group);
+      ssh_xfree(group_name);
+      tmp_namelist = ssh_name_list_step_forward(tmp_namelist);
+   
+      /* The IKE 3072, 4096 and 8192 groups are too slow, not really necessary to test 
+         anyway, since nobody should consider using them at present. */
+      if (size >= 2048)
+        break;
+    }
+  
+  /* Remove progress monitoring functions from use. */
+  ssh_crypto_library_register_progress_func(NULL_FNPTR, NULL);
+  
+  ssh_xfree(namelist);
+}
+
+
 void test_pkcs(int flag)
 {
   printf(" - selected tests (with timing).\n");
@@ -1584,5 +1715,9 @@ void test_pkcs(int flag)
 
   printf(" - random tests (with timing).\n");
   pkcs_random_tests();
+
+  printf(" - Test the predefined Diffie-Hellman groups.\n");
+  predefined_groups_tests();
+
 }
 #endif /* SSHDIST_CRYPT_GENPKCS */
