@@ -4,7 +4,7 @@
 
   Author: Mika Kojo <mkojo@ssh.fi>
 
-  Copyright (c) 2001 SSH Communications Security Corp
+  Copyright (c) 2001, 2002 SSH Communications Security Corp
   All rights reserved.
 
   Created Thu Mar  8 04:30:42 2001.
@@ -19,7 +19,7 @@ char *ssh_mprz_get_str_compat(char *str, unsigned int base,
 {
   char *out;
   out = ssh_mprz_get_str(op, base);
-  if (str)
+  if (str && out)
     {
       strcpy(str, out);
       ssh_xfree(out);
@@ -27,4 +27,3 @@ char *ssh_mprz_get_str_compat(char *str, unsigned int base,
     }
   return out;
 }
-
