@@ -4,17 +4,23 @@
 
   Author: Mika Kojo <mkojo@ssh.fi>
 
-  Copyright (c) 1999 SSH Communications Security, Finland
+  Copyright (c) 1999, 2001 SSH Communications Security, Finland
   All rights reserved.
 
   Created Tue Apr  6 20:12:58 1999.
 
-  The kernel functions of the SSH mathematics library. 
+  The kernel functions of the SSH mathematics library.
 
   */
 
-#ifndef SSHMATH_KERNEL
-#define SSHMATH_KERNEL
+#ifndef SSHMP_KERNEL
+#define SSHMP_KERNEL
+
+#include "sshmp-types.h"
+
+#ifdef HAVE_MATH_H
+#include "math.h"
+#endif /* HAVE_MATH_H */
 
 /* We define here all the macros that are needed by the SSH mathematics
    library. */
@@ -23,7 +29,7 @@
 #ifdef SSHMATH_I386
 #ifdef WINDOWS
 
-#include "winasmmp.h"
+#include "sshmp-asmopt_i386-windows.h"
 /* Rename the macros for this library. */
 #define SSH_MPK_COUNT_TRAILING_ZEROS(count,x) \
   ssh_winasm_count_trailing_zeros(count,x)
@@ -52,19 +58,19 @@ __asm__("bsfl %1,%0" : \
 __asm__("mull %3"                 \
         : "=a" ((SshWord)v), \
           "=d" ((SshWord)u)  \
-        : "%0" ((SshWord)a), \
+        : "0" ((SshWord)a), \
           "rm" ((SshWord)b))
 
 #define SSH_MPK_LONG_SQUARE(u,v,a) \
-  SSH_MPK_LONG_MUL(u,v,a,b);
-     
+  SSH_MPK_LONG_MUL(u,v,a,a);
+
 /* Fast division. */
 #define SSH_MPK_LONG_DIV(q, r, d1, d0, d)  \
 __asm__("divl %4"                  \
         : "=a" ((SshWord)q),  \
           "=d" ((SshWord)r)   \
-        : "%0"  ((SshWord)d0), \
-          "%1"  ((SshWord)d1), \
+        : "0"  ((SshWord)d0), \
+          "1"  ((SshWord)d1), \
           "rm" ((SshWord)d))
 #endif /* WINDOWS */
 #endif /* SSHMATH_I386 */
@@ -84,13 +90,13 @@ __asm__("divl %4"                  \
   w0 = n1*n0; \
 }
 #define SSH_MPK_LONG_SQUARE(w1,w0,a) SSH_MPK_LONG_MUL(w1,w0,a,a)
-                            
+
 #elif defined(SSHMATH_ALPHA_DEC_CC_ASM)
 /* DEC compiler. See the file "c_asm.h" for details of Alpha
    inline assembler. (There is an example for this very macro!)
 
    It says that DEC C++ versions before V5.3 do not support this. One should
-   have a check for this somewhere. 
+   have a check for this somewhere.
    */
 #include <c_asm.h>
 #define SSH_MPK_LONG_MUL(w1,w0,n1,n0) \
@@ -101,7 +107,7 @@ __asm__("divl %4"                  \
 #define SSH_MPK_LONG_SQUARE(w1,w0,a) SSH_MPK_LONG_MUL(w1,w0,a,a)
 #endif /* ! __GNUC__ */
 #endif /* SSHMATH_ALPHA */
-     
+
 #endif /* SSHMATH_ASSEMBLER_MACROS */
 
 /* The general purpose macros. */
@@ -116,7 +122,7 @@ __asm__("divl %4"                  \
 
 #ifndef SSH_MPK_COUNT_LEADING_ZEROS
 #define SSH_MPK_COUNT_LEADING_ZEROS(count,x) \
-  do { count = ssh_mpk_count_leading_zeros(x); } while(0)   
+  do { count = ssh_mpk_count_leading_zeros(x); } while(0)
 #endif /* SSH_MPK_COUNT_LEADING_ZEROS */
 
 #ifndef SSH_MPK_COUNT_TRAILING_ZEROS
@@ -131,7 +137,7 @@ __asm__("divl %4"                  \
    multiplication is faster by noticeable amount. Nevertheless, other
    platforms likely will benefit from the choice between algorithms. */
 #ifdef SSHMATH_MUL_IS_FAST
-     
+
 /* Multiply the two SshWords `n1' and `n0' and store the result to
    `(w1, w0)'. `w1' is the more significant word and `w0' the less
    significant.
@@ -141,7 +147,7 @@ __asm__("divl %4"                  \
 
          d c
        * b a
-       
+
    calculate e = ca, f = db, g = cb, h = ad and sum up: (a-d are
    half-words, e-h are full words. e^ is the upper half and e_ the
    lower):
@@ -163,7 +169,7 @@ __asm__("divl %4"                  \
         h^ h_
 
    If SshWord is `n' bits long, then
-    
+
         g, h <= (2^(n/2) - 1)(2^(n/2) - 1) = 2^n - 2^(n/2 + 1) + 1.
 
    Obviously e^ <= (2^(n/2) - 1). Therefore the sum, S, is bounded by
@@ -194,7 +200,7 @@ __asm__("divl %4"                  \
    complications.]
 
    */
-     
+
 #define SSH_MPK_LONG_MUL(w1, w0, n1, n0) \
 { \
   SshWord __a, __b, __c, __d, __e, __f; \
@@ -218,7 +224,7 @@ __asm__("divl %4"                  \
    Karatsuba's idea. We describe the details of the algorithm.
 
    Note also that Henri Cohen discusses the generalization of this method
-   in his book. 
+   in his book.
 
    Basic idea here is to compute w = n1*n0, 0 < n1, n0 <= W - 1, in parts as
 
@@ -273,7 +279,7 @@ __asm__("divl %4"                  \
    to the speed of addition. However, on most platforms this method is
    likely to be the fastest. It is unlikely that much faster methods
    are available.
-   
+
    */
 
 #define SSH_MPK_LONG_MUL(w1,w0,n1,n0) \
@@ -332,7 +338,7 @@ __asm__("divl %4"                  \
    There is one noteworthy distinction. This algorithm can be generalized
    to squaring of arbitrary size integers. The above one needs general
    multiplication for support, however, this method is an asymptotically
-   fast squaring method. 
+   fast squaring method.
    */
 
 #define SSH_MPK_LONG_SQUARE(w1,w0,a) \
@@ -376,11 +382,11 @@ __asm__("divl %4"                  \
 
    The method is derived from the standard elementary-school method
    for long division. The normalization idea comes from Knuth.
-   
+
    The method is the following: Let (n1, n0) = (h3, h2, h1, h0),
    i.e. `n1' contains the half-words `h3' and `h2', and `n0' contains
    the half-words `h1' and `h0'. Let `d' = `(d1, d0)' similarly.
-                   
+
    We may assume n1 < d because q must fit into a single word.
    Standard long division would first divide (h3, h2, h1) by (d1, d0)
    and obtain the higher word of the quotient:
@@ -422,7 +428,7 @@ __asm__("divl %4"                  \
 
    The next thing to do normally is to subtract
    q1 (d1, d0) from (h3, h2, h1):
-   
+
    :                         q1
    :                 _______________
    :           d1 d0 | h3 h2 h1 h0
@@ -431,7 +437,7 @@ __asm__("divl %4"                  \
 
    and then continue division by dividing (r2, r1, h0).  In other
    words, to calculate (h3, h2, h1) % (d1, d0).
-   
+
    Here, calculate an approximation of this value as
 
      (h3, h2, 0) % (d1, 0) + (0, h1) - (d0 * q1') = (r2', r1')
@@ -439,17 +445,17 @@ __asm__("divl %4"                  \
    Which is the same as
 
      (h3, h2, h1) - q1' (d1, d0)
-   
+
    and thus corresponds to the approximation of q1 that we computed.
-   
+
    The next thing is to fix the remainder and the quotient in the case
    the approximation is wrong. This is easy. Because q1 is the correct
    quotient,
 
      (h3, h2, h1) - q1 (d1, d0) < (d1, d0).
-   
+
    Therefore, if q1' > q1,
-   
+
      (h3, h2, h1) - q1' (d1, d0) < 0.  (1)
 
    Thus while (h3, h2, h1) - q1' (d1, d0) < 0, decrement q1' by one
@@ -463,7 +469,7 @@ __asm__("divl %4"                  \
    :                          q1 ??
    :                     __________
    :               d1 d0 | r2 r1 h0
- 
+
    This is done, of course, identically.
 
    III. Implementation
@@ -472,7 +478,7 @@ __asm__("divl %4"                  \
 
    __a  :  d1
    __b  :  d0
-   
+
    __c  :  q1' and later q0'
      d  :  divider
    __g  :  a temporary variable that contains the values of
@@ -504,8 +510,8 @@ __asm__("divl %4"                  \
 
    now
 
-     (n1*2^w + n0) = b1*(2^w - 1) + b0  
-     
+     (n1*2^w + n0) = b1*(2^w - 1) + b0
+
      (n1 - b1)*2^w + (n0 + b1) = b0 < (2^w-1).
 
    We may write b0' = n0 + n1, and then b1' = n1. Iff 0 <= b0' < 2^w - 1,
@@ -542,7 +548,7 @@ __asm__("divl %4"                  \
 /* Remark. The expression n-c*a can be replaced by n%a, but the
    former is usually faster. */
 #define SSH_MPK_LONG_DIV_STEP(a,n,c,g,extr,n0)  \
-do{c=n/a;g=((n%a)<<SSH_WORD_HALF_BITS)|(extr(n0));}while(0)
+do{c=n/a;g=((n-c*a)<<SSH_WORD_HALF_BITS)|(extr(n0));}while(0)
 
 #define SSH_MPK_LONG_DIV(q, r, n1, n0, d)  \
 { \
@@ -566,6 +572,22 @@ do{c=n/a;g=((n%a)<<SSH_WORD_HALF_BITS)|(extr(n0));}while(0)
 }
 #endif /* SSH_MPK_LONG_DIV */
 
+/* Workspace definitions, that will be used when on average small amount
+   of memory will be (always) needed in a function. */
+
+#define SSH_MP_WORKSPACE_SIZE 8
+#define SSH_MP_WORKSPACE_DEFINE \
+   SshWord __workspace[SSH_MP_WORKSPACE_SIZE]
+#define SSH_MP_WORKSPACE_ALLOC(t, t_n) \
+do { \
+  unsigned int __s = (t_n); \
+  if (__s < SSH_MP_WORKSPACE_SIZE) \
+    t = __workspace; \
+  else \
+    t = ssh_malloc(sizeof(SshWord) * (__s)); \
+} while(0)
+#define SSH_MP_WORKSPACE_FREE(t) \
+  if (t != __workspace) ssh_free(t)
 
 /* Definitions that force the library to use certain specific routines. */
 
@@ -578,7 +600,7 @@ do{c=n/a;g=((n%a)<<SSH_WORD_HALF_BITS)|(extr(n0));}while(0)
    The selection of Montgomery can be taken with any of the others.
 
    Note that the algorithm of Montgomery here presented is given in the
-   book by Henri Cohen also. 
+   book by Henri Cohen also.
    */
 #undef    SSH_MPK_USE_PLUMBS_ALGORITHM
 #undef    SSH_MPK_USE_SAARINENS_ALGORITHM
@@ -607,7 +629,7 @@ do{c=n/a;g=((n%a)<<SSH_WORD_HALF_BITS)|(extr(n0));}while(0)
    is the element [0], and most significant [n-1].
 
    In following op usually denotes the word array, and op_n length of the
-   array. 
+   array.
    */
 
 /* Counting bits. */
@@ -647,19 +669,25 @@ int ssh_mpk_cmp(SshWord *op1, unsigned int op1_n,
 SshWord ssh_mpk_add(SshWord *ret,
                     SshWord *op1, unsigned int op1_n,
                     SshWord *op2, unsigned int op2_n);
+SshWord ssh_mpk_add_ui(SshWord *ret,
+                       SshWord *op, unsigned int op_n,
+                       SshWord v);
 /* Subtraction of op2 from op1. Returns the carry, and is otherwise similar
    to the addition. */
 SshWord ssh_mpk_sub(SshWord *ret,
                     SshWord *op1, unsigned int op1_n,
                     SshWord *op2, unsigned int op2_n);
+SshWord ssh_mpk_sub_ui(SshWord *ret,
+                       SshWord *op, unsigned int op_n,
+                       SshWord v);
 
 /* Multiplication of 'op1' and 'op2'. The result 'ret' must have enough space
    allocated (in practice op1_n + op2_n - 1 words is enough). */
-void ssh_mpk_mul(SshWord *ret, 
+void ssh_mpk_mul(SshWord *ret,
                  SshWord *op1, unsigned int op1_n,
                  SshWord *op2, unsigned int op2_n);
 /* Multiplication of 'op' with an unsigned integer. */
-void ssh_mpk_mul_ui(SshWord *ret, 
+void ssh_mpk_mul_ui(SshWord *ret,
                     SshWord *op,  unsigned int op_n,
                     SshWord u);
 
@@ -668,21 +696,25 @@ void ssh_mpk_mul_ui(SshWord *ret,
 void ssh_mpk_square(SshWord *ret,
                     SshWord *op,  unsigned int op_n);
 
+
 /* Karatsuba squaring algorithm. */
 
 /* This function computes the number of words needed as work space in
    the Karatsuba squaring algorithm. */
 unsigned int ssh_mpk_square_karatsuba_needed_memory(unsigned int op_n);
 
-/* The Karatsuba squaring algorithm. The 'op' is squared to 'ret'. Here
-   'ret' must be large enough. The work space pointer 'work_space' may be
-   NULL, in which case 'work_space_n' should also be 0. If work space
-   is not allocated outside the routine does it itself.
+/* The Karatsuba squaring algorithm. The 'op' is squared to
+   'ret'. Here 'ret' must be large enough. The work space pointer
+   'work_space' may be NULL, in which case 'work_space_n' should also
+   be 0. If work space is not allocated outside the routine does it
+   itself. The function returns FALSE if work space allocation fails,
+   else it returns TRUE.
 
    Out side allocation may speed up the routine slightly, however, in
-   usual personal computers this is not dramatic. 
+   usual personal computers this is not dramatic.
    */
-void ssh_mpk_square_karatsuba(SshWord *ret, unsigned int ret_n,
+Boolean
+ssh_mpk_square_karatsuba(SshWord *ret, unsigned int ret_n,
                               SshWord *op,  unsigned int op_n,
                               SshWord *work_space,
                               unsigned int work_space_n);
@@ -694,9 +726,12 @@ void ssh_mpk_square_karatsuba(SshWord *ret, unsigned int ret_n,
 unsigned int ssh_mpk_mul_karatsuba_needed_memory(unsigned int op1_n,
                                                  unsigned int op2_n);
 
-/* Multiplies 'op1' and 'op2' and places the result in 'ret'. This computation
-   can use the 'work_space' if available, although it may be set to NULL. */
-void ssh_mpk_mul_karatsuba(SshWord *ret, unsigned int ret_n,
+/* Multiplies 'op1' and 'op2' and places the result in 'ret'. This
+   computation can use the 'work_space' if available, although it may
+   be set to NULL. The function return FALSE in case of workspace
+   memory allocation error, else it returns TRUE. */
+Boolean
+ssh_mpk_mul_karatsuba(SshWord *ret, unsigned int ret_n,
                            SshWord *op1, unsigned int op1_n,
                            SshWord *op2, unsigned int op2_n,
                            SshWord *work_space, unsigned int work_space_n);
@@ -710,9 +745,9 @@ unsigned int ssh_mpk_leading_zeros(SshWord *d, unsigned int d_n);
    preallocated. The divisor 'd' should be normalized.
 
    Normalization means that the integer 'd' is multiplied by 2^n so that
-   the most significant word has the most significant bit set. 
+   the most significant word has the most significant bit set.
    */
-void ssh_mpk_div(SshWord *q, unsigned int q_n,
+Boolean ssh_mpk_div(SshWord *q, unsigned int q_n,
                  SshWord *r, unsigned int r_n,
                  SshWord *d, unsigned int d_n);
 
@@ -724,7 +759,8 @@ SshWord ssh_mpk_div_ui(SshWord *q, unsigned int q_n,
 
 /* Computation of 'r' modulo 'd'. As in usual division, 'd' must be
    normalized. */
-void ssh_mpk_mod(SshWord *r, unsigned int r_n,
+Boolean
+ssh_mpk_mod(SshWord *r, unsigned int r_n,
                  SshWord *d, unsigned int d_n);
 
 /* Computation of 'r' modulo 'd', where 'd' is unsigned integer. Here 'd'
@@ -742,4 +778,14 @@ void ssh_mpmk_reduce(SshWord *ret, unsigned int ret_n,
 /* Computation of a^-1 (mod 2^n). Input 'a' must be odd. */
 SshWord ssh_mpmk_small_inv(SshWord a);
 
-#endif /* SSHMATH_KERNEL */
+/* Compute -op mod 2^n. */
+void ssh_mpmk_2adic_neg(SshWord *ret, SshWord *op, unsigned int op_n);
+
+
+
+
+
+
+
+
+#endif /* SSHMP_KERNEL */
