@@ -14,6 +14,70 @@
 #include "sshincludes.h"
 #include "sshmp.h"
 
+/* NaN routines. */
+
+Boolean ssh_mpmzm_isnan(SshMPMontIntModConst op)
+{
+  return (op->m == NULL && op->n == 0 && op->isnan);
+}
+
+void ssh_mpmzm_makenan(SshMPMontIntMod op, unsigned int kind)
+{
+  if (op->v) ssh_xfree(op->v);
+  op->v = NULL;
+  op->n = 0;
+  op->m = NULL;
+  op->isnan = TRUE;
+  op->nankind = kind;
+}
+
+Boolean ssh_mpmzm_nanresult1(SshMPMontIntMod ret, SshMPMontIntModConst op)
+{
+  if (ssh_mpmzm_isnan(ret))
+    return TRUE;
+  
+  if (ssh_mpmzm_isnan(op))
+    { 
+      ssh_mpmzm_makenan(ret, op->nankind); 
+      return TRUE;
+    }
+  
+  if (ret->m != op->m)
+    {
+      ssh_mpmzm_makenan(ret, SSH_MPMZM_NAN_IDEAL); 
+      return TRUE;   
+    }
+  
+  return FALSE;
+}
+
+Boolean ssh_mpmzm_nanresult2(SshMPMontIntMod ret, SshMPMontIntModConst op1, 
+                             SshMPMontIntModConst op2)
+{ 
+  if (ssh_mpmzm_isnan(ret))
+    return TRUE;
+  
+  if (ssh_mpmzm_isnan(op1))
+    { 
+      ssh_mpmzm_makenan(ret, op1->nankind); 
+      return TRUE;
+    }
+  
+  if (ssh_mpmzm_isnan(op2))
+    { 
+      ssh_mpmzm_makenan(ret, op2->nankind); 
+      return TRUE;
+    }
+  
+  if (ret->m != op1->m || ret->m != op2->m)
+    { 
+      ssh_mpmzm_makenan(ret, SSH_MPMZM_NAN_IDEAL); 
+      return TRUE;   
+    }
+  
+  return FALSE;
+}
+
 /* Montgomery representation implementation. The modulus is required
    to be odd. */
 
@@ -21,6 +85,9 @@
 Boolean ssh_mpmzm_init_ideal(SshMPMontIntIdeal m, SshMPIntegerConst op)
 {
   unsigned int temp_n;
+  
+  if (ssh_mprz_isnan(op))
+    return FALSE;
 
   /* If op < 3 or op % 2 == 0 we cannot work in Montgomery
      representation. */
@@ -33,9 +100,14 @@ Boolean ssh_mpmzm_init_ideal(SshMPMontIntIdeal m, SshMPIntegerConst op)
   /* Compute mp = -op^-1 (mod 2^SSH_WORD_BITS).
    */
   m->mp = SSH_WORD_NEGATE(ssh_mpmk_small_inv(op->v[0]));
-
+  m->karatsuba_work_space   = NULL;
+  m->work_space             = NULL;
+  
   /* Set the modulus up, also in normalized form. */
-  m->m = ssh_xmalloc(sizeof(SshWord) * (op->n + op->n));
+  m->m = ssh_malloc(sizeof(SshWord) * (op->n + op->n));
+  if (m->m == NULL)
+    goto failure;
+  
   m->d = m->m + op->n;
   m->m_n = op->n;
   ssh_mpk_memcopy(m->m, op->v, m->m_n);
@@ -54,8 +126,13 @@ Boolean ssh_mpmzm_init_ideal(SshMPMontIntIdeal m, SshMPIntegerConst op)
     m->karatsuba_work_space_n = temp_n;
   /* Note that it is still possible that no extra memory is needed! */
   if (m->karatsuba_work_space_n)
-    m->karatsuba_work_space = ssh_xmalloc(sizeof(SshWord) * 
-                                          m->karatsuba_work_space_n);
+    {
+      m->karatsuba_work_space = ssh_malloc(sizeof(SshWord) * 
+                                           m->karatsuba_work_space_n);
+      if (m->karatsuba_work_space == NULL)
+        goto failure;
+    } 
+  
   else
     m->karatsuba_work_space = NULL;
   
@@ -63,16 +140,22 @@ Boolean ssh_mpmzm_init_ideal(SshMPMontIntIdeal m, SshMPIntegerConst op)
   
   /* The amount of memory for multiplication and squaring! */
   m->work_space_n = (m->m_n * 2 + 1) * 2;
-  m->work_space   = ssh_xmalloc(sizeof(SshWord) * m->work_space_n);
+  m->work_space   = ssh_malloc(sizeof(SshWord) * m->work_space_n);
+  if (m->work_space == NULL)
+    goto failure;
   
 #else /* SSHMATH_USE_WORKSPACE */
-  m->karatsuba_work_space   = NULL;
   m->karatsuba_work_space_n = 0;
-  m->work_space             = NULL;
   m->work_space_n           = 0;
 #endif /* SSHMATH_USE_WORKSPACE */
   
   return TRUE;
+  
+ failure:
+  ssh_xfree(m->m);
+  ssh_xfree(m->karatsuba_work_space);
+  ssh_xfree(m->work_space);
+  return FALSE;
 }
 
 /* Clean up the used moduli space. */
@@ -102,41 +185,62 @@ void ssh_mprz_set_mpmzm_ideal(SshMPInteger ret, SshMPMontIntIdealConst m)
 void ssh_mpmzm_init(SshMPMontIntMod op, SshMPMontIntIdealConst m)
 {
   op->n = 0;
-  op->v = ssh_xmalloc(sizeof(SshWord) * (m->m_n + 1));
   op->m = m;
+  op->isnan = FALSE;
+  op->nankind = 0;
+ 
+  if ((op->v = ssh_malloc(sizeof(SshWord) * (m->m_n + 1))) == NULL)
+    ssh_mpmzm_makenan(op, SSH_MPMZM_NAN_ENOMEM);
 }
 
 void ssh_mpmzm_init_inherit(SshMPMontIntMod op1,
                             SshMPMontIntModConst op2)
 {
+  if (ssh_mpmzm_isnan(op2))
+    {
+      op1->v = NULL;    
+      op1->n = 0;
+      op1->m = NULL;
+      op1->isnan = TRUE;
+      op1->nankind = op2->nankind;
+      return;
+    }
+  
   op1->n = 0;
-  op1->v = ssh_xmalloc(sizeof(SshWord) * (op2->m->m_n + 1));
-  op1->m = op2->m;
+  op1->m = op2->m; 
+  op1->isnan = FALSE;
+  op1->nankind = 0;
+  
+  if ((op1->v = ssh_malloc(sizeof(SshWord) * (op2->m->m_n + 1))) == NULL)
+    ssh_mpmzm_makenan(op1, SSH_MPMZM_NAN_ENOMEM);
 }
 
 void ssh_mpmzm_clear(SshMPMontIntMod op)
 {
-  ssh_xfree(op->v);
+  if (!ssh_mpmzm_isnan(op))
+    ssh_xfree(op->v); 
   op->n = 0;
   op->m = NULL;
+  op->isnan = FALSE;
+  op->nankind = 0;
+  op->v = NULL;
 }
 
-Boolean ssh_mpmzm_set(SshMPMontIntMod ret, SshMPMontIntModConst op)
+void ssh_mpmzm_set(SshMPMontIntMod ret, SshMPMontIntModConst op)
 {
-  if (op->m != ret->m)
-    return FALSE;   
-  
   if (ret == op)
-    return TRUE;
-
+    return;
+  
+  if (ssh_mpmzm_nanresult1(ret, op))
+    return;
+  
   if (op->n == 0)
     {
       ret->n = 0;
-      return TRUE;
+      return;
     }
   ssh_mpk_memcopy(ret->v, op->v, op->n);
   ret->n = op->n;
-  return TRUE;
 }
 
 void ssh_mpmzm_set_mprz(SshMPMontIntMod ret, SshMPIntegerConst op)
@@ -145,6 +249,14 @@ void ssh_mpmzm_set_mprz(SshMPMontIntMod ret, SshMPIntegerConst op)
   SshWord *t;
   unsigned int t_n;
 
+  if (ssh_mpmzm_isnan(ret))
+    return;
+
+  if (ssh_mprz_isnan(op))
+    {
+      ssh_mpmzm_makenan(ret, SSH_MPMZM_NAN_ENOMEM);
+      return;
+    }
   /* Trivial case. */
   if (op->n == 0)
     {
@@ -161,6 +273,11 @@ void ssh_mpmzm_set_mprz(SshMPMontIntMod ret, SshMPIntegerConst op)
 
   /* Allocate some temporary space. */
   SSH_MP_WORKSPACE_ALLOC(t, (op->n + 1 + ret->m->m_n));
+  if (!t) 
+    {
+      ssh_mpmzm_makenan(ret, SSH_MPMZM_NAN_ENOMEM);
+      return;
+    }
 
   /* Multiply by R the remainder. */
   ssh_mpk_memzero(t, ret->m->m_n);
@@ -193,25 +310,28 @@ void ssh_mpmzm_set_mprz(SshMPMontIntMod ret, SshMPIntegerConst op)
   SSH_MP_WORKSPACE_FREE(t);
 }
 
-Boolean ssh_mpmzm_set_ui(SshMPMontIntMod ret, SshWord u)
+void ssh_mpmzm_set_ui(SshMPMontIntMod ret, SshWord u)
 {
+  if (ssh_mpmzm_isnan(ret))
+    return;
+  
   /* Do zeroing fast. */
   if (u == 0)
     {
       ret->n = 0;
-      return TRUE;
+      return;
     }
   else
     {
       SshMPIntegerStruct mp;
       /* This is slow, and unoptimized. Most of the time you
          don't need to do this. */
-      ssh_mp_init(&mp);
-      ssh_mp_set_ui(&mp, u);
+      ssh_mprz_init(&mp);
+      ssh_mprz_set_ui(&mp, u);
       ssh_mpmzm_set_mprz(ret, &mp);
-      ssh_mp_clear(&mp);
+      ssh_mprz_clear(&mp);
     }
-  return TRUE;
+  return;
 }
 
 void ssh_mprz_set_mpmzm(SshMPInteger ret, SshMPMontIntModConst op)
@@ -220,11 +340,22 @@ void ssh_mprz_set_mpmzm(SshMPInteger ret, SshMPMontIntModConst op)
   SshWord *t;
   unsigned int t_n;
 
+  if (ssh_mpmzm_isnan(op))
+    { 
+      ssh_mprz_makenan(ret, SSH_MP_NAN_ENOMEM);
+      return;
+    } 
+  
   /* Allocate enough space for reduction to happen. */
   t_n = op->m->m_n * 2 + 1;
   SSH_MP_WORKSPACE_ALLOC(t, t_n);
+  if (!t)
+    {
+      ssh_mprz_makenan(ret, SSH_MP_NAN_ENOMEM);
+      return;
+    }
   ssh_mpk_memzero(t, t_n);
-
+  
   /* Reduce. */
   ssh_mpmk_reduce(t, t_n,
                   op->v, op->n,
@@ -254,7 +385,8 @@ void ssh_mprz_set_mpmzm(SshMPInteger ret, SshMPMontIntModConst op)
 int ssh_mpmzm_cmp(SshMPMontIntModConst op1,
                   SshMPMontIntModConst op2)
 {
-  if (op1->m != op2->m)
+  /* Two NaNs are unequal, and a NaN is not equal to any MontInt. */ 
+  if (ssh_mpmzm_isnan(op1) || ssh_mpmzm_isnan(op2))
     return 1;
   
   return ssh_mpk_cmp(op1->v, op1->n, op2->v, op2->n);
@@ -263,18 +395,30 @@ int ssh_mpmzm_cmp(SshMPMontIntModConst op1,
 /* Simple wrapper. */
 int ssh_mpmzm_cmp_ui(SshMPMontIntModConst op, SshWord u)
 {
-  return ssh_mpk_cmp_ui(op->v, op->n, u);
+  int b;
+  SshMPMontIntModStruct tmp; 
+
+  if (ssh_mpmzm_isnan(op))
+    return 1;
+
+  ssh_mpmzm_init_inherit(&tmp, op);
+  ssh_mpmzm_set_ui(&tmp, u);
+
+  b = ssh_mpmzm_cmp(op, &tmp); 
+  ssh_mpmzm_clear(&tmp);
+
+  return b;  
 }
 
 /* Addition is easy with Montgomery representation. */
-Boolean ssh_mpmzm_add(SshMPMontIntMod ret, SshMPMontIntModConst op1,
+void ssh_mpmzm_add(SshMPMontIntMod ret, SshMPMontIntModConst op1,
                    SshMPMontIntModConst op2)
 {
   SshWord c;
-
-  if (op1->m != op2->m || op1->m != ret->m)
-    return FALSE;
-
+  
+  if (ssh_mpmzm_nanresult2(ret, op1, op2))
+    return;
+  
   if (op1->n < op2->n)
     {
       SshMPMontIntModConst t;
@@ -300,17 +444,15 @@ Boolean ssh_mpmzm_add(SshMPMontIntMod ret, SshMPMontIntModConst op1,
       while (ret->n && ret->v[ret->n - 1] == 0)
         ret->n--;
     }
-  return TRUE;
 }
 
 /* Subtraction is a bit more difficult. */
-Boolean ssh_mpmzm_sub(SshMPMontIntMod ret, SshMPMontIntModConst op1,
-                      SshMPMontIntModConst op2)
+void ssh_mpmzm_sub(SshMPMontIntMod ret, SshMPMontIntModConst op1,
+                   SshMPMontIntModConst op2)
 {
-
-  if (op1->m != op2->m || op1->m != ret->m)
-    return FALSE;
-
+  if (ssh_mpmzm_nanresult2(ret, op1, op2))
+    return;
+  
   if (ssh_mpk_cmp(op1->v, op1->n, op2->v, op2->n) >= 0)
     {
       ssh_mpk_sub(ret->v, op1->v, op1->n, op2->v, op2->n);
@@ -331,23 +473,22 @@ Boolean ssh_mpmzm_sub(SshMPMontIntMod ret, SshMPMontIntModConst op1,
       while (ret->n && ret->v[ret->n - 1] == 0)
         ret->n--;
     }
-  return TRUE;
 }
 
-Boolean ssh_mpmzm_mul(SshMPMontIntMod ret, SshMPMontIntModConst op1,
+void ssh_mpmzm_mul(SshMPMontIntMod ret, SshMPMontIntModConst op1,
                    SshMPMontIntModConst op2)
 {
   SSH_MP_WORKSPACE_DEFINE;
   SshWord *t, *r;
   unsigned int t_n, r_n;
 
-  if (op1->m != op2->m || op1->m != ret->m)
-    return FALSE;
-
+  if (ssh_mpmzm_nanresult2(ret, op1, op2))
+    return;
+  
   if (op1->n == 0 || op2->n == 0)
     {
       ret->n = 0;
-      return TRUE;
+      return;
     }
 
   /* Allocate some temporary space. */
@@ -357,6 +498,11 @@ Boolean ssh_mpmzm_mul(SshMPMontIntMod ret, SshMPMontIntModConst op1,
     {
       /* Use the stack based workspace if possible. */
       SSH_MP_WORKSPACE_ALLOC(t, t_n + r_n);
+      if (!t)
+        {
+          ssh_mpmzm_makenan(ret, SSH_MPMZM_NAN_ENOMEM);
+          return;
+        }
     }
   else
     t = ret->m->work_space;
@@ -364,9 +510,15 @@ Boolean ssh_mpmzm_mul(SshMPMontIntMod ret, SshMPMontIntModConst op1,
 
   /* Clear temporary space. */
   ssh_mpk_memzero(t, t_n);
-  ssh_mpk_mul_karatsuba(t, t_n, op1->v, op1->n, op2->v, op2->n,
+  if (!ssh_mpk_mul_karatsuba(t, t_n, op1->v, op1->n, op2->v, op2->n,
                         ret->m->karatsuba_work_space,
-                        ret->m->karatsuba_work_space_n);
+                             ret->m->karatsuba_work_space_n))
+    {
+      ssh_mpmzm_makenan(ret, SSH_MPMZM_NAN_ENOMEM); 
+      SSH_MP_WORKSPACE_FREE(t);
+      return;
+    }
+  
 
   /* Find the exact length. */
   while (t_n && t[t_n - 1] == 0)
@@ -393,34 +545,33 @@ Boolean ssh_mpmzm_mul(SshMPMontIntMod ret, SshMPMontIntModConst op1,
     {
       SSH_MP_WORKSPACE_FREE(t);
     }
-  return TRUE;
 }
 
 /* This should work, because op = x*R (mod m) and we can just
    compute op*u = x*R*u (mod m) as before. This should be much
    faster than standard multiplication. */
-Boolean ssh_mpmzm_mul_ui(SshMPMontIntMod ret, SshMPMontIntModConst op,
+void ssh_mpmzm_mul_ui(SshMPMontIntMod ret, SshMPMontIntModConst op,
                       SshWord u)
 {
   SSH_MP_WORKSPACE_DEFINE;
   SshWord *t;
   int t_n;
 
-  if (op->m != ret->m)
-    return FALSE;
+  if (ssh_mpmzm_nanresult1(ret, op))
+    return;
 
   /* Handle the trivial case. */
   if (op->n == 0 || u == 0)
     {
       ret->n = 0;
-      return TRUE;
+      return;
     }
 
   /* Another trivial case. */
   if (u == 1)
     {
       ssh_mpmzm_set(ret, op);
-      return TRUE;
+      return;
     }
 
   /* Multiply first. */
@@ -428,6 +579,11 @@ Boolean ssh_mpmzm_mul_ui(SshMPMontIntMod ret, SshMPMontIntModConst op,
   if (ret->m->work_space == NULL)
     {
       SSH_MP_WORKSPACE_ALLOC(t, t_n);
+      if (!t)
+        {
+          ssh_mpmzm_makenan(ret, SSH_MPMZM_NAN_ENOMEM);
+          return;
+        }
     }
   else
     t = ret->m->work_space;
@@ -473,22 +629,21 @@ Boolean ssh_mpmzm_mul_ui(SshMPMontIntMod ret, SshMPMontIntModConst op,
     {
       SSH_MP_WORKSPACE_FREE(t);
     }
-  return TRUE;
 }
 
-Boolean ssh_mpmzm_square(SshMPMontIntMod ret, SshMPMontIntModConst op)
+void ssh_mpmzm_square(SshMPMontIntMod ret, SshMPMontIntModConst op)
 {
   SSH_MP_WORKSPACE_DEFINE;
   SshWord *t, *r;
   unsigned int t_n, r_n;
 
-  if (op->m != ret->m)
-    return FALSE;
+ if (ssh_mpmzm_nanresult1(ret, op))
+    return;
 
   if (op->n == 0)
     {
       ret->n = 0;
-      return TRUE;
+      return;
     }
 
   /* Allocate some temporary space. */
@@ -497,17 +652,27 @@ Boolean ssh_mpmzm_square(SshMPMontIntMod ret, SshMPMontIntModConst op)
   if (ret->m->work_space == NULL)
     {
       SSH_MP_WORKSPACE_ALLOC(t, t_n + r_n);
+      if (!t)
+        {
+          ssh_mpmzm_makenan(ret, SSH_MPMZM_NAN_ENOMEM);
+          return;
+        }
     }
   else
     t = ret->m->work_space;
   r = t + t_n;
-
+  
   /* Clear temporary space. */
   ssh_mpk_memzero(t, t_n + r_n);
-  ssh_mpk_square_karatsuba(t, t_n, op->v, op->n,
+  if (!ssh_mpk_square_karatsuba(t, t_n, op->v, op->n,
                            ret->m->karatsuba_work_space,
-                           ret->m->karatsuba_work_space_n);
-
+                                ret->m->karatsuba_work_space_n))
+    {
+      ssh_mpmzm_makenan(ret, SSH_MPMZM_NAN_ENOMEM); 
+      SSH_MP_WORKSPACE_FREE(t);
+      return;
+    }
+  
   /* Find the exact length. */
   while (t_n && t[t_n - 1] == 0)
     t_n--;
@@ -533,10 +698,9 @@ Boolean ssh_mpmzm_square(SshMPMontIntMod ret, SshMPMontIntModConst op)
     {
       SSH_MP_WORKSPACE_FREE(t);
     }
-  return TRUE;
 }
 
-Boolean ssh_mpmzm_mul_2exp(SshMPMontIntMod ret, SshMPMontIntModConst op,
+void ssh_mpmzm_mul_2exp(SshMPMontIntMod ret, SshMPMontIntModConst op,
                         unsigned int exp)
 {
   SSH_MP_WORKSPACE_DEFINE;
@@ -544,21 +708,21 @@ Boolean ssh_mpmzm_mul_2exp(SshMPMontIntMod ret, SshMPMontIntModConst op,
   SshWord *t;
   int t_n, max;
 
-  if (op->m != ret->m)
-    return FALSE;
+  if (ssh_mpmzm_nanresult1(ret, op))
+    return;
 
   /* Check if no need to to anything. */
   if (op->n == 0)
     {
       ret->n = 0;
-      return TRUE;
+      return;
     }
 
   /* Handle some special number of bits here. */
   if (exp == 0)
     {
       ssh_mpmzm_set(ret, op);
-      return TRUE;
+      return;
     }
 
   if (exp < SSH_WORD_BITS)
@@ -567,6 +731,11 @@ Boolean ssh_mpmzm_mul_2exp(SshMPMontIntMod ret, SshMPMontIntModConst op,
       if (ret->m->work_space == NULL)
         {
           SSH_MP_WORKSPACE_ALLOC(t, t_n);
+          if (!t)
+            {
+              ssh_mpmzm_makenan(ret, SSH_MPMZM_NAN_ENOMEM);
+              return;
+            }
         }
       else
         t = ret->m->work_space;
@@ -607,7 +776,7 @@ Boolean ssh_mpmzm_mul_2exp(SshMPMontIntMod ret, SshMPMontIntModConst op,
           SSH_MP_WORKSPACE_FREE(t);
         }
 
-      return TRUE;
+      return;
     }
 
   /* Compute the maximum number of suitable bits. */
@@ -638,6 +807,11 @@ Boolean ssh_mpmzm_mul_2exp(SshMPMontIntMod ret, SshMPMontIntModConst op,
       if (ret->m->work_space == NULL)
         {
           SSH_MP_WORKSPACE_ALLOC(t, t_n);
+          if (!t)
+            {
+              ssh_mpmzm_makenan(ret, SSH_MPMZM_NAN_ENOMEM);
+              return;
+            }
         }
       else
         t = ret->m->work_space;
@@ -672,29 +846,28 @@ Boolean ssh_mpmzm_mul_2exp(SshMPMontIntMod ret, SshMPMontIntModConst op,
           SSH_MP_WORKSPACE_FREE(t);
         }
     }
-  return TRUE;
 }
 
-Boolean ssh_mpmzm_div_2exp(SshMPMontIntMod ret, SshMPMontIntModConst op,
+void ssh_mpmzm_div_2exp(SshMPMontIntMod ret, SshMPMontIntModConst op,
                         unsigned int exp)
 {
   unsigned int i;
   SshWord c;
 
-  if (op->m != ret->m)
-    return FALSE;
+  if (ssh_mpmzm_nanresult1(ret, op))
+    return;
 
   /* Handle trivial cases first. */
   if (op->n == 0)
     {
       ret->n = 0;
-      return TRUE;
+      return;
     }
 
   if (exp == 0)
     {
       ssh_mpmzm_set(ret, op);
-      return TRUE;
+      return;
     }
 
   /* Now handle the main iteration, notice that dividing by very
@@ -723,9 +896,9 @@ Boolean ssh_mpmzm_div_2exp(SshMPMontIntMod ret, SshMPMontIntModConst op,
       while (ret->n && ret->v[ret->n - 1] == 0)
         ret->n--;
     }
-  return TRUE;
 }
 
+#ifdef SSHDIST_MATH_INTMOD
 /* This will be needed in some future time. E.g. when writing fast
    polynomial arithmetic modulo large integer. Although, one should
    then also implement some other routines which would be of lots of
@@ -735,7 +908,7 @@ Boolean ssh_mpmzm_invert(SshMPMontIntMod ret, SshMPMontIntModConst op)
   SshMPIntegerStruct t, q;
   Boolean rv;
 
-  if (op->m != ret->m)
+ if (ssh_mpmzm_nanresult1(ret, op))
     return FALSE;
 
   ssh_mprz_init(&t);
@@ -755,7 +928,7 @@ Boolean ssh_mpmzm_sqrt(SshMPMontIntMod ret, SshMPMontIntModConst op)
   SshMPIntegerStruct t, q;
   Boolean rv;
 
-  if (op->m != ret->m)
+ if (ssh_mpmzm_nanresult1(ret, op))
     return FALSE;
 
   ssh_mprz_init(&t);
@@ -769,23 +942,33 @@ Boolean ssh_mpmzm_sqrt(SshMPMontIntMod ret, SshMPMontIntModConst op)
   ssh_mprz_clear(&q);
   return rv;
 }
+#endif /* SSHDIST_MATH_INTMOD */
 
-Boolean ssh_mpmzm_pow_ui(SshMPMontIntMod ret, SshWord g, SshMPIntegerConst e)
+void ssh_mpmzm_pow_ui(SshMPMontIntMod ret, SshWord g, SshMPIntegerConst e)
 {
   SshMPMontIntModStruct temp, x;
   unsigned int bits, i;
+  
+  if (ssh_mpmzm_isnan(ret))
+    return;
+  
+  if (ssh_mprz_isnan(e))
+    {
+      ssh_mpmzm_makenan(ret, SSH_MPMZM_NAN_ENOMEM);
+      return;
+    }
 
   /* Trivial cases. */
   if (ssh_mprz_cmp_ui(e, 0) == 0)
     {
       ssh_mpmzm_set_ui(ret, 1);
-      return TRUE;
+      return;
     }
 
   if (ssh_mprz_cmp_ui(e, 1) == 0)
     {
       ssh_mpmzm_set_ui(ret, g);
-      return TRUE;
+      return;
     }
 
   ssh_mpmzm_init_inherit(&temp, ret);
@@ -807,12 +990,10 @@ Boolean ssh_mpmzm_pow_ui(SshMPMontIntMod ret, SshWord g, SshMPIntegerConst e)
   ssh_mpmzm_set(ret, &temp);
   ssh_mpmzm_clear(&temp);
   ssh_mpmzm_clear(&x);
-
-  return TRUE;
 }
 
-Boolean ssh_mpmzm_pow(SshMPMontIntMod ret, SshMPMontIntModConst g,
-                      SshMPIntegerConst e)
+void ssh_mpmzm_pow(SshMPMontIntMod ret, SshMPMontIntModConst g,
+                   SshMPIntegerConst e)
 {
   SshMPMontIntModStruct temp, x;
   unsigned int table_bits, table_size;
@@ -821,20 +1002,26 @@ Boolean ssh_mpmzm_pow(SshMPMontIntMod ret, SshMPMontIntModConst g,
   unsigned int tab[] =
   { 24, 88, 277, 798, 2173, 5678, 14373, 0 };
 
-  if (ret->m != g->m)
-    return FALSE;
+  if (ssh_mpmzm_nanresult1(ret, g))
+    return;
+
+  if (ssh_mprz_isnan(e))
+    {
+      ssh_mpmzm_makenan(ret, SSH_MPMZM_NAN_ENOMEM);
+      return;
+    }
 
   /* Trivial cases. */
   if (ssh_mprz_cmp_ui(e, 0) == 0)
     {
       ssh_mpmzm_set_ui(ret, 1);
-      return TRUE;
+      return;
     }
 
   if (ssh_mprz_cmp_ui(e, 1) == 0)
     {
       ssh_mpmzm_set(ret, g);
-      return TRUE;
+      return;
     }
 
   ssh_mpmzm_init_inherit(&temp, ret);
@@ -856,7 +1043,14 @@ Boolean ssh_mpmzm_pow(SshMPMontIntMod ret, SshMPMontIntModConst g,
   table_size = ((SshWord)1 << (table_bits - 1));
 
   /* Allocate the table. */
-  table = ssh_xmalloc(sizeof(SshMPMontIntModStruct) * table_size);
+  table = ssh_malloc(sizeof(SshMPMontIntModStruct) * table_size);
+  if (!table)
+    {
+      ssh_mpmzm_clear(&temp);
+      ssh_mpmzm_clear(&x);
+      ssh_mpmzm_makenan(ret, SSH_MPMZM_NAN_ENOMEM);
+      return;
+    }
 
   /* Start computing the table. */
   ssh_mpmzm_init_inherit(&table[0], ret);
@@ -924,9 +1118,6 @@ Boolean ssh_mpmzm_pow(SshMPMontIntMod ret, SshMPMontIntModConst g,
 
   ssh_mpmzm_clear(&temp);
   ssh_mpmzm_clear(&x);
-
-  return TRUE;
 }
 
 /* sshmp-montgomery.c */
-
