@@ -4,7 +4,7 @@
 
   Author: Mika Kojo <mkojo@ssh.fi>
 
-  Copyright (c) 2000 SSH Communications Security Corp
+  Copyright (c) 2000, 2001 SSH Communications Security Corp
   All rights reserved.
 
   Created Sun Feb 18 20:32:25 2001.
@@ -18,7 +18,7 @@ void ssh_mprz_powm(SshMPInteger ret, SshMPIntegerConst g, SshMPIntegerConst e,
                    SshMPIntegerConst p);
 
 void ssh_mprz_powm_gg(SshMPInteger ret,
-                      SshMPIntegerConst g1, SshMPIntegerConst e1, 
+                      SshMPIntegerConst g1, SshMPIntegerConst e1,
                       SshMPIntegerConst g2, SshMPIntegerConst e2,
                       SshMPIntegerConst p);
 
@@ -32,8 +32,13 @@ void ssh_mprz_powm_ui_exp(SshMPInteger ret, SshMPIntegerConst g,
 void ssh_mprz_powm_precomp_init(SshMPIntModPowPrecomp precomp,
                                 SshMPIntegerConst g, SshMPIntegerConst p,
                                 SshMPIntegerConst bound);
+SshMPIntModPowPrecomp
+ssh_mprz_powm_precomp_create(SshMPIntegerConst g,
+                             SshMPIntegerConst p,
+                             SshMPIntegerConst order);
 
 void ssh_mprz_powm_precomp_clear(SshMPIntModPowPrecomp precomp);
+void ssh_mprz_powm_precomp_destroy(SshMPIntModPowPrecomp precomp);
 
 void ssh_mprz_powm_with_precomp(SshMPInteger ret, SshMPIntegerConst e,
                                 SshMPIntModPowPrecompConst precomp);
