@@ -11,7 +11,7 @@ File handling for OpenPGP files.
 
 */
 /*
- * $Id: pgp_file.c,v 1.12 2001/08/02 01:24:17 sjl Exp $
+ * $Id: pgp_file.c,v 1.13 2002/03/26 23:44:21 sjl Exp $
  * $Log: pgp_file.c,v $ * 
  * 
  * 
@@ -53,6 +53,9 @@ File handling for OpenPGP files.
  * 
  * 
  * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -86,12 +89,12 @@ Boolean ssh_pgp_read_packet(SshFileBuffer filebuf, SshPgpPacket *packet)
     type_id = *(ssh_buffer_ptr(&(filebuf->buf)));
     ssh_buffer_consume(&(filebuf->buf), 1);
     
-    SSH_DEBUG(5, ("type_id = %d\n", type_id));
+    SSH_DEBUG(5, ("type_id = %d", type_id));
 
     if (type_id & 0x40) 
       {
         /* New packet header format */
-        SSH_DEBUG(5, ("New packet header format\n"));
+        SSH_DEBUG(5, ("New packet header format"));
         if (partial_buf_init == FALSE)
           packet_type = type_id & 0x3f;
         fr = ssh_file_buffer_expand(filebuf, 1);
@@ -143,7 +146,7 @@ Boolean ssh_pgp_read_packet(SshFileBuffer filebuf, SshPgpPacket *packet)
         size_t ll;
 
         /* Old packet header format */
-        SSH_DEBUG(5, ("Old packet header format\n"));
+        SSH_DEBUG(5, ("Old packet header format"));
         partial_body = FALSE;
         if (partial_buf_init == FALSE)
           packet_type = (type_id & 0x7c) >> 2;
