@@ -11,7 +11,7 @@ Parse pgp keyblobs.
 
 */
 /*
- * $Id: pgp_key.c,v 1.16.16.1 2001/11/12 06:27:05 sjl Exp $
+ * $Id: pgp_key.c,v 1.18 2002/03/26 23:44:21 sjl Exp $
  * $Log: pgp_key.c,v $ * 
  * 
  * 
@@ -79,6 +79,12 @@ Parse pgp keyblobs.
  * 
  * 
  * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -191,7 +197,7 @@ size_t ssh_pgp_public_key_decode(const unsigned char *data,
 
         default:
           /* Unexpected algorithm type for version 2 or 3 */
-          SSH_DEBUG(3, ("wrong pk algorithm %d for old packet version %d\n",
+          SSH_DEBUG(3, ("wrong pk algorithm %d for old packet version %d",
                         pk_algorithm_byte, version));
           goto packet_parsing_failed;
         }
@@ -334,7 +340,7 @@ size_t ssh_pgp_public_key_decode(const unsigned char *data,
           break;
 
         default:
-          SSH_DEBUG(3, ("unsupported pk algorithm %d\n", pk_algorithm_byte));
+          SSH_DEBUG(3, ("unsupported pk algorithm %d", pk_algorithm_byte));
           goto packet_parsing_failed;
         }
       pubkey_data_len = skiptot - pubkey_start_offset;
@@ -392,7 +398,7 @@ size_t ssh_pgp_public_key_decode(const unsigned char *data,
         }
     } else {
       /* Unsupported key packet version */
-      SSH_DEBUG(3, ("unsupported key packet version %d\n", version));
+      SSH_DEBUG(3, ("unsupported key packet version %d", version));
       return 0;
     }
 
@@ -416,7 +422,7 @@ size_t ssh_pgp_public_key_decode(const unsigned char *data,
         if (cr != SSH_CRYPTO_OK)
           {
             r->key = NULL;
-            SSH_DEBUG(3, ("ssh_public_key_define returns %d\n", cr));
+            SSH_DEBUG(3, ("ssh_public_key_define returns %d", cr));
           }
       }
       break;
@@ -433,7 +439,7 @@ size_t ssh_pgp_public_key_decode(const unsigned char *data,
           if (cr != SSH_CRYPTO_OK)
             {
               r->key = NULL;      
-              SSH_DEBUG(3, ("ssh_public_key_define returns %d\n", cr));
+              SSH_DEBUG(3, ("ssh_public_key_define returns %d", cr));
             }
         }
       break;
@@ -449,7 +455,7 @@ size_t ssh_pgp_public_key_decode(const unsigned char *data,
           if (cr != SSH_CRYPTO_OK)
             {
               r->key = NULL;      
-              SSH_DEBUG(3, ("ssh_public_key_define returns %d\n", cr));
+              SSH_DEBUG(3, ("ssh_public_key_define returns %d", cr));
             }
         }
       break;
