@@ -39,17 +39,13 @@ Copyright (c) 1997 SSH Communications Security, Finland
 #define USING_SGTTY
 #endif /* HAVE_SGTTY_H && !USING_TERMIOS */
 
+#ifndef VXWORKS
 #ifndef WIN32
 #if !defined(USING_SGTTY) && !defined(USING_TERMIOS)
 #error ERROR NO TERMIOS OR SGTTY;
 #endif /* !USING_SGTTY && !USING_TERMIOS */
 #endif /* WIN32 */
-
-/* Define UID_ROOT to be the user id for root (normally zero, but different
-   e.g. on Amiga). */
-#ifndef UID_ROOT
-#define UID_ROOT 0
-#endif
+#endif /* VXWORKS */
 
 #if HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
