@@ -36,8 +36,12 @@ void ssh_leave_raw_mode(int fd)
   if (isatty(fd))
     {
 #ifdef USING_TERMIOS
-      if (tcsetattr(fd, TCSADRAIN, &saved_tio) < 0)
-        perror("tcsetattr");
+      while (tcsetattr(fd, TCSADRAIN, &saved_tio) < 0 && errno == EINTR)
+        {
+          if (errno != EINTR)
+            perror("tcsetattr");
+          errno = 0; /* For Solaris; just to make sure. */
+        }
 #endif /* USING_TERMIOS */
 #ifdef USING_SGTTY
       if (ioctl(fd, TIOCSETP, &saved_tio) < 0)
@@ -48,7 +52,7 @@ void ssh_leave_raw_mode(int fd)
 
 /* Puts the user\'s terminal in raw mode. */
 
-void ssh_enter_raw_mode(int fd)
+void ssh_enter_raw_mode(int fd, Boolean want_kbd_signals)
 {
   if (fd < 0)
     fd = fileno(stdin);
@@ -62,15 +66,22 @@ void ssh_enter_raw_mode(int fd)
       saved_tio = tio;
       tio.c_iflag |= IGNPAR;
       tio.c_iflag &= ~(ISTRIP|INLCR|IGNCR|ICRNL|IXON|IXANY|IXOFF);
-      tio.c_lflag &= ~(ISIG|ICANON|ECHO|ECHOE|ECHOK|ECHONL);
+      if (want_kbd_signals)
+        tio.c_lflag &= ~(ICANON|ECHO|ECHOE|ECHOK|ECHONL);
+      else
+        tio.c_lflag &= ~(ISIG|ICANON|ECHO|ECHOE|ECHOK|ECHONL);
 #ifdef IEXTEN
       tio.c_lflag &= ~IEXTEN;
 #endif /* IEXTEN */
       tio.c_oflag &= ~OPOST;
       tio.c_cc[VMIN] = 1;
       tio.c_cc[VTIME] = 0;
-      if (tcsetattr(fd, TCSADRAIN, &tio) < 0)
-        perror("tcsetattr");
+      while (tcsetattr(fd, TCSADRAIN, &tio) < 0 && errno == EINTR)
+        {
+          if (errno != EINTR)
+            perror("tcsetattr");
+          errno = 0;
+        }
       ssh_in_raw_mode = TRUE;
 #endif /* USING_TERMIOS */
 #ifdef USING_SGTTY
@@ -88,7 +99,7 @@ void ssh_enter_raw_mode(int fd)
     }
 }  
 
-/* Puts terminal in non-blocking mode. */
+/* Restores terminal to blocking mode. */
 
 void ssh_leave_non_blocking(int fd)
 {
@@ -101,7 +112,7 @@ void ssh_leave_non_blocking(int fd)
     }
 }
 
-/* Restores terminal to blocking mode. */
+/* Puts terminal in non-blocking mode. */
 
 void ssh_enter_non_blocking(int fd)
 {
