@@ -4,7 +4,7 @@ wtmp.c
 
 Author: Tatu Ylonen <ylo@ssh.fi>
 
-Copyright (c) 1995-1998 SSH Communications Security Ltd, Espoo, Finland
+Copyright (c) 1995-2002 SSH Communications Security Ltd, Espoo, Finland
 All rights reserved
 
 Performs any logging that is normally performed when a user logs in or out.
@@ -82,19 +82,19 @@ SshTime ssh_user_get_last_login_time(SshUser user,
   char lastlogfile[500];
   int fd;
 
-#ifdef _PATH_LASTLOG
   ssh_snprintf(lastlogfile, sizeof(lastlogfile),
-               "%.200s/%.200s", _PATH_LASTLOG, ssh_user_name(user));
+               "%.200s/%.200s",
+#ifdef _PATH_LASTLOG
+               _PATH_LASTLOG, 
 #else /* _PATH_LASTLOG */
 #ifdef LASTLOG_FILE
-  ssh_snprintf(lastlogfile, sizeof(lastlogfile),
-               "%.200s/%.200s", LASTLOG_FILE, ssh_user_name(user));
+               LASTLOG_FILE,
 #else /* LASTLOG_FILE */
-  ssh_snprintf(lastlogfile, sizeof(lastlogfile),
-               "%.200s/%.200s", SSH_LASTLOG, ssh_user_name(user));
+               SSH_LASTLOG,
 #endif /* LASTLOG_FILE */
 #endif /* _PATH_LASTLOG */
-
+               ssh_user_name(user));
+  
   strcpy(hostbuf, "");
 
   fd = open(lastlogfile, O_RDONLY);
@@ -223,7 +223,7 @@ void ssh_user_record_login(SshUser user, pid_t pid, const char *ttyname,
 
   /* Construct an utmp/wtmp entry. */
   memset(&u, 0, sizeof(u));
-# ifdef DEAD_PROCESS
+# if defined(DEAD_PROCESS) && defined(HAVE_TYPE_IN_UTMP)
   if (user == NULL)
     u.ut_type = DEAD_PROCESS; /* logout */
   else
