@@ -15,14 +15,9 @@
 #include "sshadt_i.h"
 #include "sshdebug.h"
 
-#ifdef _KERNEL
-#include "sshadt_array.h"           /* Because of SSH_ADT_ARRAY */
-#include "sshadt_priority_queue.h"  /* Because of SSH_ADT_PRIORITY_QUEUE */
-#include "sshadt_priority_heap.h"   /* Because of SSH_ADT_PRIORITY_HEAP */
-#endif /* _KERNEL */
-
 #define SSH_DEBUG_MODULE "SshADT"
 
+
 /******************************************************** Default callbacks. */
 
 int ssh_adt_default_compare(const void *obj1, const void *obj2,
@@ -42,7 +37,6 @@ void ssh_adt_default_copy(void *dst, size_t d_size,
                           const void *src, void *context)
 {
   SSH_DEBUG(9, ("Copying %d bytes from %p to %p.", d_size));
-
   memcpy(dst, src, d_size);
 }
 
@@ -56,14 +50,12 @@ void ssh_adt_default_init(void *obj, size_t size, void *context)
   memset(obj, 0, size);
 }
 
-void ssh_adt_no_init(void *data, void *context)
-{
-  return;
-}
-
 SshUInt32 ssh_adt_default_hash(const void *obj, void *context)
 {
-  return (SshUInt32)obj;
+  /* This kludge is here to remove a compilation warning on 64-bit
+     platforms. */
+  unsigned long ptr_num = (unsigned long) obj;
+  return (SshUInt32) ptr_num;
 }
 
 
@@ -77,9 +69,9 @@ static void set_default_values(SshADTStandardFields *f)
   f->app_methods.destr         = ssh_adt_default_destroy;
   f->app_methods.init          = ssh_adt_default_init;
   f->app_methods.hash          = ssh_adt_default_hash;
-  f->app_methods.cleanup       = NULL;
-  f->app_methods.map_attach    = NULL;
-  f->app_methods.map_detach    = NULL;
+  f->app_methods.cleanup       = NULL_FNPTR;
+  f->app_methods.map_attach    = NULL_FNPTR;
+  f->app_methods.map_detach    = NULL_FNPTR;
   f->app_methods.context       = NULL;
 }
 
@@ -88,7 +80,7 @@ static Boolean init_toplevel_container(SshADTContainer c,
                                        va_list args)
 {
   SshADTArgumentType t;
-  SshADTContainerPars pars, *ptr;
+  SshADTContainerParsStruct pars, *ptr;
   SshADTStaticData *static_data;
 
   SSH_PRECOND(type != NULL);
@@ -179,10 +171,10 @@ static Boolean init_toplevel_container(SshADTContainer c,
     }
 
 #ifdef _KERNEL
-  SSH_ASSERT((ptr->flags & SSH_ADT_FLAG_CONTAINED_HEADER) ||
-             (pars.type == SSH_ADT_ARRAY) ||
-             (pars.type == SSH_ADT_PRIORITY_QUEUE) ||
-             (pars.type == SSH_ADT_PRIORITY_HEAP));
+  /* In kernel mode, objects must be concrete.  (Also, if there is a
+     header structure at all, it must be inlined.  This is checked in
+     the implementations because at least array does not need headers
+     and thus this restriction is void.)  */
   SSH_ASSERT(!(ptr->flags & SSH_ADT_FLAG_ALLOCATE));
 #endif
 
@@ -252,33 +244,6 @@ void *ssh_adt_duplicate_object(SshADTContainer container, void *object)
   return result;
 }
 
-/************************* Genlist functions, for some internal use perhaps. */
-
-void ssh_adt_genlist_merge(void *from_list, void **to_list,
-                           int next_ptr_offset)
-{
-  while (*to_list != NULL)
-    {
-      *to_list = (unsigned char *)(*to_list) + next_ptr_offset;
-    }
-}
-
-void ssh_adt_genlist_add(void **listptr, void *object, int next_ptr_offset)
-{
-  unsigned char *o = object;
-  *((void **)(o + next_ptr_offset)) = *listptr;
-  *listptr = object;
-}
-
-void ssh_adt_genlist_remove(void **listptr, void *object, int next_ptr_offset)
-{
-  while ((*listptr) != object)
-    {
-      listptr = (void **)((unsigned char *)(*listptr) + next_ptr_offset);
-    }
-  *listptr = *((void **)((unsigned char *)object + next_ptr_offset));
-}
-
 /******************************************************************** Hooks. */
 
 int ssh_adt_initialize_hooks(SshADTContainer container)
@@ -287,12 +252,12 @@ int ssh_adt_initialize_hooks(SshADTContainer container)
   if (!(container->hooks = ssh_malloc(sizeof(*container->hooks))))
     return 1;
 
-  container->hooks->insert      = NULL;
-  container->hooks->detach      = NULL;
-  container->hooks->map         = NULL;
-  container->hooks->unmap       = NULL;
-  container->hooks->reallocated = NULL;
-  container->hooks->destr       = NULL;
+  container->hooks->insert      = NULL_FNPTR;
+  container->hooks->detach      = NULL_FNPTR;
+  container->hooks->map         = NULL_FNPTR;
+  container->hooks->unmap       = NULL_FNPTR;
+  container->hooks->reallocated = NULL_FNPTR;
+  container->hooks->destr       = NULL_FNPTR;
   return 0;
 }
 
@@ -312,9 +277,9 @@ void ssh_adt_uninitialize_hooks(SshADTContainer container)
 
 #define SSH_ADT_ASSERT_CONTAINER SSH_PRECOND(container != NULL)
 #define SSH_ADT_ASSERT_EXTERNAL  \
-SSH_PRECOND(SSH_ADT_DEFAULT_SIZE(container) == 0)
+  SSH_PRECOND(SSH_ADT_DEFAULT_SIZE(container) == 0)
 #define SSH_ADT_ASSERT_INTERNAL  \
-SSH_PRECOND(SSH_ADT_DEFAULT_SIZE(container) != 0)
+  SSH_PRECOND(SSH_ADT_DEFAULT_SIZE(container) != 0)
 #define SSH_ADT_ASSERT_HANDLE    SSH_PRECOND(handle != SSH_ADT_INVALID)
 #define SSH_ADT_ASSERT_OBJECT    SSH_PRECOND(object != NULL)
 
@@ -341,7 +306,7 @@ SshADTHandle ssh_adt_insert_at(SshADTContainer container,
   SSH_ADT_ASSERT_HANDLE;
   if (container->flags & SSH_ADT_FLAG_CONTAINED_HEADER)
     SSH_ADT_ASSERT_OBJECT;
-  SSH_ASSERT(container->static_data->methods.insert_at);
+  SSH_ASSERT(container->static_data->methods.insert_at != NULL_FNPTR);
   return ssh_adt_insert_at__(container, location, handle, object);
 }
 
@@ -353,7 +318,7 @@ SshADTHandle ssh_adt_insert_to(SshADTContainer container,
   SSH_ADT_ASSERT_EXTERNAL;
   if (container->flags & SSH_ADT_FLAG_CONTAINED_HEADER)
     SSH_ADT_ASSERT_OBJECT;
-  SSH_ASSERT(container->static_data->methods.insert_to);
+  SSH_ASSERT(container->static_data->methods.insert_to != NULL_FNPTR);
   return ssh_adt_insert_to__(container, location, object);
 }
 
@@ -364,7 +329,7 @@ SshADTHandle ssh_adt_insert(SshADTContainer container,
   SSH_ADT_ASSERT_EXTERNAL;
   if (container->flags & SSH_ADT_FLAG_CONTAINED_HEADER)
     SSH_ADT_ASSERT_OBJECT;
-  SSH_ASSERT(container->static_data->methods.insert_to);
+  SSH_ASSERT(container->static_data->methods.insert_to != NULL_FNPTR);
   return ssh_adt_insert_to(container, SSH_ADT_DEFAULT, object);
 }
 
@@ -377,7 +342,7 @@ SshADTHandle ssh_adt_duplicate_at(SshADTContainer container,
   SSH_ADT_ASSERT_EXTERNAL;
   if (container->flags & SSH_ADT_FLAG_CONTAINED_HEADER)
     SSH_ADT_ASSERT_OBJECT;
-  SSH_ASSERT(container->static_data->methods.insert_at);
+  SSH_ASSERT(container->static_data->methods.insert_at != NULL_FNPTR);
   return ssh_adt_insert_at(container, location, handle,
                            ssh_adt_duplicate_object(container, object));
 }
@@ -390,7 +355,7 @@ SshADTHandle ssh_adt_duplicate_to(SshADTContainer container,
   SSH_ADT_ASSERT_EXTERNAL;
   if (container->flags & SSH_ADT_FLAG_CONTAINED_HEADER)
     SSH_ADT_ASSERT_OBJECT;
-  SSH_ASSERT(container->static_data->methods.insert_to);
+  SSH_ASSERT(container->static_data->methods.insert_to != NULL_FNPTR);
   return ssh_adt_insert_to(container, location,
                            ssh_adt_duplicate_object(container, object));
 }
@@ -402,13 +367,11 @@ SshADTHandle ssh_adt_duplicate(SshADTContainer container,
   SSH_ADT_ASSERT_EXTERNAL;
   if (container->flags & SSH_ADT_FLAG_CONTAINED_HEADER)
     SSH_ADT_ASSERT_OBJECT;
-  SSH_ASSERT(container->static_data->methods.insert_to);
+  SSH_ASSERT(container->static_data->methods.insert_to != NULL_FNPTR);
   return ssh_adt_insert(container,
                         ssh_adt_duplicate_object(container, object));
 }
 
-/* 2. Internally allocated objects. */
-
 SshADTHandle ssh_adt_alloc_n_at(SshADTContainer container,
                                 SshADTRelativeLocation location,
                                 SshADTHandle handle,
@@ -417,7 +380,7 @@ SshADTHandle ssh_adt_alloc_n_at(SshADTContainer container,
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
   SSH_ADT_ASSERT_HANDLE;
-  SSH_ASSERT(container->static_data->methods.alloc_n_at);
+  SSH_ASSERT(container->static_data->methods.alloc_n_at != NULL_FNPTR);
   return ssh_adt_alloc_n_at__(container, location, handle, size);
 }
 
@@ -427,7 +390,7 @@ SshADTHandle ssh_adt_alloc_n_to(SshADTContainer container,
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
-  SSH_ASSERT(container->static_data->methods.alloc_n_to);
+  SSH_ASSERT(container->static_data->methods.alloc_n_to != NULL_FNPTR);
   return ssh_adt_alloc_n_to__(container, location, size);
 }
 
@@ -438,7 +401,7 @@ SshADTHandle ssh_adt_alloc_at(SshADTContainer container,
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
   SSH_ADT_ASSERT_HANDLE;
-  SSH_ASSERT(container->static_data->methods.alloc_n_at);
+  SSH_ASSERT(container->static_data->methods.alloc_n_at != NULL_FNPTR);
   return ssh_adt_alloc_n_at(container, location,
                             handle, SSH_ADT_DEFAULT_SIZE(container));
 }
@@ -448,7 +411,7 @@ SshADTHandle ssh_adt_alloc_to(SshADTContainer container,
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
-  SSH_ASSERT(container->static_data->methods.alloc_n_to);
+  SSH_ASSERT(container->static_data->methods.alloc_n_to != NULL_FNPTR);
   return ssh_adt_alloc_n_to(container, location,
                             SSH_ADT_DEFAULT_SIZE(container));
 }
@@ -458,7 +421,7 @@ SshADTHandle ssh_adt_alloc_n(SshADTContainer container,
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
-  SSH_ASSERT(container->static_data->methods.alloc_n_to);
+  SSH_ASSERT(container->static_data->methods.alloc_n_to != NULL_FNPTR);
   return ssh_adt_alloc_n_to(container, SSH_ADT_DEFAULT, size);
 }
 
@@ -466,7 +429,7 @@ SshADTHandle ssh_adt_alloc(SshADTContainer container)
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
-  SSH_ASSERT(container->static_data->methods.alloc_n_to);
+  SSH_ASSERT(container->static_data->methods.alloc_n_to != NULL_FNPTR);
   return ssh_adt_alloc_n(container, SSH_ADT_DEFAULT_SIZE(container));
 }
 
@@ -480,7 +443,7 @@ SshADTHandle ssh_adt_put_n_at(SshADTContainer container,
   SSH_ADT_ASSERT_INTERNAL;
   SSH_ADT_ASSERT_HANDLE;
   SSH_ADT_ASSERT_OBJECT;
-  SSH_ASSERT(container->static_data->methods.put_n_at);
+  SSH_ASSERT(container->static_data->methods.put_n_at != NULL_FNPTR);
   return ssh_adt_put_n_at__(container, location, handle, size, object);
 }
 
@@ -492,7 +455,7 @@ SshADTHandle ssh_adt_put_n_to(SshADTContainer container,
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
   SSH_ADT_ASSERT_OBJECT;
-  SSH_ASSERT(container->static_data->methods.put_n_to);
+  SSH_ASSERT(container->static_data->methods.put_n_to != NULL_FNPTR);
   return ssh_adt_put_n_to__(container, location, size, object);
 }
 
@@ -505,7 +468,7 @@ SshADTHandle ssh_adt_put_at(SshADTContainer container,
   SSH_ADT_ASSERT_INTERNAL;
   SSH_ADT_ASSERT_HANDLE;
   SSH_ADT_ASSERT_OBJECT;
-  SSH_ASSERT(container->static_data->methods.put_n_at);
+  SSH_ASSERT(container->static_data->methods.put_n_at != NULL_FNPTR);
   return ssh_adt_put_n_at(container, location, handle,
                           SSH_ADT_DEFAULT_SIZE(container), object);
 }
@@ -517,7 +480,7 @@ SshADTHandle ssh_adt_put_to(SshADTContainer container,
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
   SSH_ADT_ASSERT_OBJECT;
-  SSH_ASSERT(container->static_data->methods.put_n_to);
+  SSH_ASSERT(container->static_data->methods.put_n_to != NULL_FNPTR);
   return ssh_adt_put_n_to(container, location,
                           SSH_ADT_DEFAULT_SIZE(container), object);
 }
@@ -529,7 +492,7 @@ SshADTHandle ssh_adt_put_n(SshADTContainer container,
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
   SSH_ADT_ASSERT_OBJECT;
-  SSH_ASSERT(container->static_data->methods.put_n_to);
+  SSH_ASSERT(container->static_data->methods.put_n_to != NULL_FNPTR);
   return ssh_adt_put_n_to(container, SSH_ADT_DEFAULT, size, object);
 }
 
@@ -539,7 +502,7 @@ SshADTHandle ssh_adt_put(SshADTContainer container,
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_INTERNAL;
   SSH_ADT_ASSERT_OBJECT;
-  SSH_ASSERT(container->static_data->methods.put_n_to);
+  SSH_ASSERT(container->static_data->methods.put_n_to != NULL_FNPTR);
   return ssh_adt_put_n(container, SSH_ADT_DEFAULT_SIZE(container), object);
 }
 
@@ -547,14 +510,14 @@ void *ssh_adt_get(SshADTContainer container, SshADTHandle handle)
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_HANDLE;
-  SSH_ASSERT(container->static_data->methods.get);
+  SSH_ASSERT(container->static_data->methods.get != NULL_FNPTR);
   return ssh_adt_get__(container, handle);
 }
 
 size_t ssh_adt_num_objects(SshADTContainer container)
 {
   SSH_ADT_ASSERT_CONTAINER;
-  SSH_ASSERT(container->static_data->methods.num_objects);
+  SSH_ASSERT(container->static_data->methods.num_objects != NULL_FNPTR);
   return ssh_adt_num_objects__(container);
 }
 
@@ -565,7 +528,7 @@ SshADTHandle ssh_adt_get_handle_to(SshADTContainer container,
   if (container->flags & (SSH_ADT_FLAG_CONTAINED_HEADER |
                           SSH_ADT_FLAG_ALLOCATE))
     SSH_ADT_ASSERT_OBJECT;
-  SSH_ASSERT(container->static_data->methods.get_handle_to);
+  SSH_ASSERT(container->static_data->methods.get_handle_to != NULL_FNPTR);
   return ssh_adt_get_handle_to__(container, object);
 }
 
@@ -576,15 +539,29 @@ SshADTHandle ssh_adt_get_handle_to_equal(SshADTContainer container,
   if (container->flags & (SSH_ADT_FLAG_CONTAINED_HEADER |
                           SSH_ADT_FLAG_ALLOCATE))
     SSH_ADT_ASSERT_OBJECT;
-  SSH_ASSERT(container->static_data->methods.get_handle_to_equal);
+  SSH_ASSERT(container->static_data->methods.get_handle_to_equal !=
+             NULL_FNPTR);
   return ssh_adt_get_handle_to_equal__(container, object);
 }
 
+void *ssh_adt_get_object_from_equal(SshADTContainer container,
+                                    void *object)
+{
+  SSH_ADT_ASSERT_CONTAINER;
+  if (container->flags & (SSH_ADT_FLAG_CONTAINED_HEADER |
+                          SSH_ADT_FLAG_ALLOCATE))
+    SSH_ADT_ASSERT_OBJECT;
+  SSH_ASSERT(container->static_data->methods.get_handle_to_equal !=
+             NULL_FNPTR);
+  return ssh_adt_get(container,
+                     ssh_adt_get_handle_to_equal__(container, object));
+}
+
 SshADTHandle ssh_adt_next(SshADTContainer container, SshADTHandle handle)
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_HANDLE;
-  SSH_ASSERT(container->static_data->methods.next);
+  SSH_ASSERT(container->static_data->methods.next != NULL_FNPTR);
   return ssh_adt_next__(container, handle);
 }
 
@@ -592,7 +569,7 @@ SshADTHandle ssh_adt_previous(SshADTContainer container, SshADTHandle handle)
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_HANDLE;
-  SSH_ASSERT(container->static_data->methods.previous);
+  SSH_ASSERT(container->static_data->methods.previous != NULL_FNPTR);
   return ssh_adt_previous__(container, handle);
 }
 
@@ -600,7 +577,8 @@ SshADTHandle ssh_adt_get_handle_to_location(SshADTContainer container,
                                             SshADTAbsoluteLocation location)
 {
   SSH_ADT_ASSERT_CONTAINER;
-  SSH_ASSERT(container->static_data->methods.get_handle_to_location);
+  SSH_ASSERT(container->static_data->methods.get_handle_to_location !=
+             NULL_FNPTR);
   return ssh_adt_get_handle_to_location__(container, location);
 }
 
@@ -608,9 +586,10 @@ void *ssh_adt_get_object_from_location(SshADTContainer container,
                                        SshADTAbsoluteLocation location)
 {
   SSH_ADT_ASSERT_CONTAINER;
-  SSH_ASSERT(container->static_data->methods.get_handle_to_location);
+  SSH_ASSERT(container->static_data->methods.get_handle_to_location !=
+             NULL_FNPTR);
   return ssh_adt_get(container,
-                     ssh_adt_get_handle_to_location(container, location));
+                     ssh_adt_get_handle_to_location__(container, location));
 }
 
 void *ssh_adt_detach(SshADTContainer container, SshADTHandle handle)
@@ -618,20 +597,7 @@ void *ssh_adt_detach(SshADTContainer container, SshADTHandle handle)
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_EXTERNAL;
   SSH_ADT_ASSERT_HANDLE;
-  SSH_ASSERT(container->static_data->methods.detach);
-  return ssh_adt_detach__(container, handle);
-}
-
-/* This is called from inside the container implementations when
-   objects are detached just prior to destroying them.  In this case
-   we need not require that objects are external because no user will
-   ever get hold on them.  (See the sshadt_std_i.h::SSH_ADT_STD_DELETE
-   macro.)  */
-void *ssh_adt_detach_i(SshADTContainer container, SshADTHandle handle)
-{
-  SSH_ADT_ASSERT_CONTAINER;
-  SSH_ADT_ASSERT_HANDLE;
-  SSH_ASSERT(container->static_data->methods.detach);
+  SSH_ASSERT(container->static_data->methods.detach != NULL_FNPTR);
   return ssh_adt_detach__(container, handle);
 }
 
@@ -641,8 +607,9 @@ void *ssh_adt_detach_from(SshADTContainer container,
   SshADTHandle handle;
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_EXTERNAL;
-  SSH_ASSERT(container->static_data->methods.detach);
-  SSH_ASSERT(container->static_data->methods.get_handle_to_location);
+  SSH_ASSERT(container->static_data->methods.detach != NULL_FNPTR);
+  SSH_ASSERT(container->static_data->methods.get_handle_to_location !=
+             NULL_FNPTR);
   handle = ssh_adt_get_handle_to_location(container, location);
   SSH_ADT_ASSERT_HANDLE;
   return ssh_adt_detach(container, handle);
@@ -657,8 +624,8 @@ void *ssh_adt_detach_object(SshADTContainer container,
   if (container->flags & (SSH_ADT_FLAG_CONTAINED_HEADER |
                           SSH_ADT_FLAG_ALLOCATE))
     SSH_ADT_ASSERT_OBJECT;
-  SSH_ASSERT(container->static_data->methods.detach);
-  SSH_ASSERT(container->static_data->methods.get_handle_to);
+  SSH_ASSERT(container->static_data->methods.detach != NULL_FNPTR);
+  SSH_ASSERT(container->static_data->methods.get_handle_to != NULL_FNPTR);
   handle = ssh_adt_get_handle_to(container, object);
   SSH_ADT_ASSERT_HANDLE;
   return ssh_adt_detach(container, handle);
@@ -668,7 +635,7 @@ void ssh_adt_delete(SshADTContainer container, SshADTHandle handle)
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_HANDLE;
-  SSH_ASSERT(container->static_data->methods.delet);
+  SSH_ASSERT(container->static_data->methods.delet != NULL_FNPTR);
   ssh_adt_delete__(container, handle);
 }
 
@@ -677,8 +644,9 @@ void ssh_adt_delete_from(SshADTContainer container,
 {
   SshADTHandle handle;
   SSH_ADT_ASSERT_CONTAINER;
-  SSH_ASSERT(container->static_data->methods.delet);
-  SSH_ASSERT(container->static_data->methods.get_handle_to_location);
+  SSH_ASSERT(container->static_data->methods.delet != NULL_FNPTR);
+  SSH_ASSERT(container->static_data->methods.get_handle_to_location !=
+             NULL_FNPTR);
   handle = ssh_adt_get_handle_to_location(container, location);
   SSH_ADT_ASSERT_HANDLE;
   ssh_adt_delete(container, handle);
@@ -692,8 +660,8 @@ void ssh_adt_delete_object(SshADTContainer container,
   if (container->flags & (SSH_ADT_FLAG_CONTAINED_HEADER |
                           SSH_ADT_FLAG_ALLOCATE))
     SSH_ADT_ASSERT_OBJECT;
-  SSH_ASSERT(container->static_data->methods.delet);
-  SSH_ASSERT(container->static_data->methods.get_handle_to);
+  SSH_ASSERT(container->static_data->methods.delet != NULL_FNPTR);
+  SSH_ASSERT(container->static_data->methods.get_handle_to != NULL_FNPTR);
   handle = ssh_adt_get_handle_to(container, object);
   SSH_ADT_ASSERT_HANDLE;
   ssh_adt_delete(container, handle);
@@ -702,7 +670,7 @@ void ssh_adt_delete_object(SshADTContainer container,
 SshADTHandle ssh_adt_enumerate_start(SshADTContainer container)
 {
   SSH_ADT_ASSERT_CONTAINER;
-  SSH_ASSERT(container->static_data->methods.enumerate_start);
+  SSH_ASSERT(container->static_data->methods.enumerate_start != NULL_FNPTR);
   return ssh_adt_enumerate_start__(container);
 }
 
@@ -711,7 +679,7 @@ SshADTHandle ssh_adt_enumerate_next(SshADTContainer container,
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_HANDLE;
-  SSH_ASSERT(container->static_data->methods.enumerate_next);
+  SSH_ASSERT(container->static_data->methods.enumerate_next != NULL_FNPTR);
   return ssh_adt_enumerate_next__(container, handle);
 }
 
@@ -719,7 +687,7 @@ void *ssh_adt_map_lookup(SshADTContainer container, SshADTHandle handle)
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_HANDLE;
-  SSH_ASSERT(container->static_data->methods.map_lookup);
+  SSH_ASSERT(container->static_data->methods.map_lookup != NULL_FNPTR);
   return ssh_adt_map_lookup__(container, handle);
 }
 
@@ -728,90 +696,18 @@ void ssh_adt_map_attach(SshADTContainer container,
 {
   SSH_ADT_ASSERT_CONTAINER;
   SSH_ADT_ASSERT_HANDLE;
-  SSH_ASSERT(container->static_data->methods.map_attach);
+  SSH_ASSERT(container->static_data->methods.map_attach != NULL_FNPTR);
   ssh_adt_map_attach__(container, handle, obj);
 }
 
+/* (for ssh_adt_detach_i, see comment in sshadt_i.h) */
 
-#endif /* !SSH_ADT_WITH_MACRO_INTERFACE */
-
-/************************************************* duplication of containers */
-
-/* This creates a new container from an old one, independently of the
-   latters class.  It simply walks through an enumeration loop and
-   inserts all elements to the default location.  It might work on new
-   container classes immediately.
-
-   The hooks and callbacks are copied as function pointers, so doing
-   weird stuff with static variables will probably not work.  Callback
-   and hook contexts need to be NULL in the duplicated container.  */
-
-
-#define FAIL(a)                         \
-do                                      \
-{                                       \
- SSH_DEBUG(3, ("failed: %s.", a));      \
- ssh_free(c2);                      \
- return NULL;                           \
-}                                       \
-while(0)
-
-SshADTContainer ssh_adt_duplicate_container(SshADTContainer c1)
+void *ssh_adt_detach_i(SshADTContainer container, SshADTHandle handle)
 {
-  SshADTContainer c2;
-  SshADTContainerPars pars;
-  SshADTHandle h;
-
-  /* init a new container */
-  if (!(c2 = ssh_malloc(sizeof(*c2))))
-    return NULL;
-
-  /* init pars (the structure is defined in sshadt_i.h) */
-
-  /* class type & information (including methods) */
-  pars.type = (SshADTContainerType) c1->static_data;
-
-  /* the instance flags */
-  pars.flags = c1->flags;
-
-  /* callbacks and memory management setting */
-  pars.f = c1->f;
-  SSH_ASSERT(pars.f.app_methods.context == NULL);
-
-  /* call the init function of the appropriate type */
-  if ((*(((SshADTStaticData *) pars.type)->methods.container_init))(c2, &pars)
-      == FALSE)
-    FAIL("create");
-
-  /* hooks */
-  if (c1->hooks)
-    {
-      SSH_ASSERT(c1->hooks->reallocated_ctx == NULL);
-      SSH_ASSERT(c1->hooks->destr_ctx == NULL);
-      c2->hooks = c1->hooks;
-    }
-
-  /* copy elements */
-  if ((h = ssh_adt_enumerate_start(c1)) != SSH_ADT_INVALID)
-    {
-      do {
-        if (c1->flags & SSH_ADT_FLAG_ALLOCATE)      /* liballoc */
-          {
-            void *o = ssh_adt_get(c1, h);
-            SshADTHandle b = ssh_adt_put(c2, o);
-            if (b == SSH_ADT_INVALID) FAIL("insert, liballoc");
-          }
-        else                                        /* voidptr */
-          {
-            void *o = ssh_adt_get(c1, h);
-            SshADTHandle h = ssh_adt_duplicate(c2, o);
-            if (h == SSH_ADT_INVALID) FAIL("insert, voidptr");
-          }
-      }
-      while ((h = ssh_adt_enumerate_next(c1, h)) != SSH_ADT_INVALID);
-    }
-
-  return c2;
+  SSH_ADT_ASSERT_CONTAINER;
+  SSH_ADT_ASSERT_HANDLE;
+  SSH_ASSERT(container->static_data->methods.detach != NULL_FNPTR);
+  return ssh_adt_detach__(container, handle);
 }
 
-#undef FAIL
+#endif /* !SSH_ADT_WITH_MACRO_INTERFACE */
