@@ -20,6 +20,7 @@
 #define SSH_DEBUG_MODULE "SshADTAvlTree"
 
 #include "sshadt.h"
+#include "sshbuffer.h"
 #include "sshadt_i.h"
 #include "sshadt_std_i.h"
 #include "sshadt_avltree.h"
@@ -27,11 +28,17 @@
 #include "sshadt_avltree_i.h"
 
 
-/************************************************************** construction */
+/**************************************************  Container construction. */
 
-static Boolean avl_init(SshADTContainer c)
+static Boolean avl_init(SshADTContainer c, void *container_specific)
 {
-  if (!(c->container_specific = ssh_malloc(sizeof(SshADTAvlCRootStruct))))
+# ifdef _KERNEL
+  SSH_ASSERT((c->flags & SSH_ADT_FLAG_CONTAINED_HEADER));
+# endif
+
+  /* For some reason, Irix CC doesn't like casts on the left side, so don't
+     use the CROOT() macro here. Please, don't. //sjl */
+  if (!(c->container_specific = container_specific))
     return FALSE;
 
   ROOT(c) = NULL;
@@ -45,35 +52,48 @@ static Boolean avl_init(SshADTContainer c)
   return TRUE;
 }
 
+static Boolean avl_ranges_init(SshADTContainer c)
+{
+  if (!avl_init(c, ssh_malloc(sizeof(SshADTRangesCRootStruct))))
+    return FALSE;
+
+  if ((RROOT(c)->merge_additions = ssh_buffer_allocate()) == NULL ||
+      (RROOT(c)->merge_deletions = ssh_buffer_allocate()) == NULL)
+    return FALSE;
+
+  return TRUE;
+}
+
 /* $$METHOD(avltree, container_init) */
+SSH_ADT_STD_INIT(container_init_tree,
+                 avl_init(c, ssh_malloc(sizeof(SshADTAvlCRootStruct))))
 /* $$METHOD(ranges, container_init) */
-SSH_ADT_STD_INIT(container_init, avl_init(c))
+SSH_ADT_STD_INIT(container_init_ranges, avl_ranges_init(c))
 
 
-/***************************************************************** insertion */
+/********************************************************* Object insertion. */
 
 static Boolean avl_insert_(SshADTContainer c,
                            SshADTAbsoluteLocation l,
-                           SshADTHandle h,
-                           Boolean multiset);
+                           SshADTHandle h);
 
 /* $$METHOD(avltree, insert_to) */
 SSH_ADT_STD_INSERT_TO(avl_insert_to,
-                      avl_insert_(c, location, h, TRUE),
+                      avl_insert_(c, location, h),
                       __handle = ssh_malloc(sizeof(SshADTAvlENodeStruct));)
 
 /* $$METHOD(ranges, insert_to) */
 SSH_ADT_STD_INSERT_TO(avl_insert_to_ranges,
-                      avl_insert_(c, location, h, FALSE),
+                      avl_insert_(c, location, h),
                       __handle = ssh_malloc(sizeof(SshADTAvlENodeStruct));)
 
 /* $$METHOD(avltree, put_n_to) */
 SSH_ADT_STD_PUT_N_TO(avl_put_to,
-                     avl_insert_(c, location, h, TRUE);)
+                     avl_insert_(c, location, h);)
 
 /* $$METHOD(ranges, put_n_to) */
 SSH_ADT_STD_PUT_N_TO(avl_put_to_ranges,
-                     avl_insert_(c, location, h, FALSE);)
+                     avl_insert_(c, location, h);)
 
 
 /* Now for the algorithmic stuff: given a handle containing an object,
@@ -320,19 +340,11 @@ static void avl_balance_i(SshADTContainer c,
 
 /* Inject a new node into the tree.  The node appears to be of type
    SshADTHandle but actually is a Node.  When this function returns,
-   the tree is consistent and balanced.
-
-   multiset: if TRUE, equal nodes are allowed.  If False and if the
-   node to be inserted is already in the tree (the compare callback
-   decides what is equal), it is silently discared.
-
-   NOTE: if you add a node with ssh_adt_insert and continue using the
-   pointer to that node, you are in deep trouble.  */
+   the tree is consistent and balanced.  */
 
 static Boolean avl_insert_(SshADTContainer c,
                            SshADTAbsoluteLocation location,
-                           SshADTHandle new_handle,
-                           Boolean multiset)
+                           SshADTHandle new_handle)
 {
   /* There are two pointers for traversing the tree in addition to
      that to the new object.  */
@@ -349,7 +361,7 @@ static Boolean avl_insert_(SshADTContainer c,
   AVL_SET_SKEW(new_node, AVL_BALANCED);
   new_node->left  = NULL;
   new_node->right = NULL;
-  new_node->image = NULL;
+  new_node->image = NULL;  /* the mapping value */
 
   if (ROOT(c) == NULL)
     {
@@ -362,7 +374,7 @@ static Boolean avl_insert_(SshADTContainer c,
     {
       /* find location of new node */
       this_node = ROOT(c);
-      SSH_ASSERT(this_node);
+      SSH_ASSERT(this_node != NULL);
       do
         {
           parent_node = this_node;
@@ -384,10 +396,6 @@ static Boolean avl_insert_(SshADTContainer c,
           AVL_SET_CHILD_SLOT(new_node, AVL_RIGHT_CHILD);
           avl_balance_i(c, parent_node, AVL_RIGHT_CHILD);
         }
-      else if (!multiset && cmp == 0)  /* XXX - this issue is far from clear.  */
-        {
-          return FALSE;
-        }
       else
         {
           parent_node->left = new_node;
@@ -402,14 +410,14 @@ static Boolean avl_insert_(SshADTContainer c,
 }
 
 
-/***************************************************************** retrieval */
+/********************************************************* Object retrieval. */
 
 static SshADTHandle avl_geth_(SshADTContainer c, void *o)
 {
   SshADTHandle h;
   void *result;
 
-  SSH_DEBUG(6, ("*** Linear runtime alert."));
+  SSH_DEBUG(0, ("*** Linear runtime alert."));
 
   h = ssh_adt_enumerate_start(c);
   while(h != SSH_ADT_INVALID)
@@ -473,9 +481,9 @@ static SshADTHandle avl_geth_loc(SshADTContainer c,
          enumerate the first l elements and return the last.  */
 
       SshADTHandle h;
-      int i = 0, k = SSH_ADT_INDEX(l);
+      int i = 0, k = SSH_ADT_GET_INDEX(l);
 
-      SSH_DEBUG(6, ("*** Linear runtime alert."));
+      SSH_DEBUG(0, ("*** Linear runtime alert."));
       SSH_ASSERT(k >= 0);
 
       h = ssh_adt_enumerate_start(c);
@@ -596,8 +604,7 @@ static SshADTHandle avl_previous(SshADTContainer c, SshADTHandle h)
 SSH_ADT_STD_NUM_OBJECTS(avl_num_objects)
 
 
-/****************************************************** destruction (object) */
-
+/******************************************************* Object destruction. */
 
 static void avl_detach_(SshADTContainer c, SshADTHandle h);
 
@@ -607,7 +614,7 @@ SSH_ADT_STD_DETACH(avl_detach, avl_detach_(c, handle);, ssh_free(node);)
 /* $$METHOD(ranges, detach) */
 static void *avl_detach_range(SshADTContainer c, SshADTHandle handle)
 {
-  merge_detach(c, handle);
+  merge_detach_prepare(c, handle);
   return avl_detach(c, handle);
 }
 
@@ -837,7 +844,7 @@ static void avl_detach_(SshADTContainer c, SshADTHandle h)
 }
 
 
-/*************************************************** destruction (container) */
+/**************************************************** Container destruction. */
 
 /* $$METHOD(avltree, clear) */
 /* $$METHOD(ranges, clear) */
@@ -852,10 +859,10 @@ static void avlclear(SshADTContainer c)
 
 /* $$METHOD(avltree, destr) */
 /* $$METHOD(ranges, destr) */
-SSH_ADT_STD_DESTROY(destr, avlclear(c);)
+SSH_ADT_STD_DESTROY(destr, avlclear(c);ssh_free(CROOT(c));)
 
 
-/******************************************************************* mapping */
+/****************************************************************** Mapping. */
 
 /* $$METHOD(avltree, map_attach) */
 SSH_ADT_STD_MAP_ATTACH(avl_map_attach, ((SshADTAvlNode)handle)->image)
@@ -864,7 +871,7 @@ SSH_ADT_STD_MAP_ATTACH(avl_map_attach, ((SshADTAvlNode)handle)->image)
 static void avl_map_attach_ranges(SshADTContainer c, SshADTHandle h, void *i)
 {
   avl_map_attach(c, h, i);
-  merge_map_attach(c, h);
+  merge_map_attach_prepare(c, h);
 }
 
 /* $$METHOD(avltree, map_lookup) */
@@ -872,19 +879,17 @@ static void avl_map_attach_ranges(SshADTContainer c, SshADTHandle h, void *i)
 SSH_ADT_STD_MAP_LOOKUP(avl_map_lookup, ((SshADTAvlNode)handle)->image)
 
 
-/***************************************************** ranges specific stuff */
-
-/* Greatest lower bound of o (greatest n such that n <= o).  If there
-   is no such n, SSH_ADT_INVALID is returned.  o does not need to be
-   in the container (i.e. have a valid header structure attached to
-   it), it only needs to make the compare callback happy.  */
+/********************************** Find greatest lower / least upper bound. */
 
 SshADTHandle ssh_adt_get_handle_to_glb(SshADTContainer c, void *o)
 {
   SshADTAvlNode n = ROOT(c);
   int cmp;
 
-  SSH_ASSERT(c != NULL); SSH_ASSERT(o != NULL);
+  SSH_ASSERT(c != NULL);
+  SSH_ASSERT(c->static_data == SSH_ADT_AVLTREE ||
+             c->static_data == SSH_ADT_RANGES);
+  SSH_ASSERT(o != NULL);
 
   /* special case: empty tree */
   if (n == NULL) return SSH_ADT_INVALID;
@@ -893,7 +898,7 @@ SshADTHandle ssh_adt_get_handle_to_glb(SshADTContainer c, void *o)
   SSH_ADT_STD_COMPARE_H_O(c, n, o, cmp);
   if (cmp == 0) return (SshADTHandle)n;
 
-  /* default: find some n larger than o... */
+  /* default: find some n greater or equal than o... */
   while (cmp <= 0)
     {
       if (n->right == NULL)
@@ -908,25 +913,63 @@ SshADTHandle ssh_adt_get_handle_to_glb(SshADTContainer c, void *o)
 
   SSH_ASSERT(n != NULL);
 
-  /* ... move back until it is no longer greater than o... */
+  /* ... and move back until it is no longer greater than o. */
   while (cmp > 0)
     {
       n = (SshADTAvlNode)avl_previous(c, (SshADTHandle)n);
 
-      if (n == NULL)
-        {
-          /* (If no minium element is found, avl_previous will have
-             returned SSH_ADT_INVALID, which is just what we want.)  */
-          return (SshADTHandle)SSH_ADT_INVALID;
-        }
-
-      SSH_ADT_STD_COMPARE_H_O(c, n, o, cmp);
+      if (n == NULL)  /* (no minium element is found)  */
+        return (SshADTHandle)SSH_ADT_INVALID;
+      else
+        SSH_ADT_STD_COMPARE_H_O(c, n, o, cmp);
     }
 
-  /* ... and return.  */
   return (SshADTHandle)n;
 }
 
+/* Same for least upper bound.  */
+
+SshADTHandle ssh_adt_get_handle_to_lub(SshADTContainer c, void *o)
+{
+  SshADTAvlNode n = ROOT(c);
+  int cmp;
+
+  SSH_ASSERT(c != NULL);
+  SSH_ASSERT(c->static_data == SSH_ADT_AVLTREE ||
+             c->static_data == SSH_ADT_RANGES);
+  SSH_ASSERT(o != NULL);
+
+  if (n == NULL) return SSH_ADT_INVALID;
+
+  SSH_ADT_STD_COMPARE_H_O(c, n, o, cmp);
+  if (cmp == 0) return (SshADTHandle)n;
+
+  while (cmp >= 0)
+    {
+      if (n->left == NULL)
+        return (SshADTHandle)n;
+
+      n = n->left;
+      SSH_ADT_STD_COMPARE_H_O(c, n, o, cmp);
+    }
+
+  SSH_ASSERT(n != NULL);
+
+  while (cmp < 0)
+    {
+      n = (SshADTAvlNode)avl_next(c, (SshADTHandle)n);
+
+      if (n == NULL)
+        return (SshADTHandle)SSH_ADT_INVALID;
+      else
+        SSH_ADT_STD_COMPARE_H_O(c, n, o, cmp);
+    }
+
+  return (SshADTHandle)n;
+}
+
+
+/**************************************************** Ranges specific stuff. */
 
 /* Merging.  Merge the two neighboring intervals [a, b) and [b, c) if
    appropriate.  If a merge occurs, a is returned.  Otherwise, b is
@@ -937,10 +980,15 @@ static SshADTAvlNode merge_ab(SshADTContainer c,
 {
   void *ai, *bi;  /* images */
 
+  /* SSH_DEBUG(0, ("%p %p.", a, b));
+     ssh_adt_ranges_dump("merge_ab: entering!", c); */
+
+  SSH_ASSERT(a == NULL || avl_next(c, (SshADTHandle)a) == b);
+  SSH_ASSERT(b == NULL || avl_previous(c, (SshADTHandle)b) == a);
+
   if (b == NULL)
     {
-      /* if b does not exist, a extends naturally all the way to c.  */
-
+      /* if b does not exist, a extends all the way to +inf.  */
       return a;
     }
 
@@ -950,7 +998,6 @@ static SshADTAvlNode merge_ab(SshADTContainer c,
          and delete it in case it does.  */
 
       bi = ssh_adt_map_lookup(c, (SshADTHandle)b);
-
       if (bi == NULL)
         {
           avl_delete(c, (SshADTHandle)b);
@@ -970,10 +1017,10 @@ static SshADTAvlNode merge_ab(SshADTContainer c,
       if (ai == bi)
         {
           avl_delete(c, (SshADTHandle)b);
-          b = a;
+          return a;
         }
-
-      return a;
+      else
+        return b;
     }
 
   SSH_NOTREACHED;
@@ -981,17 +1028,18 @@ static SshADTAvlNode merge_ab(SshADTContainer c,
 }
 
 
-/* This function is called when range [N, M) has been mapped to val by
-   ssh_adt_map_attach.  If range [K, N) maps to val, [K, N) and [N, M)
+/* This function is called (ie. registered in the merge cache) when
+   range [N, M) has been mapped to a new value V by
+   ssh_adt_map_attach.  If range [K, N) maps to V, [K, N) and [N, M)
    are merged to [K, M); similarly for [M, L).  (-inf, min) is
-   implicitly mapped to NULL.  This means that if a smalles node is
-   inserted and mapped to NULL, it is discared immediately.  */
+   implicitly mapped to NULL.  This means that if a smallest node is
+   inserted and mapped to NULL, it is discared during the next merge.  */
 
 static Boolean merge_map_attach(SshADTContainer c, SshADTHandle h)
 {
-  SshADTAvlNode k, n, m;
+  SshADTAvlNode k = NULL, n = NULL, m = NULL;
 
-  SSH_DEBUG(99, ("merge_map_attach [start]"));
+  SSH_DEBUG(9, ("in."));
 
   n = (SshADTAvlNode)h;
   SSH_ASSERT(n != NULL);
@@ -1002,36 +1050,135 @@ static Boolean merge_map_attach(SshADTContainer c, SshADTHandle h)
   n = merge_ab(c, k, n);
   merge_ab(c, n, m);
 
-  SSH_DEBUG(99, ("merge_map_attach [done]"));
-
+  SSH_DEBUG(9, ("done."));
   return TRUE;
 }
 
 
 /* If [N, M) is going to be removed in a second, so [K, N) and [M, L)
    need to be merged if appropriate.  This operation leaves N
-   untouched.  It is triggered by ssh_adt_detach and thus also by
-   ssh_adt_delete.  */
+   untouched.  It is triggered (ie. prepared) by ssh_adt_detach and
+   thus also by ssh_adt_delete.  */
 
 static Boolean merge_detach(SshADTContainer c, SshADTHandle h)
 {
-  SshADTAvlNode k, n, m;
+  SshADTAvlNode k, m;
 
-  SSH_DEBUG(99, ("merge_detach [start]"));
+  SSH_DEBUG(9, ("in."));
 
-  n = (SshADTAvlNode)h;
-  SSH_ASSERT(n != NULL);
-
-  k = (SshADTAvlNode)avl_previous(c, (SshADTHandle)n);
-  m = (SshADTAvlNode)avl_next(c, (SshADTHandle)n);
+  k = (SshADTAvlNode)h;
+  m = k != NULL
+    ? (SshADTAvlNode)avl_next(c, (SshADTHandle)k)
+    : (SshADTAvlNode)enumerate_start(c);
 
   merge_ab(c, k, m);
 
-  SSH_DEBUG(99, ("merge_detach [done]"));
-
+  SSH_DEBUG(9, ("done."));
   return TRUE;
 }
 
+/* Before the actual merging takes place, a sequence of change can be
+   made.  Those changes are registered in the container_specifics
+   field and merged by ssh_adt_ranges_merge.  */
+
+static Boolean merge_map_attach_prepare(SshADTContainer c, SshADTHandle h)
+{
+  SshADTHandle *hp;
+  SshBufferStatus s;
+
+  SSH_DEBUG(9, ("in."));
+
+  if ((hp = ssh_malloc(sizeof(*hp))) == NULL)
+    {
+      SSH_DEBUG(0, ("out of memory!"));
+      return FALSE;
+    }
+  *hp = h;
+  s = ssh_buffer_append(RROOT(c)->merge_additions,
+                        (const char *)hp, sizeof(*hp));
+  ssh_free(hp);
+
+  SSH_DEBUG(9, ("out: %i.", s));
+  return (s == SSH_BUFFER_OK);
+}
+
+static Boolean merge_detach_prepare(SshADTContainer c, SshADTHandle h)
+{
+  SshADTHandle *hp;
+  SshBufferStatus s;
+
+  SSH_DEBUG(9, ("in."));
+
+  /* find the previous node and store it in merge_deletions.  */
+  if ((hp = ssh_malloc(sizeof(*hp))) == NULL)
+    {
+      SSH_DEBUG(0, ("out of memory!"));
+      return FALSE;
+    }
+  *hp = avl_previous(c, h);
+  s = ssh_buffer_append(RROOT(c)->merge_deletions,
+                        (const char *)hp, sizeof(*hp));
+
+  {
+    /* Remove h and previous(h) from both merge_additions and
+       merge_deletions if they are found there.  */
+
+    SshBuffer b;
+    SshADTHandle *hs;
+    int len, i;
+
+#   define GN                                                                 \
+    hs = (SshADTHandle *)ssh_buffer_ptr(b);                                   \
+    len = ssh_buffer_len(b) / sizeof(SshADTHandle);                           \
+    SSH_DEBUG(99, ("%p / %i.", hs, len));                                     \
+    for (i = 0; i < len; i++)                                                 \
+      {                                                                       \
+        SSH_DEBUG(99, ("%i.", i));                                            \
+        if (h == hs[i] || h == *hp)                                           \
+          {                                                                   \
+            hs[i] = hs[len-1];  /* (also works for i == len-1) */             \
+            ssh_buffer_consume_end(b, sizeof(SshADTHandle));                  \
+            len--;                                                            \
+          }                                                                   \
+      }
+
+    b = RROOT(c)->merge_additions; GN;
+    b = RROOT(c)->merge_deletions; GN;
+
+#   undef GN
+  }
+
+  ssh_free(hp);
+  SSH_DEBUG(9, ("out: %i.", s));
+  return (s == SSH_BUFFER_OK);
+}
+
+void ssh_adt_ranges_merge(SshADTContainer c)
+{
+  SshBuffer b;
+  SshADTHandle *hs;
+  int len, i;
+
+  SSH_DEBUG(9, ("in."));
+
+# define NG(op)                                                               \
+  hs = (SshADTHandle *)ssh_buffer_ptr(b);                                     \
+  len = ssh_buffer_len(b) / sizeof(SshADTHandle);                             \
+  for (i = 0; i < len; i++)                                                   \
+    op(c, hs[i]);                                                             \
+  ssh_buffer_clear(b);
+
+  b = RROOT(c)->merge_additions; NG(merge_map_attach);
+  b = RROOT(c)->merge_deletions; NG(merge_detach);
+
+# undef NG
+
+  SSH_ASSERT(ssh_buffer_len(RROOT(c)->merge_additions) == 0);
+  SSH_ASSERT(ssh_buffer_len(RROOT(c)->merge_deletions) == 0);
+
+  SSH_DEBUG(9, ("out."));
+}
+
 
 /************************************************** exported container types */
 
@@ -1039,16 +1186,16 @@ SshADTStaticData ssh_adt_avltree_static_data =
 {
   {
     /* $$METHODS(avltree) */
-    /* DO NOT EDIT THIS, edit METHODS.h and 
+    /* DO NOT EDIT THIS, edit METHODS.h and
        the method implementations above instead. */
-    container_init, /* container_init */
+    container_init_tree, /* container_init */
     avlclear, /* clear */
     destr, /* destr */
-    NULL, /* insert_at */
+    NULL_FNPTR, /* insert_at */
     avl_insert_to, /* insert_to */
-    NULL, /* alloc_n_at */
-    NULL, /* alloc_n_to */
-    NULL, /* put_n_at */
+    NULL_FNPTR, /* alloc_n_at */
+    NULL_FNPTR, /* alloc_n_to */
+    NULL_FNPTR, /* put_n_at */
     avl_put_to, /* put_n_to */
     get, /* get */
     avl_num_objects, /* num_objects */
@@ -1059,7 +1206,7 @@ SshADTStaticData ssh_adt_avltree_static_data =
     enumerate_start, /* enumerate_start */
     avl_next, /* enumerate_next */
     avl_geth_eq, /* get_handle_to_equal */
-    NULL, /* reallocate */
+    NULL_FNPTR, /* reallocate */
     avl_detach, /* detach */
     avl_delete, /* delet */
     avl_map_lookup, /* map_lookup */
@@ -1077,16 +1224,16 @@ SshADTStaticData ssh_adt_ranges_static_data =
 {
   {
     /* $$METHODS(ranges) */
-    /* DO NOT EDIT THIS, edit METHODS.h and 
+    /* DO NOT EDIT THIS, edit METHODS.h and
        the method implementations above instead. */
-    container_init, /* container_init */
+    container_init_ranges, /* container_init */
     avlclear, /* clear */
     destr, /* destr */
-    NULL, /* insert_at */
+    NULL_FNPTR, /* insert_at */
     avl_insert_to_ranges, /* insert_to */
-    NULL, /* alloc_n_at */
-    NULL, /* alloc_n_to */
-    NULL, /* put_n_at */
+    NULL_FNPTR, /* alloc_n_at */
+    NULL_FNPTR, /* alloc_n_to */
+    NULL_FNPTR, /* put_n_at */
     avl_put_to_ranges, /* put_n_to */
     get, /* get */
     avl_num_objects, /* num_objects */
@@ -1097,7 +1244,7 @@ SshADTStaticData ssh_adt_ranges_static_data =
     enumerate_start, /* enumerate_start */
     avl_next, /* enumerate_next */
     avl_geth_eq, /* get_handle_to_equal */
-    NULL, /* reallocate */
+    NULL_FNPTR, /* reallocate */
     avl_detach_range, /* detach */
     avl_delete, /* delet */
     avl_map_lookup, /* map_lookup */
@@ -1253,6 +1400,44 @@ static Boolean avl_consistent(SshADTAvlNode n)
 
 
 
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
