@@ -21,11 +21,6 @@ extern const SshADTContainerType ssh_adt_bag_type;
 
 #define SSH_ADT_BAG (ssh_adt_bag_type)
 
-/* Tell the bag object that the object at `object_with_changed_contents' has
-   been changed [and that its hash value should be recomputed]. */
-Boolean ssh_adt_bag_changed(SshADTContainer bag,
-                            SshADTHandle object_with_changed_contents);
-
 /* Use this instead of SshADTHeaderStruct if you want to save 8 bytes
    of memory per header.  */
 typedef struct {
