@@ -11,9 +11,8 @@
 
   */
 
-/* NOTE: `bag' shares implementation with `map'. It just does not use
-   the extra void pointer available for associating an external
-   pointer. */
-
 
 #include "sshadt_map_i.h"
+
+/* 'bag' shares implementation with 'map'.  It simply does not use the
+   extra void pointer for storing the map value. */
