@@ -28,10 +28,10 @@ void ssh_debug_output(int level, const char *file, unsigned int line,
                       const char *module, const char *function, char *message);
 Boolean ssh_debug_enabled(const char *module, int level);
 void ssh_debug_hexdump(size_t offset, const unsigned char *buf, size_t len);
-void ssh_generic_assert(int value, const char *expression,
-                        const char *file, unsigned int line,
-                        const char *module,
-                        const char *function, int type);
+int ssh_generic_assert(int value, const char *expression,
+                       const char *file, unsigned int line,
+                       const char *module,
+                       const char *function, int type);
 void ssh_debug_change_indentation(int change);
 
 /***********************************************************************
@@ -218,14 +218,26 @@ void ssh_debug_change_indentation(int change);
 #define SSH_NOTREACHED          _SSH_GEN_ASSERT(0,    4)
 #define SSH_ELSE_NOTREACHED     else SSH_NOTREACHED
 #else
-#define SSH_PRECOND(x)
-#define SSH_POSTCOND(x)
-#define SSH_ASSERT(x)
-#define SSH_INVARIANT(x)
-#define SSH_NOTREACHED
+#define SSH_PRECOND(x)          ((void) 0)
+#define SSH_POSTCOND(x)         ((void) 0)
+#define SSH_ASSERT(x)           ((void) 0)
+#define SSH_INVARIANT(x)        ((void) 0)
+#define SSH_NOTREACHED          ((void) 0)
 #define SSH_ELSE_NOTREACHED
 #endif
 
+#ifdef DEBUG_HEAVY
+#define SSH_HEAVY_PRECOND(expr)       _SSH_GEN_ASSERT(expr, 0)
+#define SSH_HEAVY_POSTCOND(expr)      _SSH_GEN_ASSERT(expr, 1)
+#define SSH_HEAVY_ASSERT(expr)        _SSH_GEN_ASSERT(expr, 2)
+#define SSH_HEAVY_INVARIANT(expr)     _SSH_GEN_ASSERT(expr, 3)
+#else
+#define SSH_HEAVY_PRECOND(x)          ((void) 0)
+#define SSH_HEAVY_POSTCOND(x)         ((void) 0)
+#define SSH_HEAVY_ASSERT(x)           ((void) 0)
+#define SSH_HEAVY_INVARIANT(x)        ((void) 0)
+#endif
+
 /* SSH_DEBUG is compiled in only if DEBUG_LIGHT is defined. */
 #ifdef DEBUG_LIGHT
 #define SSH_DEBUG(level, varcall) SSH_TRACE((level), varcall)
