@@ -4,7 +4,7 @@ sshmalloc.c
 
 Author: Tatu Ylonen <ylo@cs.hut.fi>
 
-Copyright (c) 1995-2001 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
+Copyright (c) 1995-2002 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
                    All rights reserved
 
 Created: Mon Mar 20 21:23:10 1995 ylo
@@ -15,7 +15,7 @@ failure (they call fatal if they encounter an error).
 */
 
 /*
- * $Id: sshmalloc.c,v 1.13.2.1 2002/04/10 00:10:09 sjl Exp $
+ * $Id: sshmalloc.c,v 1.15 2002/02/28 14:20:06 mtr Exp $
  * $Log: sshmalloc.c,v $ * 
  * 
  * 
@@ -180,6 +180,7 @@ failure (they call fatal if they encounter an error).
  * 
  * 
  * 
+ * 
  * $EndLog$
  */
 
@@ -191,18 +192,12 @@ failure (they call fatal if they encounter an error).
    library functions that are not marked thread safe from this file */
 #undef SSH_DEBUG_MODULE
 
-#ifdef VXWORKS
-#undef __GNUC__
-#endif /* VXWORKS */
-
-#ifdef malloc
-# undef malloc
-# undef calloc
-# undef realloc
-# undef free
-# undef memdup
-# undef strdup
-#endif
+#undef malloc
+#undef calloc
+#undef realloc
+#undef free
+#undef memdup
+#undef strdup
 
 #ifdef SSH_DEBUG_MALLOC
 #include "sshgetput.h"
@@ -231,7 +226,7 @@ failure (they call fatal if they encounter an error).
 /* Callback that is called if set when the malloc runs out of memory */
 Boolean (*ssh_malloc_failed_cb)(void);
 
-void *_ssh_malloc(size_t size)
+void *ssh_malloc(size_t size)
 {
   void *ptr;
 
@@ -279,7 +274,7 @@ void *_ssh_malloc(size_t size)
   return ptr;
 }
 
-void *_ssh_calloc(size_t nitems, size_t size)
+void *ssh_calloc(size_t nitems, size_t size)
 {
   void *ptr;
 
@@ -324,13 +319,13 @@ void *_ssh_calloc(size_t nitems, size_t size)
   return ptr;
 }
 
-void *_ssh_realloc(void *ptr,
+void *ssh_realloc(void *ptr,
                   size_t old_size, size_t new_size)
 {
   void *new_ptr = NULL;
 
   if (ptr == NULL)
-    return _ssh_malloc(new_size);
+    return ssh_malloc(new_size);
 
   if (new_size > XMALLOC_MAX_SIZE)
     return NULL;
@@ -409,7 +404,7 @@ void *_ssh_realloc(void *ptr,
   return new_ptr;
 }
 
-void _ssh_free(void *ptr)
+void ssh_free(void *ptr)
 {
 #ifdef SSH_DEBUG_MALLOC
   if (ptr != NULL)
@@ -448,7 +443,7 @@ void _ssh_free(void *ptr)
 #endif /* SSH_DEBUG_MALLOC */
 }
 
-void *_ssh_strdup(const void *p)
+void *ssh_strdup(const void *p)
 {
   const char *str;
   char *cp = NULL;
@@ -456,20 +451,20 @@ void *_ssh_strdup(const void *p)
   if (p)
     {
       str = (const char *)p;
-      if ((cp = _ssh_malloc(strlen(str) + 1)) != NULL)
+      if ((cp = ssh_malloc(strlen(str) + 1)) != NULL)
         strcpy(cp, str);
     }
   return (void *)cp;
 }
 
-void *_ssh_memdup(const void *p, size_t len)
+void *ssh_memdup(const void *p, size_t len)
 {
   const char *str = (const char *)p;
   char *cp = NULL;
 
   if (len < XMALLOC_MAX_SIZE)
     {
-      if ((cp = _ssh_malloc(len + 1)) != NULL)
+      if ((cp = ssh_malloc(len + 1)) != NULL)
         {
           memcpy(cp, str, (size_t)len);
           cp[len] = '\0';
@@ -477,60 +472,3 @@ void *_ssh_memdup(const void *p, size_t len)
     }
   return (void *)cp;
 }
-
-/* Now, provide weak aliases for the above routines under
-   platforms/compilers where we can perform such operations. Currently
-   this works under Sun's Workshop//Forte compilers and the GNU C
-   Compiler.
-
-   Important notice: Whenever adding a new platform/compiler
-   supporting weak linkage, edit sshmalloc.h so that the symbol
-   SSH_ALLOC_WEAK_SYMBOLS is defined.*/
-
-#if defined(__SUNPRO_C)
-#pragma weak ssh_malloc = _ssh_malloc
-#pragma weak ssh_calloc = _ssh_calloc
-#pragma weak ssh_realloc = _ssh_realloc
-#pragma weak ssh_free = _ssh_free
-#pragma weak ssh_strdup = _ssh_strdup
-#pragma weak ssh_memdup = _ssh_memdup
-#elif defined(HAVE_GCC_ATTRIBUTE_WEAK)
-void *ssh_malloc(size_t size)
-     __attribute__ ((weak, alias("_ssh_malloc")));
-void *ssh_realloc(void *ptr, size_t old_size, size_t new_size)
-     __attribute__((weak, alias("_ssh_realloc")));
-void *ssh_calloc(size_t nitems, size_t size)
-     __attribute__((weak, alias("_ssh_calloc")));
-void *ssh_strdup(const void *p)
-     __attribute__((weak, alias("_ssh_strdup")));
-void *ssh_memdup(const void *p, size_t len)
-     __attribute__((weak, alias("_ssh_memdup")));
-void ssh_free(void *ptr)
-     __attribute__((weak, alias("_ssh_free")));
-#else /* !__SUNPRO_C && !__GNUC__ */
-void *ssh_malloc(size_t size)
-{
-  return _ssh_malloc(size);
-}
-void *ssh_realloc(void *ptr, size_t old_size, size_t new_size)
-{
-  return _ssh_realloc(ptr, old_size, new_size);
-}
-void *ssh_calloc(size_t nitems, size_t size)
-{
-  return _ssh_calloc(nitems, size);
-}
-void *ssh_strdup(const void *p)
-{
-  return _ssh_strdup(p);
-}
-void *ssh_memdup(const void *p, size_t len)
-{
-  return _ssh_memdup(p, len);
-}
-void ssh_free(void *ptr)
-{
-  _ssh_free(ptr);
-}
-
-#endif
