@@ -15,7 +15,7 @@
 */
 
 /*
- * $Id: sshoperation.h,v 1.5 2001/01/19 12:56:01 napo Exp $
+ * $Id: sshoperation.h,v 1.9 2002/03/08 13:15:33 vlaurika Exp $
  * $Log: sshoperation.h,v $ * 
  * 
  * 
@@ -51,6 +51,25 @@
  * 
  * 
  * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -66,6 +85,7 @@ extern "C" {
    the asyncronous operation so the caller can store it, and call the
    ssh_operation_abort function later. */
 typedef struct SshOperationHandleRec *SshOperationHandle;
+typedef struct SshOperationHandleRec  SshOperationHandleStruct;
 
 /* Interface which is used by the functions which gets the
    SshOperationHandles from the other functions */
@@ -79,18 +99,40 @@ typedef struct SshOperationHandleRec *SshOperationHandle;
    associated with this asynchronous call. */
 void ssh_operation_abort(SshOperationHandle handle);
 
-/* Interface that is used by intermediae libraries passing the handle. */
+/* Interface that is used by intermediate libraries passing the handle. */
 
 /* This type of callbacks are used with
    ssh_operation_attach_destructor. The callback is called whenewer
    the handle is unregistered, or if the
-   ssh_operation_attach_destructor is called with a NULL handle. */
-typedef void (*SshOperationDestructorCB)(void *context);
+   ssh_operation_attach_destructor is called with a NULL handle. The
+   aborted argument is set to TRUE when the call was aborted and FALSE
+   otherwise. */
+typedef void (*SshOperationDestructorCB)(Boolean aborted,
+                                         void *context);
+
+/* The destructor record that is used to save attached destructors for
+   the handle. Do not access any of the fields of this record. The
+   contents of the structure are visible only because the
+   ssh_operation_attach_destructor_no_alloc function uses this
+   structure. */
+typedef struct SshOperationDestructorRec
+{
+  /* The link to  the next destructor. */
+  struct SshOperationDestructorRec *next;
+  /* Callback and the context for the callback. */
+  SshOperationDestructorCB destructor_cb;
+  void *context;
+  Boolean dynamic_mem;
+} *SshOperationDestructor, SshOperationDestructorStruct;
+
 
 /* Registeres a callback to be called when a handle is unregistered or
-   aborted. If this function is called with a NULL operation handle,
-   the destructor_cb is called inside the call. The application does
-   not need to check whether the application returned NULL or a valid
+   aborted. Returs TRUE if attaching is successfull, or FALSE when failed
+   for memory allocation reasons.
+
+   If this function is called with a NULL operation handle, the
+   destructor_cb is called inside the call. The application does not
+   need to check whether the application returned NULL or a valid
    handle. The destructor will be called regardless.
 
    Attaching destructors is usefull for modules that pass handles from
@@ -109,9 +151,17 @@ typedef void (*SshOperationDestructorCB)(void *context);
    the asynchronous function should call the completion callback
    first, and then ssh_operation_unregister which will call the
    attached destructors. */
-void ssh_operation_attach_destructor(SshOperationHandle handle,
-                                     SshOperationDestructorCB destructor_cb,
-                                     void *context);
+Boolean ssh_operation_attach_destructor(SshOperationHandle handle,
+                                        SshOperationDestructorCB callback,
+                                        void *context);
+
+/* Attach destructor and provide the memory for the operation. This
+   function can not fail. Caller must provide the pointer to the
+   allocated structure in dstr_ptr. */
+void ssh_operation_attach_destructor_no_alloc(SshOperationDestructor dest,
+                                              SshOperationHandle handle,
+                                              SshOperationDestructorCB cb,
+                                              void *context);
 
 /* Interface which is used inside the functions that implement the
    asyncronous operations. */
@@ -124,14 +174,38 @@ void ssh_operation_attach_destructor(SshOperationHandle handle,
    immediately, or postpone their freeing for the later time. */
 typedef void (*SshOperationAbortCB)(void *operation_context);
 
+/* The implementation of SshOperationHandle. Do not access any of the
+   fields of this record. The contents of the structure are visible
+   only because the ssh_operation_register_no_alloc function uses this
+   structure. */
+struct SshOperationHandleRec {
+  SshOperationAbortCB abort_cb;
+  void *operation_context;
+  SshOperationDestructor destructor_list;
+  Boolean in_abort;
+  Boolean dynamic_mem;
+};
+
 /* Register started asynchronous call and corresponding abort
    callback. This call is called by the function that starts the real
    operation, and the handle must be returned to the caller of the
    function, so it can then abort the operation using the
-   ssh_operation_abort. */
+   ssh_operation_abort. This call returns NULL if memory allocation
+   fails. */
 SshOperationHandle ssh_operation_register(SshOperationAbortCB abort_cb,
                                           void *operation_context);
 
+/* Register started asynchronous call and corresponding abort callback
+   with no memory allocation. The caller must provide the pointer to
+   the allocated handle (which typically is stored inside the
+   operation_context). This call is called by the function that
+   starts the real operation, and the handle must be returned to the
+   caller of the function, so it can then abort the operation using
+   the ssh_operation_abort. This function can not fail. */
+void ssh_operation_register_no_alloc(SshOperationHandle handle,
+                                     SshOperationAbortCB abort_cb,
+                                     void *operation_context);
+
 /* Unregister the handle. The abort callback will not be called, and
    the handle is invalid after this (i.e any other part of the code
    must not call ssh_operation_abort using the handle). This should be
