@@ -5,7 +5,7 @@
   Author: Antti Huima <huima@ssh.fi>
           Markku Rossi <mtr@ssh.fi>
 
-  Copyright (c) 1999-2001 SSH Communications Security, Finland
+  Copyright (c) 1999-2002 SSH Communications Security, Finland
   All rights reserved.
 
   Created Thu Aug 26 14:27:14 1999.
@@ -110,13 +110,22 @@ void ssh_fsm_destroy(SshFSM fsm)
   ssh_register_timeout(0L, 0L, destroy_callback, (void *)fsm);
 }
 
-void ssh_fsm_uninit(SshFSM fsm)
+static void uninit_callback(void *ctx)
 {
 #ifdef DEBUG_LIGHT
+  SshFSM fsm = (SshFSM)ctx;
+
   if (fsm->num_threads > 0)
     ssh_fatal("Tried to destroy a FSM that has %d thread(s) left",
               fsm->num_threads);
 #endif /* DEBUG_LIGHT */
+
+  SSH_DEBUG(8, ("FSM context uninitialized"));
+}
+
+void ssh_fsm_uninit(SshFSM fsm)
+{
+  ssh_register_timeout(0L, 0L, uninit_callback, (void *)fsm);
 }
 
 void ssh_fsm_register_debug_names(SshFSM fsm, SshFSMStateDebug states,
@@ -156,7 +165,7 @@ static void delete_thread(SshFSMThread thread)
     }
 
   if (thread->destructor)
-    (*thread->destructor)(thread->context_data);
+    (*thread->destructor)(thread->fsm, thread->context_data);
 
   if (thread->flags & SSH_FSM_DYNAMIC_THREAD)
     ssh_free(thread);
@@ -261,7 +270,7 @@ static void scheduler(SshFSM fsm)
 
               ring_remove(&(fsm->waiting_message_handler), msg_thr);
 
-              SSH_ASSERT(msg_thr->message_handler);
+              SSH_ASSERT(msg_thr->message_handler != NULL_FNPTR);
               SSH_ASSERT(msg_thr->flags & SSH_FSM_IN_MESSAGE_QUEUE);
 
               SSH_DEBUG(8, ("Delivering the message %u to thread `%s'",
@@ -569,7 +578,6 @@ SshFSMCondition ssh_fsm_condition_create(SshFSM fsm)
 void ssh_fsm_condition_init(SshFSM fsm, SshFSMCondition condition)
 {
   memset(condition, 0, sizeof(*condition));
-  condition->fsm = fsm;
 }
 
 void ssh_fsm_condition_destroy(SshFSMCondition condition)
@@ -650,7 +658,7 @@ void ssh_fsm_throw(SshFSMThread thread,
   SSH_ASSERT(thread->fsm->flags & SSH_FSM_IN_SCHEDULER);
   SSH_ASSERT(thread != recipient);
 
-  if (recipient->message_handler == NULL)
+  if (recipient->message_handler == NULL_FNPTR)
     /* Nothing to do. */
     return;
 
