@@ -960,6 +960,44 @@ while (0)
 #define SSH_AHH_OFS_SPI         4
 #define SSH_AHH_OFS_SEQ         8
 
+/******************************* SCTP definitions ***************************/
+
+#define SSH_SCTPH_HDRLEN        12
+
+#define SSH_SCTPH_OFS_SRCPORT 0
+#define SSH_SCTPH_OFS_DSTPORT 2
+#define SSH_SCTPH_OFS_VERIFTAG 4
+#define SSH_SCTPH_OFS_CHECKSUM 8
+
+#define SSH_SCTPH_SRCPORT(ucp) SSH_GET_16BIT((ucp) + SSH_SCTPH_OFS_SRCPORT)
+#define SSH_SCTPH_DSTPORT(ucp) SSH_GET_16BIT((ucp) + SSH_SCTPH_OFS_DSTPORT)
+#define SSH_SCTPH_VERIFTAG(ucp) SSH_GET_32BIT((ucp) + SSH_SCTPH_OFS_VERIFTAG)
+#define SSH_SCTPH_CHECKSUM(ucp) SSH_GET_32BIT((ucp) + SSH_SCTPH_OFS_CHECKSUM)
+
+#define SSH_SCTPH_SET_SRCPORT(ucp, v) \
+        SSH_PUT_16BIT((ucp) + SSH_SCTPH_OFS_SRCPORT, (v))
+#define SSH_SCTPH_SET_DSTPORT(ucp, v) \
+        SSH_PUT_16BIT((ucp) + SSH_SCTPH_OFS_DSTPORT, (v))
+#define SSH_SCTPH_SET_VERIFTAG(ucp, v) \
+        SSH_PUT_32BIT((ucp) + SSH_SCTPH_OFS_VERIFTAG, (v))
+#define SSH_SCTPH_SET_CHECKSUM(ucp, v) \
+        SSH_PUT_32BIT((ucp) + SSH_SCTPH_OFS_CHECKSUM, (v))
+
+/********************************* Services *********************************/
+
+/* Looks up the service (port number) by name and protocol.
+   `protocol' must be either "tcp" or "udp".  Returns -1 if the
+   service could not be found. */
+int ssh_inet_get_port_by_service(const char *name, const char *proto);
+
+/* Looks up the name of the service based on port number and protocol.
+   `protocol' must be either "tcp" or "udp".  The name is stored in
+   the given buffer; is the service is not found, the port number is
+   stored instead (without the protocol specification).  The name will
+   be truncated if it is too long. */
+void ssh_inet_get_service_by_port(unsigned int port, const char *protocol,
+                                  char *buf, size_t buflen);
+
 /***************************** Helper functions *****************************/
 
 /* Sets all rightmost bits after keeping `keep_bits' bits on the left to
@@ -1010,6 +1048,13 @@ int ssh_ipmask_render(char *buf, int buf_size, int precision, void *datum);
  * char[6] array. */
 int ssh_etheraddr_render(char *buf, int buf_size, int precision, void *datum);
 
+/* Compares two port number addresses, and returns <0 if port1 is
+   smaller, 0 if they denote the same number (though possibly written
+   differently), and >0 if port2 is smaller.  The result is zero if
+   either address is invalid. */
+int ssh_inet_port_number_compare(const char *port1, const char *port2,
+                                 const char *proto);
+
 /*************************** Internal definitions ***************************/
 
 /* Some prototypes for internal functions. */
