@@ -1,10 +1,10 @@
 /*
 
   cstack.c
-  
+
   Author: Mika Kojo <mkojo@ssh.fi>
 
-  Copyright (C) 1997 SSH Communications Security Oy, Espoo, Finland
+  Copyright (C) 1997, 2001 SSH Communications Security Oy, Espoo, Finland
   All rights reserved.
 
   Created: Mon May 26 13:33:44 1997 [mkojo]
@@ -14,12 +14,55 @@
   someone can figure out better names I'd be glad to know.
 
   Moved to sshutil where they make better sense.
-  
+
   */
 
 /*
- * $Id: sshcstack.c,v 1.3 2000/06/14 00:43:40 kivinen Exp $
- * $Log: sshcstack.c,v $
+ * $Id: sshcstack.c,v 1.5 2001/07/25 08:11:33 tmo Exp $
+ * $Log: sshcstack.c,v $ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -29,6 +72,7 @@
 void ssh_cstack_push(SshCStack *head, void *thisp)
 {
   SshCStack stack = thisp;
+
   stack->next = *head;
   *head = stack;
 }
@@ -37,6 +81,7 @@ unsigned int ssh_cstack_count(SshCStack *head, SshCStackToken token)
 {
   SshCStack temp;
   unsigned int count;
+
   for (temp = *head, count = 0; temp; temp = temp->next)
     if (temp->token == token)
       count++;
@@ -104,7 +149,7 @@ void *ssh_cstack_free(void *head)
 
 /* This is the implementation for the simpler stack system. Useful
    also occasionally. */
-   
+
 void *ssh_dstack_pop(SshDStack *stack)
 {
   void *data;
@@ -112,12 +157,12 @@ void *ssh_dstack_pop(SshDStack *stack)
 
   if (stack == NULL)
     return NULL;
-  
+
   if (*stack != NULL)
     {
       data = (*stack)->data;
       next = (*stack)->next;
-      ssh_xfree(*stack);
+      ssh_free(*stack);
       *stack = next;
       return data;
     }
@@ -127,20 +172,23 @@ void *ssh_dstack_pop(SshDStack *stack)
 void ssh_dstack_push(SshDStack *stack, void *data)
 {
   SshDStack node;
+
   if (stack == NULL)
     return;
-  
-  node = ssh_xmalloc(sizeof(*node));
-  node->data = data;
-  node->next = *stack;
-  *stack = node;
+
+  if ((node = ssh_malloc(sizeof(*node))) != NULL)
+    {
+      node->data = data;
+      node->next = *stack;
+      *stack = node;
+    }
 }
 
 void *ssh_dstack_current(SshDStack *stack)
 {
   if (stack == NULL)
     return NULL;
-  
+
   if (*stack)
     return (*stack)->data;
   return NULL;
