@@ -5,7 +5,7 @@ sshpacketimpl.c
 Author: Tatu Ylonen <ylo@ssh.fi>
         Tero Kivinen <kivinen@ssh.fi>
 
-Copyright (c) 1998, 2001 SSH Communications Security, Finland
+Copyright (c) 1998, 2001, 2002 SSH Communications Security, Finland
                    All rights reserved
 
 This module implements a SshStream that sends/receives packets.  This has
@@ -226,6 +226,8 @@ normal:
       if (size - offset < 4)
         goto partial;  /* Need partial packet processing. */
       payload_len = SSH_GET_32BIT(buf + offset);
+      if (payload_len < 1)
+        goto partial;
 
       if (size - offset < 4 + payload_len)
         goto partial;  /* Need partial packet processing. */
@@ -269,6 +271,13 @@ partial:
   /* Get the length of the packet. */
   ucp = ssh_buffer_ptr(&up->incoming);
   payload_len = SSH_GET_32BIT(ucp);
+  if (payload_len < 1)
+    {
+      /* Received an invalid packet with length = 0, even though we should
+         always have at least the packet type. */
+      (*up->received_eof)(up->context);
+      return 0;
+    }
 
   /* Add remaining data in the packet to the buffer. */
   len = 4 + payload_len - ssh_buffer_len(&up->incoming);
@@ -433,7 +442,7 @@ SshStream ssh_packet_impl_create(SshPacketReceiveProc received_packet,
   up->destroy = destroy;
   up->context = context;
 
-  up->up_callback = NULL;
+  up->up_callback = NULL_FNPTR;
   up->up_context = NULL;
 
   /* Cause the send callback to be called if non-NULL.  Note that it isn't
