@@ -415,6 +415,9 @@ ssh_stream_filter_accept(SshStreamFilter sf,
                          SshStreamFilterDataDir dir,
                          int op)
 {
+  SSH_DEBUG(10, ("Entered with op %d in direction %s", op,
+                 dir == SSH_STREAM_FILTER_INBOUND ? "inbound" : "outbound"));
+  
   if (dir == SSH_STREAM_FILTER_INBOUND)
     {
       /* First handle the case that we accepted a non-zero number of bytes. */
@@ -467,6 +470,13 @@ ssh_stream_filter_accept(SshStreamFilter sf,
       /* First handle the case that we accepted a non-zero number of bytes. */
       if (op > 0)
         {
+          if (op > (ssh_buffer_len(&sf->from_buffer) -
+                    sf->from_accepted_bytes))
+            {
+              ssh_fatal("ssh_stream_filter_accept: trying to accept more "
+                        "bytes (%d) than there are unaccepted bytes in buffer",
+                        op);
+            }
           /* We accepted some bytes. */
           sf->from_accepted_bytes += op;
         }
@@ -491,7 +501,7 @@ ssh_stream_filter_accept(SshStreamFilter sf,
              equivalent to accepting zero bytes. */
           if (sf->from_accepted_bytes == 0 &&
               ssh_buffer_len(&sf->from_buffer) >= sf->max_buffer_size)
-            ssh_fatal("ssh_stream_filter_try_read: SSH_FILTER_HOLD "
+            ssh_fatal("ssh_stream_filter_accept: SSH_FILTER_HOLD "
                       "returned, but buffer already full.");
           
           /* Must wake up reads if we got EOF. */
@@ -510,7 +520,7 @@ ssh_stream_filter_accept(SshStreamFilter sf,
           break;
           
         default:
-          ssh_fatal("ssh_stream_filter_try_read: filter returned bad op %d",
+          ssh_fatal("ssh_stream_filter_accept: filter returned bad op %d",
                     op);
         }
     }
@@ -611,7 +621,7 @@ ssh_stream_filter_destroy(void *context)
   /* Cancel any callbacks to the stream callback. */
   ssh_cancel_timeouts(ssh_stream_filter_read_upcall, (void *)sf);
   ssh_cancel_timeouts(ssh_stream_filter_write_upcall, (void *)sf);
-  sf->callback = NULL;
+  sf->callback = NULL_FNPTR;
 
   /* If we have no more bytes to write, destroy immediately.  Otherwise,
      destroy when the buffer has been drained. */
@@ -826,7 +836,7 @@ ssh_stream_filter_create(SshStream stream,
   ssh_buffer_init(&sf->to_buffer);
   sf->read_blocked = TRUE;
   sf->write_blocked = TRUE;
-  sf->callback = NULL;
+  sf->callback = NULL_FNPTR;
   sf->callback_context = NULL;
 
   /* Set the original stream's callback to our callback. */
