@@ -13,8 +13,74 @@
 */
 
 /*
- * $Id: sshstream.h,v 1.4 2000/06/15 13:43:26 tas Exp $
- * $Log: sshstream.h,v $
+ * $Id: sshstream.h,v 1.8 2001/04/24 18:29:56 fis Exp $
+ * $Log: sshstream.h,v $ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -92,7 +158,7 @@ typedef struct {
    instead, applications call stream type specific creation functions that
    will eventually call this. */
 SshStream
-ssh_stream_create(const SshStreamMethodsStruct *methods,
+ssh_stream_create(SshStreamMethods methods,
                   void *context);
 
 /* Reads at most `size' bytes to the buffer `buffer'.  Returns 0 if
@@ -137,8 +203,7 @@ ssh_stream_destroy(SshStream stream);
 /* Returns the methods table for the stream.  This function is primarily
    used by various stream implementations to determine whether a particular
    stream is of the appropriate type. */
-const SshStreamMethodsStruct
-*ssh_stream_get_methods(SshStream stream);
+SshStreamMethods ssh_stream_get_methods(SshStream stream);
 
 /* Returns the method context of the stream.  This function is intended
    for use by stream implementations only. */
