@@ -12,7 +12,7 @@ Created: Thu Aug 29 23:22:29 1996 ylo
 */
 
 /*
- * $Id: sshstreampair.c,v 1.4 2001/04/24 18:29:56 fis Exp $
+ * $Id: sshstreampair.c,v 1.6 2002/02/22 10:39:48 mstenber Exp $
  * $Log: sshstreampair.c,v $ * 
  * 
  * 
@@ -55,6 +55,12 @@ Created: Thu Aug 29 23:22:29 1996 ylo
  * 
  * 
  * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -96,11 +102,11 @@ typedef struct SshStreamHalfRec
 
   /* True if this half of the stream has been destroyed. */
   Boolean destroyed;
-  
+
   /* Callback to be called when we are signalled. */
   SshStreamCallback callback;
   void *context;
-  
+
   /* Pointer to the other side of the pair. */
   struct SshStreamHalfRec *other;
 
@@ -126,7 +132,7 @@ typedef struct SshStreamPairRec
 void ssh_stream_pair_can_output(void *context)
 {
   SshStreamHalf me = (SshStreamHalf)context;
-  
+
   /* Signal that we can take more data. */
   if (me->callback)
     (*me->callback)(SSH_STREAM_CAN_OUTPUT, me->context);
@@ -166,7 +172,7 @@ int ssh_stream_pair_read(void *context, unsigned char *buf, size_t size)
   /* Return EOF if appropriate. */
   if (bytes == 0 && me->eof_received)
     return 0;  /* An EOF has been received. */
-  
+
   /* Mark that read was successful. */
   me->read_has_failed = FALSE;
 
@@ -236,7 +242,7 @@ int ssh_stream_pair_write(void *context, const unsigned char *buf, size_t size)
         len = sizeof(me->other->buf) - offset; /* Ring end limit */
       if (len > sizeof(me->other->buf) - me->other->inbuf)
         len = sizeof(me->other->buf) - me->other->inbuf;
-      
+
       /* Copy some bytes into the buffer. */
       memcpy(me->other->buf + offset, buf + bytes, len);
 
@@ -303,7 +309,7 @@ void ssh_stream_pair_destroy(void *context)
   SshStreamHalf me = (SshStreamHalf)context;
   SshStreamPair pair;
 
-  me->callback = NULL;
+  me->callback = NULL_FNPTR;
   me->eof_received = TRUE;
   me->destroyed = TRUE;
   if (!me->other->eof_received)
@@ -319,13 +325,6 @@ void ssh_stream_pair_destroy(void *context)
       memset(pair, 'F', sizeof(*pair));
       ssh_xfree(pair);
     }
-  else
-    {
-      /* Wake up the other side to notify it of eof. */
-      SSH_ASSERT(!me->other->destroyed);
-      ssh_register_timeout(0L, 0L, ssh_stream_pair_input_available,
-                           (void *)me->other);
-    }
 }
 
 /* Methods table for stream pairs. */
