@@ -14,7 +14,7 @@
   */
 
 /*
- * $Id: sshbase64.c,v 1.5 2001/10/25 14:58:23 ttsalo Exp $
+ * $Id: sshbase64.c,v 1.6 2001/11/22 08:06:26 tmo Exp $
  * $Log: sshbase64.c,v $ * 
  * 
  * 
@@ -93,6 +93,10 @@
  * 
  * 
  * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -292,11 +296,12 @@ ssh_base64_remove_headers(const unsigned char *str,
                           size_t *start_ret,
                           size_t *end_ret)
 {
-  size_t i, end, start, header, inside, skip;
-  
+  size_t i, end, start, header, inside, skip, bol;
+
   /* Remove all before and after headers. */
-  for (i = 0, skip = 0, end = 0, start = 0, header = 0, inside = 0;
-       i < len; i++)
+  for (i = 0, skip = 0, end = 0, start = 0, header = 0, inside = 0, bol = 1;
+       i < len;
+       i++)
     {
       switch (str[i])
         {
@@ -304,14 +309,19 @@ ssh_base64_remove_headers(const unsigned char *str,
           if (skip)
             break;
 
-          if (inside)
-            end = i;
-          header = 1;
-          inside ^= 1;
-          skip = 1;
+          if (bol)
+            {
+              if (inside)
+                end = i;
+              header = 1;
+              inside ^= 1;
+              skip = 1;
+              bol = 0;
+            }
           break;
         case '\n':
         case '\r':
+          bol = 1;
           if (header)
             {
               header = 0;
@@ -320,11 +330,16 @@ ssh_base64_remove_headers(const unsigned char *str,
             }
           skip = 0;
           break;
+        case ' ':
+        case '\t':
+          break;
+
         default:
+          bol = 0;
           break;
         }
     }
-  
+
   if (end == 0 && start == 0)
     {
       start = 0;
@@ -339,6 +354,6 @@ ssh_base64_remove_headers(const unsigned char *str,
 
   *start_ret = start;
   *end_ret = end;
-  
+
   return TRUE;
 }
