@@ -12,7 +12,7 @@
 */
 
 /*
- * $Id: sshmiscstring.h,v 1.9 2001/09/04 07:11:01 vsuontam Exp $
+ * $Id: sshmiscstring.h,v 1.10 2002/01/30 09:10:34 vsuontam Exp $
  * $Log: sshmiscstring.h,v $ * 
  * 
  * 
@@ -63,6 +63,13 @@
  * 
  * 
  * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
+ * 
  * $EndLog$
  */
 
@@ -109,15 +116,19 @@ char *ssh_format_number(char *buffer, size_t len, SshUInt64 number, int scale);
  */
 char *ssh_format_time(char *buffer, size_t len, SshTime t);
 
-/* Get data from a string. Component identifies the data which to get. 
- * The source string is assumed to be in format 
- * "component1(component1_data), component2(component2_data)". 
- * The function handles parentheses correctly inside the component data.
+/* Get mallocated data from a string. Component identifies which part
+ * of data to get.  The source string is assumed to be in format
+ * "component1(component1_data), component2(component2_data)".  The
+ * function handles parentheses correctly inside the component data.
+ *
+ * Occurance identifies which occurance of the data to get, 0 giving
+ * the first occurance. 
  *
  * Returns NULL, if the component is not found in the string and an empty 
- * string, if the component is empty. 
- */
-char *ssh_str_get_component_data(const char *source, const char *component);
+ * string, if the component is empty.  */
+char *ssh_get_component_data_from_string(const char *source,
+                                         const char *component,
+                                         SshUInt32 occurance);
 
 
 /* Free an array of strings. The strings of the array are freed individually
