@@ -107,13 +107,20 @@ static Boolean ssh_url_parse_internal(const char *url, char **scheme,
           while (*p && (isxdigit((int)*p) || *p == '.' || *p == ':'))
             p++;
           if (*p == ']')
-            p++;
+            {
+              start++;
+              if (host != NULL)
+                *host = ssh_xmemdup(start, p - start);
+              p++;
+            }
           else
             {
               /* Invalid IPV6 ip address, assume it to be host name */
               p = start;
               while (*p && *p != ':' && *p != '/')
                 p++;
+              if (host != NULL)
+                *host = ssh_xmemdup(start, p - start);
             }
         }
       else
@@ -121,10 +128,10 @@ static Boolean ssh_url_parse_internal(const char *url, char **scheme,
           /* Check for host name */
           while (*p && *p != ':' && *p != '/')
             p++;
+          if (host != NULL)
+            *host = ssh_xmemdup(start, p - start);
         }
 
-      if (host != NULL)
-        *host = ssh_xmemdup(start, p - start);
       start = p;
 
       if (*p == ':')
@@ -273,13 +280,20 @@ static Boolean ssh_url_parse_internal_mod(const char *url, char **scheme,
           while (*p && (isxdigit((int)*p) || *p == '.' || *p == ':'))
             p++;
           if (*p == ']')
-            p++;
+            {
+              start++;
+              if (host != NULL)
+                *host = ssh_xmemdup(start, p - start);
+              p++;
+            }
           else
             {
               /* Invalid IPV6 ip address, assume it to be host name */
               p = start;
               while (*p && *p != ':' && *p != '/')
                 p++;
+              if (host != NULL)
+                *host = ssh_xmemdup(start, p - start);
             }
         }
       else
@@ -287,10 +301,10 @@ static Boolean ssh_url_parse_internal_mod(const char *url, char **scheme,
           /* Check for host name */
           while (*p && *p != ':' && *p != '/' && *p != '?')
             p++;
+          if (host != NULL)
+            *host = ssh_xmemdup(start, p - start);
         }
 
-      if (host != NULL)
-        *host = ssh_xmemdup(start, p - start);
       start = p;
 
       if (*p == ':')
@@ -660,7 +674,7 @@ Boolean ssh_url_decode_bin(char *url, size_t len,
 char *ssh_url_encode(const char *s)
 {
 
-#define URL_ENCODE_SAFE_CHARS "$-_.+!*'()," /* RFC1738, 2.2 */
+#define URL_ENCODE_SAFE_CHARS "$-_.!*'()," /* RFC1738, 2.2 */
   char *r, *ret;
 
   ret = ssh_xmalloc(3 * strlen(s) + 2);
