@@ -4,7 +4,7 @@
 
   Author: Santeri Paavolainen <santtu@ssh.com>
 
-  Copyright (c) 2001 SSH Communications Security, Finland
+  Copyright (c) 2001-2002 SSH Communications Security, Finland
   All rights reserved
 
   Threaded message box interface implementation. This implementation
@@ -39,6 +39,10 @@
    since the non-thread one is simpler and of course -- doesn't have
    threads. Doh. */
 
+#ifdef WIN32
+#undef HAVE_THREADS
+#endif /* WIN32 */
+
 #ifdef HAVE_THREADS
 
 typedef struct SshThreadedMboxMsgRec {
@@ -528,7 +532,7 @@ SshThreadedMbox ssh_threaded_mbox_create(SshInt32 max_threads)
   /* XXX: notreacheds */
 
 #ifdef WIN32
-  SSH_NOTREACHED;
+  ssh_warning("ssh_threaded_mbox_create() not implemented for WIN32");
 #else /* !WIN32 */
   {
     int fds[2];
